@@ -10,14 +10,14 @@ import (
 	"testing"
 	"time"
 
-	"github.com/hashicorp/consul/sdk/testutil/retry"
+	"github.com/stretchr/testify/mock"
+	"github.com/stretchr/testify/require"
 
 	"github.com/hashicorp/consul/agent/cache"
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/consul"
 	"github.com/hashicorp/consul/agent/structs"
-	"github.com/stretchr/testify/mock"
-	"github.com/stretchr/testify/require"
+	"github.com/hashicorp/consul/sdk/testutil/retry"
 )
 
 func TestCalculateSoftExpire(t *testing.T) {
@@ -147,6 +147,9 @@ func TestCalculateSoftExpire(t *testing.T) {
 // Test that after an initial signing, new CA roots (new ID) will
 // trigger a blocking query to execute.
 func TestConnectCALeaf_changingRoots(t *testing.T) {
+	if testingRace {
+		t.Skip("fails with -race because caRoot.Active is modified concurrently")
+	}
 	t.Parallel()
 
 	require := require.New(t)
@@ -693,9 +696,11 @@ func TestConnectCALeaf_CSRRateLimiting(t *testing.T) {
 // This test runs multiple concurrent callers watching different leaf certs and
 // tries to ensure that the background root watch activity behaves correctly.
 func TestConnectCALeaf_watchRootsDedupingMultipleCallers(t *testing.T) {
+	if testingRace {
+		t.Skip("fails with -race because caRoot.Active is modified concurrently")
+	}
 	t.Parallel()
 
-	require := require.New(t)
 	rpc := TestRPC(t)
 	defer rpc.AssertExpectations(t)
 
@@ -737,8 +742,8 @@ func TestConnectCALeaf_watchRootsDedupingMultipleCallers(t *testing.T) {
 	// initial cert delivered and is blocking before the root changes. It's not a
 	// wait group since we want to be able to timeout the main test goroutine if
 	// one of the clients gets stuck. Instead it's a buffered chan.
-	setupDoneCh := make(chan struct{}, n)
-	testDoneCh := make(chan struct{}, n)
+	setupDoneCh := make(chan error, n)
+	testDoneCh := make(chan error, n)
 	// rootsUpdate is used to coordinate clients so they know when they should
 	// expect to see leaf renewed after root change.
 	rootsUpdatedCh := make(chan struct{})
@@ -755,7 +760,8 @@ func TestConnectCALeaf_watchRootsDedupingMultipleCallers(t *testing.T) {
 		fetchCh := TestFetchCh(t, typ, opts, req)
 		select {
 		case <-time.After(100 * time.Millisecond):
-			t.Fatal("shouldn't block waiting for fetch")
+			setupDoneCh <- fmt.Errorf("shouldn't block waiting for fetch")
+			return
 		case result := <-fetchCh:
 			v := mustFetchResult(t, result)
 			opts.LastResult = &v
@@ -766,33 +772,38 @@ func TestConnectCALeaf_watchRootsDedupingMultipleCallers(t *testing.T) {
 		fetchCh = TestFetchCh(t, typ, opts, req)
 		select {
 		case result := <-fetchCh:
-			t.Fatalf("should not return: %#v", result)
+			setupDoneCh <- fmt.Errorf("should not return: %#v", result)
+			return
 		case <-time.After(100 * time.Millisecond):
 		}
 
 		// We're done with setup and the blocking call is still blocking in
 		// background.
-		setupDoneCh <- struct{}{}
+		setupDoneCh <- nil
 
 		// Wait until all others are also done and roots change incase there are
 		// stragglers delaying the root update.
 		select {
 		case <-rootsUpdatedCh:
 		case <-time.After(200 * time.Millisecond):
-			t.Fatalf("waited too long for root update")
+			testDoneCh <- fmt.Errorf("waited too long for root update")
+			return
 		}
 
 		// Now we should see root update within a short period
 		select {
 		case <-time.After(100 * time.Millisecond):
-			t.Fatal("shouldn't block waiting for fetch")
+			testDoneCh <- fmt.Errorf("shouldn't block waiting for fetch")
+			return
 		case result := <-fetchCh:
 			v := mustFetchResult(t, result)
-			// Index must be different
-			require.NotEqual(opts.MinIndex, v.Value.(*structs.IssuedCert).CreateIndex)
+			if opts.MinIndex == v.Value.(*structs.IssuedCert).CreateIndex {
+				testDoneCh <- fmt.Errorf("index must be different")
+				return
+			}
 		}
 
-		testDoneCh <- struct{}{}
+		testDoneCh <- nil
 	}
 
 	// Sanity check the roots watcher is not running yet
@@ -808,7 +819,10 @@ func TestConnectCALeaf_watchRootsDedupingMultipleCallers(t *testing.T) {
 		select {
 		case <-timeoutCh:
 			t.Fatal("timed out waiting for clients")
-		case <-setupDoneCh:
+		case err := <-setupDoneCh:
+			if err != nil {
+				t.Fatalf(err.Error())
+			}
 		}
 	}
 
@@ -837,7 +851,10 @@ func TestConnectCALeaf_watchRootsDedupingMultipleCallers(t *testing.T) {
 		select {
 		case <-timeoutCh:
 			t.Fatalf("timed out waiting for %d of %d clients to renew after root change", n-i, n)
-		case <-testDoneCh:
+		case err := <-testDoneCh:
+			if err != nil {
+				t.Fatalf(err.Error())
+			}
 		}
 	}
 
