@@ -331,23 +331,25 @@ func (c *CheckTTL) SetStatus(status, output string) string {
 // or if the request returns an error
 // Supports failures_before_critical and success_before_passing.
 type CheckHTTP struct {
-	CheckID         structs.CheckID
-	ServiceID       structs.ServiceID
-	HTTP            string
-	Header          map[string][]string
-	Method          string
-	Body            string
-	Interval        time.Duration
-	Timeout         time.Duration
-	Logger          hclog.Logger
-	TLSClientConfig *tls.Config
-	OutputMaxSize   int
-	StatusHandler   *StatusHandler
+	CheckID          structs.CheckID
+	ServiceID        structs.ServiceID
+	HTTP             string
+	Header           map[string][]string
+	Method           string
+	Body             string
+	Interval         time.Duration
+	Timeout          time.Duration
+	Logger           hclog.Logger
+	TLSClientConfig  *tls.Config
+	OutputMaxSize    int
+	StatusHandler    *StatusHandler
+	DisableRedirects bool
 
 	httpClient *http.Client
 	stop       bool
 	stopCh     chan struct{}
 	stopLock   sync.Mutex
+	stopWg     sync.WaitGroup
 
 	// Set if checks are exposed through Connect proxies
 	// If set, this is the target of check()
@@ -388,6 +390,11 @@ func (c *CheckHTTP) Start() {
 			Timeout:   10 * time.Second,
 			Transport: trans,
 		}
+		if c.DisableRedirects {
+			c.httpClient.CheckRedirect = func(req *http.Request, via []*http.Request) error {
+				return http.ErrUseLastResponse
+			}
+		}
 		if c.Timeout > 0 {
 			c.httpClient.Timeout = c.Timeout
 		}
@@ -399,6 +406,7 @@ func (c *CheckHTTP) Start() {
 
 	c.stop = false
 	c.stopCh = make(chan struct{})
+	c.stopWg.Add(1)
 	go c.run()
 }
 
@@ -410,10 +418,14 @@ func (c *CheckHTTP) Stop() {
 		c.stop = true
 		close(c.stopCh)
 	}
+
+	// Wait for the c.run() goroutine to complete before returning.
+	c.stopWg.Wait()
 }
 
 // run is invoked by a goroutine to run until Stop() is called
 func (c *CheckHTTP) run() {
+	defer c.stopWg.Done()
 	// Get the randomized initial pause time
 	initialPauseTime := lib.RandomStagger(c.Interval)
 	next := time.After(initialPauseTime)
