@@ -143,29 +143,18 @@ func (c *ConsulProvider) State() (map[string]string, error) {
 	return c.testState, nil
 }
 
-// ActiveRoot returns the active root CA certificate.
-func (c *ConsulProvider) ActiveRoot() (string, error) {
-	_, providerState, err := c.getState()
+// GenerateRoot initializes a new root certificate and private key if needed.
+func (c *ConsulProvider) GenerateRoot() (RootResult, error) {
+	providerState, err := c.getState()
 	if err != nil {
-		return "", err
-	}
-
-	return providerState.RootCert, nil
-}
-
-// GenerateRoot initializes a new root certificate and private key
-// if needed.
-func (c *ConsulProvider) GenerateRoot() error {
-	_, providerState, err := c.getState()
-	if err != nil {
-		return err
+		return RootResult{}, err
 	}
 
 	if !c.isPrimary {
-		return fmt.Errorf("provider is not the root certificate authority")
+		return RootResult{}, fmt.Errorf("provider is not the root certificate authority")
 	}
 	if providerState.RootCert != "" {
-		return nil
+		return RootResult{PEM: providerState.RootCert}, nil
 	}
 
 	// Generate a private key if needed
@@ -173,7 +162,7 @@ func (c *ConsulProvider) GenerateRoot() error {
 	if c.config.PrivateKey == "" {
 		_, pk, err := connect.GeneratePrivateKeyWithConfig(c.config.PrivateKeyType, c.config.PrivateKeyBits)
 		if err != nil {
-			return err
+			return RootResult{}, err
 		}
 		newState.PrivateKey = pk
 	} else {
@@ -184,12 +173,12 @@ func (c *ConsulProvider) GenerateRoot() error {
 	if c.config.RootCert == "" {
 		nextSerial, err := c.incrementAndGetNextSerialNumber()
 		if err != nil {
-			return fmt.Errorf("error computing next serial number: %v", err)
+			return RootResult{}, fmt.Errorf("error computing next serial number: %v", err)
 		}
 
 		ca, err := c.generateCA(newState.PrivateKey, nextSerial)
 		if err != nil {
-			return fmt.Errorf("error generating CA: %v", err)
+			return RootResult{}, fmt.Errorf("error generating CA: %v", err)
 		}
 		newState.RootCert = ca
 	} else {
@@ -202,16 +191,16 @@ func (c *ConsulProvider) GenerateRoot() error {
 		ProviderState: &newState,
 	}
 	if _, err := c.Delegate.ApplyCARequest(args); err != nil {
-		return err
+		return RootResult{}, err
 	}
 
-	return nil
+	return RootResult{PEM: newState.RootCert}, nil
 }
 
 // GenerateIntermediateCSR creates a private key and generates a CSR
 // for another datacenter's root to sign.
 func (c *ConsulProvider) GenerateIntermediateCSR() (string, error) {
-	_, providerState, err := c.getState()
+	providerState, err := c.getState()
 	if err != nil {
 		return "", err
 	}
@@ -255,7 +244,7 @@ func (c *ConsulProvider) GenerateIntermediateCSR() (string, error) {
 // SetIntermediate validates that the given intermediate is for the right private key
 // and writes the given intermediate and root certificates to the state.
 func (c *ConsulProvider) SetIntermediate(intermediatePEM, rootPEM string) error {
-	_, providerState, err := c.getState()
+	providerState, err := c.getState()
 	if err != nil {
 		return err
 	}
@@ -264,12 +253,10 @@ func (c *ConsulProvider) SetIntermediate(intermediatePEM, rootPEM string) error
 		return fmt.Errorf("cannot set an intermediate using another root in the primary datacenter")
 	}
 
-	err = validateSetIntermediate(
-		intermediatePEM, rootPEM,
-		providerState.PrivateKey,
-		c.spiffeID,
-	)
-	if err != nil {
+	if err = validateSetIntermediate(intermediatePEM, rootPEM, c.spiffeID); err != nil {
+		return err
+	}
+	if err := validateIntermediateSignedByPrivateKey(intermediatePEM, providerState.PrivateKey); err != nil {
 		return err
 	}
 
@@ -288,18 +275,15 @@ func (c *ConsulProvider) SetIntermediate(intermediatePEM, rootPEM string) error
 	return nil
 }
 
-// We aren't maintaining separate root/intermediate CAs for the builtin
-// provider, so just return the root.
 func (c *ConsulProvider) ActiveIntermediate() (string, error) {
-	if c.isPrimary {
-		return c.ActiveRoot()
-	}
-
-	_, providerState, err := c.getState()
+	providerState, err := c.getState()
 	if err != nil {
 		return "", err
 	}
 
+	if c.isPrimary {
+		return providerState.RootCert, nil
+	}
 	return providerState.IntermediateCert, nil
 }
 
@@ -310,7 +294,11 @@ func (c *ConsulProvider) GenerateIntermediate() (string, error) {
 }
 
 // Remove the state store entry for this provider instance.
-func (c *ConsulProvider) Cleanup() error {
+func (c *ConsulProvider) Cleanup(_ bool, _ map[string]interface{}) error {
+	// This method only gets called for final cleanup. Therefore we don't
+	// need to worry about the case where a ca config update is made to
+	// change the cert ttls but leaving the private key and root cert the
+	// same. Changing those would change the id field on the provider.
 	args := &structs.CARequest{
 		Op:            structs.CAOpDeleteProviderState,
 		ProviderState: &structs.CAConsulProviderState{ID: c.id},
@@ -331,7 +319,7 @@ func (c *ConsulProvider) Sign(csr *x509.CertificateRequest) (string, error) {
 	defer c.Unlock()
 
 	// Get the provider state
-	_, providerState, err := c.getState()
+	providerState, err := c.getState()
 	if err != nil {
 		return "", err
 	}
@@ -443,7 +431,7 @@ func (c *ConsulProvider) Sign(csr *x509.CertificateRequest) (string, error) {
 // are met. It should return a signed CA certificate with a path length constraint
 // of 0 to ensure that the certificate cannot be used to generate further CA certs.
 func (c *ConsulProvider) SignIntermediate(csr *x509.CertificateRequest) (string, error) {
-	_, providerState, err := c.getState()
+	providerState, err := c.getState()
 	if err != nil {
 		return "", err
 	}
@@ -526,7 +514,7 @@ func (c *ConsulProvider) CrossSignCA(cert *x509.Certificate) (string, error) {
 	}
 
 	// Get the provider state
-	_, providerState, err := c.getState()
+	providerState, err := c.getState()
 	if err != nil {
 		return "", err
 	}
@@ -592,18 +580,18 @@ func (c *ConsulProvider) SupportsCrossSigning() (bool, error) {
 
 // getState returns the current provider state from the state delegate, and returns
 // ErrNotInitialized if no entry is found.
-func (c *ConsulProvider) getState() (uint64, *structs.CAConsulProviderState, error) {
+func (c *ConsulProvider) getState() (*structs.CAConsulProviderState, error) {
 	stateStore := c.Delegate.State()
-	idx, providerState, err := stateStore.CAProviderState(c.id)
+	_, providerState, err := stateStore.CAProviderState(c.id)
 	if err != nil {
-		return 0, nil, err
+		return nil, err
 	}
 
 	if providerState == nil {
-		return 0, nil, ErrNotInitialized
+		return nil, ErrNotInitialized
 	}
 
-	return idx, providerState, nil
+	return providerState, nil
 }
 
 func (c *ConsulProvider) incrementAndGetNextSerialNumber() (uint64, error) {
