@@ -6,15 +6,44 @@ import (
 	"sync"
 	"time"
 
-	metrics "github.com/armon/go-metrics"
-	"github.com/hashicorp/consul/acl"
-	"github.com/hashicorp/consul/agent/structs"
-	"github.com/hashicorp/consul/logging"
+	"github.com/armon/go-metrics"
+	"github.com/armon/go-metrics/prometheus"
 	"github.com/hashicorp/go-hclog"
 	"golang.org/x/sync/singleflight"
 	"golang.org/x/time/rate"
+
+	"github.com/hashicorp/consul/acl"
+	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/agent/token"
+	"github.com/hashicorp/consul/logging"
 )
 
+var ACLCounters = []prometheus.CounterDefinition{
+	{
+		Name: []string{"acl", "token", "cache_hit"},
+		Help: "Increments if Consul is able to resolve a token's identity, or a legacy token, from the cache.",
+	},
+	{
+		Name: []string{"acl", "token", "cache_miss"},
+		Help: "Increments if Consul cannot resolve a token's identity, or a legacy token, from the cache.",
+	},
+}
+
+var ACLSummaries = []prometheus.SummaryDefinition{
+	{
+		Name: []string{"acl", "resolveTokenLegacy"},
+		Help: "This measures the time it takes to resolve an ACL token using the legacy ACL system.",
+	},
+	{
+		Name: []string{"acl", "ResolveToken"},
+		Help: "This measures the time it takes to resolve an ACL token.",
+	},
+	{
+		Name: []string{"acl", "ResolveTokenToIdentity"},
+		Help: "This measures the time it takes to resolve an ACL token to an Identity.",
+	},
+}
+
 // These must be kept in sync with the constants in command/agent/acl.go.
 const (
 	// anonymousToken is the token ID we re-write to if there is no token ID
@@ -140,7 +169,6 @@ func tokenSecretCacheID(token string) string {
 }
 
 type ACLResolverDelegate interface {
-	ACLsEnabled() bool
 	ACLDatacenter(legacy bool) string
 	UseLegacyACLs() bool
 	ResolveIdentityFromToken(token string) (bool, structs.ACLIdentity, error)
@@ -178,6 +206,9 @@ type ACLResolverConfig struct {
 	// ACLConfig is the configuration necessary to pass through to the acl package when creating authorizers
 	// and when authorizing access
 	ACLConfig *acl.Config
+
+	// Tokens is the token store of locally managed tokens
+	Tokens *token.Store
 }
 
 // ACLResolver is the type to handle all your token and policy resolution needs.
@@ -212,6 +243,8 @@ type ACLResolver struct {
 	delegate ACLResolverDelegate
 	aclConf  *acl.Config
 
+	tokens *token.Store
+
 	cache         *structs.ACLCaches
 	identityGroup singleflight.Group
 	policyGroup   singleflight.Group
@@ -223,6 +256,33 @@ type ACLResolver struct {
 	autoDisable  bool
 	disabled     time.Time
 	disabledLock sync.RWMutex
+
+	agentMasterAuthz acl.Authorizer
+}
+
+func agentMasterAuthorizer(nodeName string) (acl.Authorizer, error) {
+	// Build a policy for the agent master token.
+	// The builtin agent master policy allows reading any node information
+	// and allows writes to the agent with the node name of the running agent
+	// only. This used to allow a prefix match on agent names but that seems
+	// entirely unnecessary so it is now using an exact match.
+	policy := &acl.Policy{
+		PolicyRules: acl.PolicyRules{
+			Agents: []*acl.AgentRule{
+				{
+					Node:   nodeName,
+					Policy: acl.PolicyWrite,
+				},
+			},
+			NodePrefixes: []*acl.NodeRule{
+				{
+					Name:   "",
+					Policy: acl.PolicyRead,
+				},
+			},
+		},
+	}
+	return acl.NewPolicyAuthorizerWithDefaults(acl.DenyAll(), []*acl.Policy{policy}, nil)
 }
 
 func NewACLResolver(config *ACLResolverConfig) (*ACLResolver, error) {
@@ -254,19 +314,26 @@ func NewACLResolver(config *ACLResolverConfig) (*ACLResolver, error) {
 	case "deny":
 		down = acl.DenyAll()
 	case "async-cache", "extend-cache":
-		// Leave the down policy as nil to signal this.
+		down = acl.RootAuthorizer(config.Config.ACLDefaultPolicy)
 	default:
 		return nil, fmt.Errorf("invalid ACL down policy %q", config.Config.ACLDownPolicy)
 	}
 
+	authz, err := agentMasterAuthorizer(config.Config.NodeName)
+	if err != nil {
+		return nil, fmt.Errorf("failed to initialize the agent master authorizer")
+	}
+
 	return &ACLResolver{
-		config:      config.Config,
-		logger:      config.Logger.Named(logging.ACL),
-		delegate:    config.Delegate,
-		aclConf:     config.ACLConfig,
-		cache:       cache,
-		autoDisable: config.AutoDisable,
-		down:        down,
+		config:           config.Config,
+		logger:           config.Logger.Named(logging.ACL),
+		delegate:         config.Delegate,
+		aclConf:          config.ACLConfig,
+		cache:            cache,
+		autoDisable:      config.AutoDisable,
+		down:             down,
+		tokens:           config.Tokens,
+		agentMasterAuthz: authz,
 	}, nil
 }
 
@@ -1013,11 +1080,6 @@ func (r *ACLResolver) collectRolesForIdentity(identity structs.ACLIdentity, role
 	return roles, nil
 }
 
-func (r *ACLResolver) resolveTokenToPolicies(token string) (structs.ACLPolicies, error) {
-	_, policies, err := r.resolveTokenToIdentityAndPolicies(token)
-	return policies, err
-}
-
 func (r *ACLResolver) resolveTokenToIdentityAndPolicies(token string) (structs.ACLIdentity, structs.ACLPolicies, error) {
 	var lastErr error
 	var lastIdentity structs.ACLIdentity
@@ -1109,6 +1171,19 @@ func (r *ACLResolver) disableACLsWhenUpstreamDisabled(err error) error {
 	return err
 }
 
+func (r *ACLResolver) resolveLocallyManagedToken(token string) (structs.ACLIdentity, acl.Authorizer, bool) {
+	// can only resolve local tokens if we were given a token store
+	if r.tokens == nil {
+		return nil, nil, false
+	}
+
+	if r.tokens.IsAgentMasterToken(token) {
+		return structs.NewAgentMasterTokenIdentity(r.config.NodeName, token), r.agentMasterAuthz, true
+	}
+
+	return r.resolveLocallyManagedEnterpriseToken(token)
+}
+
 func (r *ACLResolver) ResolveTokenToIdentityAndAuthorizer(token string) (structs.ACLIdentity, acl.Authorizer, error) {
 	if !r.ACLsEnabled() {
 		return nil, nil, nil
@@ -1123,6 +1198,10 @@ func (r *ACLResolver) ResolveTokenToIdentityAndAuthorizer(token string) (structs
 		token = anonymousToken
 	}
 
+	if ident, authz, ok := r.resolveLocallyManagedToken(token); ok {
+		return ident, authz, nil
+	}
+
 	if r.delegate.UseLegacyACLs() {
 		identity, authorizer, err := r.resolveTokenLegacy(token)
 		return identity, authorizer, r.disableACLsWhenUpstreamDisabled(err)
@@ -1165,11 +1244,6 @@ func (r *ACLResolver) ResolveTokenToIdentityAndAuthorizer(token string) (structs
 	return identity, acl.NewChainedAuthorizer(chain), nil
 }
 
-func (r *ACLResolver) ResolveToken(token string) (acl.Authorizer, error) {
-	_, authz, err := r.ResolveTokenToIdentityAndAuthorizer(token)
-	return authz, err
-}
-
 func (r *ACLResolver) ResolveTokenToIdentity(token string) (structs.ACLIdentity, error) {
 	if !r.ACLsEnabled() {
 		return nil, nil
@@ -1184,6 +1258,10 @@ func (r *ACLResolver) ResolveTokenToIdentity(token string) (structs.ACLIdentity,
 		token = anonymousToken
 	}
 
+	if ident, _, ok := r.resolveLocallyManagedToken(token); ok {
+		return ident, nil
+	}
+
 	if r.delegate.UseLegacyACLs() {
 		identity, _, err := r.resolveTokenLegacy(token)
 		return identity, r.disableACLsWhenUpstreamDisabled(err)
@@ -1196,7 +1274,7 @@ func (r *ACLResolver) ResolveTokenToIdentity(token string) (structs.ACLIdentity,
 
 func (r *ACLResolver) ACLsEnabled() bool {
 	// Whether we desire ACLs to be enabled according to configuration
-	if !r.delegate.ACLsEnabled() {
+	if !r.config.ACLsEnabled {
 		return false
 	}
 
@@ -1410,11 +1488,23 @@ func (f *aclFilter) filterCheckServiceNodes(nodes *structs.CheckServiceNodes) {
 	*nodes = csn
 }
 
+// filterServiceTopology is used to filter upstreams/downstreams based on ACL rules.
+// this filter is unlike others in that it also returns whether the result was filtered by ACLs
+func (f *aclFilter) filterServiceTopology(topology *structs.ServiceTopology) bool {
+	numUp := len(topology.Upstreams)
+	numDown := len(topology.Downstreams)
+
+	f.filterCheckServiceNodes(&topology.Upstreams)
+	f.filterCheckServiceNodes(&topology.Downstreams)
+
+	return numUp != len(topology.Upstreams) || numDown != len(topology.Downstreams)
+}
+
 // filterDatacenterCheckServiceNodes is used to filter nodes based on ACL rules.
 func (f *aclFilter) filterDatacenterCheckServiceNodes(datacenterNodes *map[string]structs.CheckServiceNodes) {
 	dn := *datacenterNodes
 	out := make(map[string]structs.CheckServiceNodes)
-	for dc, _ := range dn {
+	for dc := range dn {
 		nodes := dn[dc]
 		f.filterCheckServiceNodes(&nodes)
 		if len(nodes) > 0 {
@@ -1847,6 +1937,12 @@ func (r *ACLResolver) filterACLWithAuthorizer(authorizer acl.Authorizer, subj in
 	case *structs.IndexedCheckServiceNodes:
 		filt.filterCheckServiceNodes(&v.Nodes)
 
+	case *structs.IndexedServiceTopology:
+		filtered := filt.filterServiceTopology(v.ServiceTopology)
+		if filtered {
+			v.FilteredByACLs = true
+		}
+
 	case *structs.DatacenterIndexedCheckServiceNodes:
 		filt.filterDatacenterCheckServiceNodes(&v.DatacenterNodes)
 
@@ -1935,7 +2031,7 @@ func (r *ACLResolver) filterACLWithAuthorizer(authorizer acl.Authorizer, subj in
 // rules configured for the provided token.
 func (r *ACLResolver) filterACL(token string, subj interface{}) error {
 	// Get the ACL from the token
-	authorizer, err := r.ResolveToken(token)
+	_, authorizer, err := r.ResolveTokenToIdentityAndAuthorizer(token)
 	if err != nil {
 		return err
 	}
@@ -2151,23 +2247,6 @@ func vetNodeTxnOp(op *structs.TxnNodeOp, rule acl.Authorizer) error {
 	return nil
 }
 
-// vetServiceTxnOp applies the given ACL policy to a service transaction operation.
-func vetServiceTxnOp(op *structs.TxnServiceOp, rule acl.Authorizer) error {
-	// Fast path if ACLs are not enabled.
-	if rule == nil {
-		return nil
-	}
-
-	var authzContext acl.AuthorizerContext
-	op.FillAuthzContext(&authzContext)
-
-	if rule.ServiceWrite(op.Service.Service, &authzContext) != acl.Allow {
-		return acl.ErrPermissionDenied
-	}
-
-	return nil
-}
-
 // vetCheckTxnOp applies the given ACL policy to a check transaction operation.
 func vetCheckTxnOp(op *structs.TxnCheckOp, rule acl.Authorizer) error {
 	// Fast path if ACLs are not enabled.
