@@ -11,7 +11,8 @@ import (
 	"regexp"
 	"time"
 
-	metrics "github.com/armon/go-metrics"
+	"github.com/armon/go-metrics"
+	"github.com/armon/go-metrics/prometheus"
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/consul/authmethod"
 	"github.com/hashicorp/consul/agent/consul/state"
@@ -30,6 +31,73 @@ const (
 	aclBootstrapReset = "acl-bootstrap-reset"
 )
 
+var ACLEndpointSummaries = []prometheus.SummaryDefinition{
+	{
+		Name: []string{"acl", "token", "clone"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "token", "upsert"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "token", "delete"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "policy", "upsert"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "policy", "delete"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "policy", "delete"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "role", "upsert"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "role", "delete"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "bindingrule", "upsert"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "bindingrule", "delete"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "authmethod", "upsert"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "authmethod", "delete"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "login"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "login"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "logout"},
+		Help: "",
+	},
+	{
+		Name: []string{"acl", "logout"},
+		Help: "",
+	},
+}
+
 // Regex for matching
 var (
 	validPolicyName              = regexp.MustCompile(`^[A-Za-z0-9\-_]{1,128}$`)
@@ -98,7 +166,7 @@ func (a *ACL) removeBootstrapResetFile() {
 }
 
 func (a *ACL) aclPreCheck() error {
-	if !a.srv.ACLsEnabled() {
+	if !a.srv.config.ACLsEnabled {
 		return acl.ErrDisabled
 	}
 
@@ -2354,7 +2422,7 @@ func (a *ACL) Login(args *structs.ACLLoginRequest, reply *structs.ACLToken) erro
 	if err != nil {
 		return err
 	} else if method == nil {
-		return acl.ErrNotFound
+		return fmt.Errorf("%w: auth method %q not found", acl.ErrNotFound, auth.AuthMethod)
 	}
 
 	if err := a.enterpriseAuthMethodTypeValidation(method.Type); err != nil {
