@@ -337,7 +337,7 @@ func TestACLReplication_Tokens(t *testing.T) {
 			ACLToken: structs.ACLToken{
 				Description: fmt.Sprintf("token-%d", i),
 				Policies: []structs.ACLTokenPolicyLink{
-					structs.ACLTokenPolicyLink{
+					{
 						ID: structs.ACLPolicyGlobalManagementID,
 					},
 				},
@@ -423,7 +423,7 @@ func TestACLReplication_Tokens(t *testing.T) {
 			ACLToken: structs.ACLToken{
 				Description: fmt.Sprintf("token-%d", i),
 				Policies: []structs.ACLTokenPolicyLink{
-					structs.ACLTokenPolicyLink{
+					{
 						ID: structs.ACLPolicyGlobalManagementID,
 					},
 				},
@@ -443,7 +443,7 @@ func TestACLReplication_Tokens(t *testing.T) {
 			ACLToken: structs.ACLToken{
 				Description: fmt.Sprintf("token-%d", i),
 				Policies: []structs.ACLTokenPolicyLink{
-					structs.ACLTokenPolicyLink{
+					{
 						ID: structs.ACLPolicyGlobalManagementID,
 					},
 				},
@@ -464,7 +464,7 @@ func TestACLReplication_Tokens(t *testing.T) {
 				SecretID:    tokens[i].SecretID,
 				Description: fmt.Sprintf("token-%d-modified", i),
 				Policies: []structs.ACLTokenPolicyLink{
-					structs.ACLTokenPolicyLink{
+					{
 						ID: structs.ACLPolicyGlobalManagementID,
 					},
 				},
@@ -657,7 +657,7 @@ func TestACLReplication_TokensRedacted(t *testing.T) {
 		ACLToken: structs.ACLToken{
 			Description: "dc2-replication",
 			Policies: []structs.ACLTokenPolicyLink{
-				structs.ACLTokenPolicyLink{
+				{
 					ID: policy.ID,
 				},
 			},
@@ -732,7 +732,7 @@ func TestACLReplication_TokensRedacted(t *testing.T) {
 		ACLToken: structs.ACLToken{
 			Description: "management",
 			Policies: []structs.ACLTokenPolicyLink{
-				structs.ACLTokenPolicyLink{
+				{
 					ID: structs.ACLPolicyGlobalManagementID,
 				},
 			},
