@@ -6,6 +6,7 @@ import (
 	"time"
 
 	"github.com/armon/go-metrics"
+	"github.com/armon/go-metrics/prometheus"
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
@@ -17,6 +18,52 @@ import (
 	"github.com/hashicorp/go-uuid"
 )
 
+var CatalogCounters = []prometheus.CounterDefinition{
+	{
+		Name: []string{"catalog", "service", "query"},
+		Help: "Increments for each catalog query for the given service.",
+	},
+	{
+		Name: []string{"catalog", "connect", "query"},
+		Help: "Increments for each connect-based catalog query for the given service.",
+	},
+	{
+		Name: []string{"catalog", "service", "query-tag"},
+		Help: "Increments for each catalog query for the given service with the given tag.",
+	},
+	{
+		Name: []string{"catalog", "connect", "query-tag"},
+		Help: "Increments for each connect-based catalog query for the given service with the given tag.",
+	},
+	{
+		Name: []string{"catalog", "service", "query-tags"},
+		Help: "Increments for each catalog query for the given service with the given tags.",
+	},
+	{
+		Name: []string{"catalog", "connect", "query-tags"},
+		Help: "Increments for each connect-based catalog query for the given service with the given tags.",
+	},
+	{
+		Name: []string{"catalog", "service", "not-found"},
+		Help: "Increments for each catalog query where the given service could not be found.",
+	},
+	{
+		Name: []string{"catalog", "connect", "not-found"},
+		Help: "Increments for each connect-based catalog query where the given service could not be found.",
+	},
+}
+
+var CatalogSummaries = []prometheus.SummaryDefinition{
+	{
+		Name: []string{"catalog", "deregister"},
+		Help: "Measures the time it takes to complete a catalog deregister operation.",
+	},
+	{
+		Name: []string{"catalog", "register"},
+		Help: "Measures the time it takes to complete a catalog register operation.",
+	},
+}
+
 // Catalog endpoint is used to manipulate the service catalog
 type Catalog struct {
 	srv    *Server
@@ -37,7 +84,7 @@ func nodePreApply(nodeName, nodeID string) error {
 	return nil
 }
 
-func servicePreApply(service *structs.NodeService, authz acl.Authorizer) error {
+func servicePreApply(service *structs.NodeService, authz acl.Authorizer, authzCtxFill func(*acl.AuthorizerContext)) error {
 	// Validate the service. This is in addition to the below since
 	// the above just hasn't been moved over yet. We should move it over
 	// in time.
@@ -62,7 +109,7 @@ func servicePreApply(service *structs.NodeService, authz acl.Authorizer) error {
 	}
 
 	var authzContext acl.AuthorizerContext
-	service.FillAuthzContext(&authzContext)
+	authzCtxFill(&authzContext)
 
 	// Apply the ACL policy if any. The 'consul' service is excluded
 	// since it is managed automatically internally (that behavior
@@ -127,7 +174,7 @@ func (c *Catalog) Register(args *structs.RegisterRequest, reply *struct{}) error
 
 	// Handle a service registration.
 	if args.Service != nil {
-		if err := servicePreApply(args.Service, authz); err != nil {
+		if err := servicePreApply(args.Service, authz, args.Service.FillAuthzContext); err != nil {
 			return err
 		}
 	}
@@ -257,19 +304,21 @@ func (c *Catalog) ListNodes(args *structs.DCSpecificRequest, reply *structs.Inde
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			var index uint64
-			var nodes structs.Nodes
 			var err error
 			if len(args.NodeMetaFilters) > 0 {
-				index, nodes, err = state.NodesByMeta(ws, args.NodeMetaFilters)
+				reply.Index, reply.Nodes, err = state.NodesByMeta(ws, args.NodeMetaFilters)
 			} else {
-				index, nodes, err = state.Nodes(ws)
+				reply.Index, reply.Nodes, err = state.Nodes(ws)
 			}
 			if err != nil {
 				return err
 			}
+			if isUnmodified(args.QueryOptions, reply.Index) {
+				reply.QueryMeta.NotModified = true
+				reply.Nodes = nil
+				return nil
+			}
 
-			reply.Index, reply.Nodes = index, nodes
 			if err := c.srv.filterACL(args.Token, reply); err != nil {
 				return err
 			}
@@ -284,14 +333,16 @@ func (c *Catalog) ListNodes(args *structs.DCSpecificRequest, reply *structs.Inde
 		})
 }
 
+func isUnmodified(opts structs.QueryOptions, index uint64) bool {
+	return opts.AllowNotModifiedResponse && opts.MinQueryIndex > 0 && opts.MinQueryIndex == index
+}
+
 // ListServices is used to query the services in a DC
 func (c *Catalog) ListServices(args *structs.DCSpecificRequest, reply *structs.IndexedServices) error {
 	if done, err := c.srv.ForwardRPC("Catalog.ListServices", args, args, reply); done {
 		return err
 	}
 
-	(*reply).EnterpriseMeta = args.EnterpriseMeta
-
 	authz, err := c.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, nil)
 	if err != nil {
 		return err
@@ -301,23 +352,29 @@ func (c *Catalog) ListServices(args *structs.DCSpecificRequest, reply *structs.I
 		return err
 	}
 
+	// Set reply enterprise metadata after resolving and validating the token so
+	// that we can properly infer metadata from the token.
+	reply.EnterpriseMeta = args.EnterpriseMeta
+
 	return c.srv.blockingQuery(
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			var index uint64
-			var services structs.Services
 			var err error
 			if len(args.NodeMetaFilters) > 0 {
-				index, services, err = state.ServicesByNodeMeta(ws, args.NodeMetaFilters, &args.EnterpriseMeta)
+				reply.Index, reply.Services, err = state.ServicesByNodeMeta(ws, args.NodeMetaFilters, &args.EnterpriseMeta)
 			} else {
-				index, services, err = state.Services(ws, &args.EnterpriseMeta)
+				reply.Index, reply.Services, err = state.Services(ws, &args.EnterpriseMeta)
 			}
 			if err != nil {
 				return err
 			}
+			if isUnmodified(args.QueryOptions, reply.Index) {
+				reply.Services = nil
+				reply.QueryMeta.NotModified = true
+				return nil
+			}
 
-			reply.Index, reply.Services, reply.EnterpriseMeta = index, services, args.EnterpriseMeta
 			return c.srv.filterACLWithAuthorizer(authz, reply)
 		})
 }
