@@ -783,6 +783,21 @@ func TestCatalog_ListNodes(t *testing.T) {
 	if out.Nodes[0].Address != "127.0.0.1" {
 		t.Fatalf("bad: %v", out)
 	}
+	require.False(t, out.QueryMeta.NotModified)
+
+	t.Run("with option AllowNotModifiedResponse", func(t *testing.T) {
+		args.QueryOptions = structs.QueryOptions{
+			MinQueryIndex:            out.QueryMeta.Index,
+			MaxQueryTime:             20 * time.Millisecond,
+			AllowNotModifiedResponse: true,
+		}
+		err := msgpackrpc.CallWithCodec(codec, "Catalog.ListNodes", &args, &out)
+		require.NoError(t, err)
+
+		require.Equal(t, out.Index, out.QueryMeta.Index)
+		require.Len(t, out.Nodes, 0)
+		require.True(t, out.QueryMeta.NotModified, "NotModified should be true")
+	})
 }
 
 func TestCatalog_ListNodes_NodeMetaFilter(t *testing.T) {
@@ -1322,6 +1337,21 @@ func TestCatalog_ListServices(t *testing.T) {
 	if out.Services["db"][0] != "primary" {
 		t.Fatalf("bad: %v", out)
 	}
+	require.False(t, out.QueryMeta.NotModified)
+
+	t.Run("with option AllowNotModifiedResponse", func(t *testing.T) {
+		args.QueryOptions = structs.QueryOptions{
+			MinQueryIndex:            out.QueryMeta.Index,
+			MaxQueryTime:             20 * time.Millisecond,
+			AllowNotModifiedResponse: true,
+		}
+		err := msgpackrpc.CallWithCodec(codec, "Catalog.ListServices", &args, &out)
+		require.NoError(t, err)
+
+		require.Equal(t, out.Index, out.QueryMeta.Index)
+		require.Len(t, out.Services, 0)
+		require.True(t, out.QueryMeta.NotModified, "NotModified should be true")
+	})
 }
 
 func TestCatalog_ListServices_NodeMetaFilter(t *testing.T) {
@@ -1418,10 +1448,11 @@ func TestCatalog_ListServices_Blocking(t *testing.T) {
 	go func() {
 		time.Sleep(100 * time.Millisecond)
 		if err := s1.fsm.State().EnsureNode(idx+1, &structs.Node{Node: "foo", Address: "127.0.0.1"}); err != nil {
-			t.Fatalf("err: %v", err)
+			t.Errorf("err: %v", err)
+			return
 		}
 		if err := s1.fsm.State().EnsureService(idx+2, "foo", &structs.NodeService{ID: "db", Service: "db", Tags: []string{"primary"}, Address: "127.0.0.1", Port: 5000}); err != nil {
-			t.Fatalf("err: %v", err)
+			t.Errorf("err: %v", err)
 		}
 	}()
 
@@ -1643,11 +1674,11 @@ func TestCatalog_ListServiceNodes_ByAddress(t *testing.T) {
 	fooAddress := "10.1.2.3"
 	fooPort := 1111
 	fooTaggedAddresses := map[string]structs.ServiceAddress{
-		"lan": structs.ServiceAddress{
+		"lan": {
 			Address: "10.1.2.3",
 			Port:    fooPort,
 		},
-		"wan": structs.ServiceAddress{
+		"wan": {
 			Address: "198.18.1.2",
 			Port:    fooPort,
 		},
@@ -1655,11 +1686,11 @@ func TestCatalog_ListServiceNodes_ByAddress(t *testing.T) {
 	barAddress := "10.1.2.3"
 	barPort := 2222
 	barTaggedAddresses := map[string]structs.ServiceAddress{
-		"lan": structs.ServiceAddress{
+		"lan": {
 			Address: "10.1.2.3",
 			Port:    barPort,
 		},
-		"wan": structs.ServiceAddress{
+		"wan": {
 			Address: "198.18.2.3",
 			Port:    barPort,
 		},
@@ -1667,11 +1698,11 @@ func TestCatalog_ListServiceNodes_ByAddress(t *testing.T) {
 	bazAddress := "192.168.1.35"
 	bazPort := 2222
 	bazTaggedAddresses := map[string]structs.ServiceAddress{
-		"lan": structs.ServiceAddress{
+		"lan": {
 			Address: "192.168.1.35",
 			Port:    barPort,
 		},
-		"wan": structs.ServiceAddress{
+		"wan": {
 			Address: "198.18.2.4",
 			Port:    barPort,
 		},
