@@ -9,9 +9,11 @@ import (
 	"time"
 
 	"github.com/armon/go-metrics"
+	"github.com/armon/go-metrics/prometheus"
 	"github.com/hashicorp/consul/agent/pool"
 	"github.com/hashicorp/consul/agent/router"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/agent/token"
 	"github.com/hashicorp/consul/lib"
 	"github.com/hashicorp/consul/logging"
 	"github.com/hashicorp/consul/tlsutil"
@@ -21,19 +23,22 @@ import (
 	"golang.org/x/time/rate"
 )
 
+var ClientCounters = []prometheus.CounterDefinition{
+	{
+		Name: []string{"client", "rpc"},
+		Help: "Increments whenever a Consul agent in client mode makes an RPC request to a Consul server.",
+	},
+	{
+		Name: []string{"client", "rpc", "exceeded"},
+		Help: "Increments whenever a Consul agent in client mode makes an RPC request to a Consul server gets rate limited by that agent's limits configuration.",
+	},
+	{
+		Name: []string{"client", "rpc", "failed"},
+		Help: "Increments whenever a Consul agent in client mode makes an RPC request to a Consul server and fails.",
+	},
+}
+
 const (
-	// clientRPCConnMaxIdle controls how long we keep an idle connection
-	// open to a server.  127s was chosen as the first prime above 120s
-	// (arbitrarily chose to use a prime) with the intent of reusing
-	// connections who are used by once-a-minute cron(8) jobs *and* who
-	// use a 60s jitter window (e.g. in vixie cron job execution can
-	// drift by up to 59s per job, or 119s for a once-a-minute cron job).
-	clientRPCConnMaxIdle = 127 * time.Second
-
-	// clientMaxStreams controls how many idle streams we keep
-	// open to a server
-	clientMaxStreams = 32
-
 	// serfEventBacklog is the maximum number of unprocessed Serf Events
 	// that will be held in queue before new serf events block.  A
 	// blocking serf event queue is a bad thing.
@@ -68,8 +73,7 @@ type Client struct {
 	// from an agent.
 	rpcLimiter atomic.Value
 
-	// eventCh is used to receive events from the
-	// serf cluster in the datacenter
+	// eventCh is used to receive events from the serf cluster in the datacenter
 	eventCh chan serf.Event
 
 	// Logger uses the provided LogOutput
@@ -87,15 +91,13 @@ type Client struct {
 	EnterpriseClient
 
 	tlsConfigurator *tlsutil.Configurator
+
+	// tokens is the agent wide token store
+	tokens *token.Store
 }
 
 // NewClient creates and returns a Client
-func NewClient(config *Config, options ...ConsulOption) (*Client, error) {
-	flat := flattenConsulOptions(options)
-
-	tlsConfigurator := flat.tlsConfigurator
-	connPool := flat.connPool
-
+func NewClient(config *Config, deps Deps) (*Client, error) {
 	if err := config.CheckProtocolVersion(); err != nil {
 		return nil, err
 	}
@@ -105,32 +107,15 @@ func NewClient(config *Config, options ...ConsulOption) (*Client, error) {
 	if err := config.CheckACL(); err != nil {
 		return nil, err
 	}
-	if flat.logger == nil {
-		return nil, fmt.Errorf("logger is required")
-	}
 
-	if connPool == nil {
-		connPool = &pool.ConnPool{
-			Server:          false,
-			SrcAddr:         config.RPCSrcAddr,
-			Logger:          flat.logger.StandardLogger(&hclog.StandardLoggerOptions{InferLevels: true}),
-			MaxTime:         clientRPCConnMaxIdle,
-			MaxStreams:      clientMaxStreams,
-			TLSConfigurator: tlsConfigurator,
-			Datacenter:      config.Datacenter,
-		}
-	}
-
-	logger := flat.logger.NamedIntercept(logging.ConsulClient)
-
-	// Create client
 	c := &Client{
 		config:          config,
-		connPool:        connPool,
+		connPool:        deps.ConnPool,
 		eventCh:         make(chan serf.Event, serfEventBacklog),
-		logger:          logger,
+		logger:          deps.Logger.NamedIntercept(logging.ConsulClient),
 		shutdownCh:      make(chan struct{}),
-		tlsConfigurator: tlsConfigurator,
+		tlsConfigurator: deps.TLSConfigurator,
+		tokens:          deps.Tokens,
 	}
 
 	c.rpcLimiter.Store(rate.NewLimiter(config.RPCRate, config.RPCMaxBurst))
@@ -148,6 +133,7 @@ func NewClient(config *Config, options ...ConsulOption) (*Client, error) {
 		AutoDisable: true,
 		CacheConfig: clientACLCacheConfig,
 		ACLConfig:   newACLConfig(c.logger),
+		Tokens:      deps.Tokens,
 	}
 	var err error
 	if c.acls, err = NewACLResolver(&aclConfig); err != nil {
@@ -156,23 +142,17 @@ func NewClient(config *Config, options ...ConsulOption) (*Client, error) {
 	}
 
 	// Initialize the LAN Serf
-	c.serf, err = c.setupSerf(config.SerfLANConfig,
-		c.eventCh, serfLANSnapshot)
+	c.serf, err = c.setupSerf(config.SerfLANConfig, c.eventCh, serfLANSnapshot)
 	if err != nil {
 		c.Shutdown()
 		return nil, fmt.Errorf("Failed to start lan serf: %v", err)
 	}
 
-	rpcRouter := flat.router
-	if rpcRouter == nil {
-		rpcRouter = router.NewRouter(logger, config.Datacenter, fmt.Sprintf("%s.%s", config.NodeName, config.Datacenter))
-	}
-
-	if err := rpcRouter.AddArea(types.AreaLAN, c.serf, c.connPool); err != nil {
+	if err := deps.Router.AddArea(types.AreaLAN, c.serf, c.connPool); err != nil {
 		c.Shutdown()
 		return nil, fmt.Errorf("Failed to add LAN area to the RPC router: %w", err)
 	}
-	c.router = rpcRouter
+	c.router = deps.Router
 
 	// Start LAN event handlers after the router is complete since the event
 	// handlers depend on the router and the router depends on Serf.
@@ -361,7 +341,7 @@ func (c *Client) SnapshotRPC(args *structs.SnapshotRequest, in io.Reader, out io
 	// Let the caller peek at the reply.
 	if replyFn != nil {
 		if err := replyFn(&reply); err != nil {
-			return nil
+			return err
 		}
 	}
 
@@ -384,7 +364,7 @@ func (c *Client) Stats() map[string]map[string]string {
 		return strconv.FormatUint(v, 10)
 	}
 	stats := map[string]map[string]string{
-		"consul": map[string]string{
+		"consul": {
 			"server":        "false",
 			"known_servers": toString(uint64(numServers)),
 		},
@@ -392,7 +372,7 @@ func (c *Client) Stats() map[string]map[string]string {
 		"runtime":  runtimeStats(),
 	}
 
-	if c.ACLsEnabled() {
+	if c.config.ACLsEnabled {
 		if c.UseLegacyACLs() {
 			stats["consul"]["acl"] = "legacy"
 		} else {
