@@ -2,16 +2,47 @@ package consul
 
 import (
 	"fmt"
+	"reflect"
 	"time"
 
+	"github.com/armon/go-metrics/prometheus"
+
 	metrics "github.com/armon/go-metrics"
+	memdb "github.com/hashicorp/go-memdb"
+	"github.com/mitchellh/copystructure"
+
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
-	memdb "github.com/hashicorp/go-memdb"
-	"github.com/mitchellh/copystructure"
 )
 
+var ConfigSummaries = []prometheus.SummaryDefinition{
+	{
+		Name: []string{"config_entry", "apply"},
+		Help: "",
+	},
+	{
+		Name: []string{"config_entry", "get"},
+		Help: "",
+	},
+	{
+		Name: []string{"config_entry", "list"},
+		Help: "",
+	},
+	{
+		Name: []string{"config_entry", "listAll"},
+		Help: "",
+	},
+	{
+		Name: []string{"config_entry", "delete"},
+		Help: "",
+	},
+	{
+		Name: []string{"config_entry", "resolve_service_config"},
+		Help: "",
+	},
+}
+
 // The ConfigEntry endpoint is used to query centralized config information
 type ConfigEntry struct {
 	srv *Server
@@ -38,7 +69,11 @@ func (c *ConfigEntry) Apply(args *structs.ConfigEntryRequest, reply *bool) error
 		return err
 	}
 
-	// Normalize and validate the incoming config entry.
+	if err := c.preflightCheck(args.Entry.GetKind()); err != nil {
+		return err
+	}
+
+	// Normalize and validate the incoming config entry as if it came from a user.
 	if err := args.Entry.Normalize(); err != nil {
 		return err
 	}
@@ -46,6 +81,13 @@ func (c *ConfigEntry) Apply(args *structs.ConfigEntryRequest, reply *bool) error
 		return err
 	}
 
+	// Log any applicable warnings about the contents of the config entry.
+	if warnEntry, ok := args.Entry.(structs.WarningConfigEntry); ok {
+		warnings := warnEntry.Warnings()
+		for _, warning := range warnings {
+			c.srv.logger.Warn(warning)
+		}
+	}
 	if authz != nil && !args.Entry.CanWrite(authz) {
 		return acl.ErrPermissionDenied
 	}
@@ -53,6 +95,14 @@ func (c *ConfigEntry) Apply(args *structs.ConfigEntryRequest, reply *bool) error
 	if args.Op != structs.ConfigEntryUpsert && args.Op != structs.ConfigEntryUpsertCAS {
 		args.Op = structs.ConfigEntryUpsert
 	}
+
+	if skip, err := c.shouldSkipOperation(args); err != nil {
+		return err
+	} else if skip {
+		*reply = true
+		return nil
+	}
+
 	resp, err := c.srv.raftApply(structs.ConfigEntryRequestType, args)
 	if err != nil {
 		return err
@@ -67,6 +117,62 @@ func (c *ConfigEntry) Apply(args *structs.ConfigEntryRequest, reply *bool) error
 	return nil
 }
 
+// shouldSkipOperation returns true if the result of the operation has
+// already happened and is safe to skip.
+//
+// It is ok if this incorrectly detects something as changed when it
+// in fact has not, the important thing is that it doesn't do
+// the reverse and incorrectly detect a change as a no-op.
+func (c *ConfigEntry) shouldSkipOperation(args *structs.ConfigEntryRequest) (bool, error) {
+	state := c.srv.fsm.State()
+	_, currentEntry, err := state.ConfigEntry(nil, args.Entry.GetKind(), args.Entry.GetName(), args.Entry.GetEnterpriseMeta())
+	if err != nil {
+		return false, fmt.Errorf("error reading current config entry value: %w", err)
+	}
+
+	switch args.Op {
+	case structs.ConfigEntryUpsert, structs.ConfigEntryUpsertCAS:
+		return c.shouldSkipUpsertOperation(currentEntry, args.Entry)
+	case structs.ConfigEntryDelete:
+		return (currentEntry == nil), nil
+	default:
+		return false, fmt.Errorf("invalid config entry operation type: %v", args.Op)
+	}
+}
+
+func (c *ConfigEntry) shouldSkipUpsertOperation(currentEntry, updatedEntry structs.ConfigEntry) (bool, error) {
+	if currentEntry == nil {
+		return false, nil
+	}
+
+	if currentEntry.GetKind() != updatedEntry.GetKind() ||
+		currentEntry.GetName() != updatedEntry.GetName() ||
+		!currentEntry.GetEnterpriseMeta().IsSame(updatedEntry.GetEnterpriseMeta()) {
+		return false, nil
+	}
+
+	// The only reason a fully Normalized and Validated config entry may
+	// legitimately differ from the persisted one is due to the embedded
+	// RaftIndex.
+	//
+	// So, to intercept more no-op upserts we temporarily set the new config
+	// entry's raft index field to that of the existing data for the purposes
+	// of comparison, and then restore it.
+	var (
+		currentRaftIndex      = currentEntry.GetRaftIndex()
+		userProvidedRaftIndex = updatedEntry.GetRaftIndex()
+
+		currentRaftIndexCopy      = *currentRaftIndex
+		userProvidedRaftIndexCopy = *userProvidedRaftIndex
+	)
+
+	*userProvidedRaftIndex = currentRaftIndexCopy         // change
+	same := reflect.DeepEqual(currentEntry, updatedEntry) // compare
+	*userProvidedRaftIndex = userProvidedRaftIndexCopy    // restore
+
+	return same, nil
+}
+
 // Get returns a single config entry by Kind/Name.
 func (c *ConfigEntry) Get(args *structs.ConfigEntryQuery, reply *structs.ConfigEntryResponse) error {
 	if err := c.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
@@ -159,8 +265,18 @@ func (c *ConfigEntry) List(args *structs.ConfigEntryQuery, reply *structs.Indexe
 		})
 }
 
+var configEntryKindsFromConsul_1_8_0 = []string{
+	structs.ServiceDefaults,
+	structs.ProxyDefaults,
+	structs.ServiceRouter,
+	structs.ServiceSplitter,
+	structs.ServiceResolver,
+	structs.IngressGateway,
+	structs.TerminatingGateway,
+}
+
 // ListAll returns all the known configuration entries
-func (c *ConfigEntry) ListAll(args *structs.DCSpecificRequest, reply *structs.IndexedGenericConfigEntries) error {
+func (c *ConfigEntry) ListAll(args *structs.ConfigEntryListAllRequest, reply *structs.IndexedGenericConfigEntries) error {
 	if err := c.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
 		return err
 	}
@@ -175,6 +291,15 @@ func (c *ConfigEntry) ListAll(args *structs.DCSpecificRequest, reply *structs.In
 		return err
 	}
 
+	if len(args.Kinds) == 0 {
+		args.Kinds = configEntryKindsFromConsul_1_8_0
+	}
+
+	kindMap := make(map[string]struct{})
+	for _, kind := range args.Kinds {
+		kindMap[kind] = struct{}{}
+	}
+
 	return c.srv.blockingQuery(
 		&args.QueryOptions,
 		&reply.QueryMeta,
@@ -184,12 +309,21 @@ func (c *ConfigEntry) ListAll(args *structs.DCSpecificRequest, reply *structs.In
 				return err
 			}
 
-			// Filter the entries returned by ACL permissions.
+			// Filter the entries returned by ACL permissions or by the provided kinds.
 			filteredEntries := make([]structs.ConfigEntry, 0, len(entries))
 			for _, entry := range entries {
 				if authz != nil && !entry.CanRead(authz) {
 					continue
 				}
+				// Doing this filter outside of memdb isn't terribly
+				// performant. This kind filter is currently only used across
+				// version upgrades, so in the common case we are going to
+				// always return all of the data anyway, so it should be fine.
+				// If that changes at some point, then we should move this down
+				// into memdb.
+				if _, ok := kindMap[entry.GetKind()]; !ok {
+					continue
+				}
 				filteredEntries = append(filteredEntries, entry)
 			}
 
@@ -219,6 +353,10 @@ func (c *ConfigEntry) Delete(args *structs.ConfigEntryRequest, reply *struct{})
 		return err
 	}
 
+	if err := c.preflightCheck(args.Entry.GetKind()); err != nil {
+		return err
+	}
+
 	// Normalize the incoming entry.
 	if err := args.Entry.Normalize(); err != nil {
 		return err
@@ -229,6 +367,13 @@ func (c *ConfigEntry) Delete(args *structs.ConfigEntryRequest, reply *struct{})
 	}
 
 	args.Op = structs.ConfigEntryDelete
+
+	if skip, err := c.shouldSkipOperation(args); err != nil {
+		return err
+	} else if skip {
+		return nil
+	}
+
 	resp, err := c.srv.raftApply(structs.ConfigEntryRequestType, args)
 	if err != nil {
 		return err
@@ -263,9 +408,9 @@ func (c *ConfigEntry) ResolveServiceConfig(args *structs.ServiceConfigRequest, r
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			reply.Reset()
+			var thisReply structs.ServiceConfigResponse
 
-			reply.MeshGateway.Mode = structs.MeshGatewayModeDefault
+			thisReply.MeshGateway.Mode = structs.MeshGatewayModeDefault
 			// Pass the WatchSet to both the service and proxy config lookups. If either is updated
 			// during the blocking query, this function will be rerun and these state store lookups
 			// will both be current.
@@ -299,28 +444,28 @@ func (c *ConfigEntry) ResolveServiceConfig(args *structs.ServiceConfigRequest, r
 				if err != nil {
 					return fmt.Errorf("failed to copy global proxy-defaults: %v", err)
 				}
-				reply.ProxyConfig = mapCopy.(map[string]interface{})
-				reply.MeshGateway = proxyConf.MeshGateway
-				reply.Expose = proxyConf.Expose
+				thisReply.ProxyConfig = mapCopy.(map[string]interface{})
+				thisReply.MeshGateway = proxyConf.MeshGateway
+				thisReply.Expose = proxyConf.Expose
 			}
 
-			reply.Index = index
+			thisReply.Index = index
 
 			if serviceConf != nil {
 				if serviceConf.Expose.Checks {
-					reply.Expose.Checks = true
+					thisReply.Expose.Checks = true
 				}
 				if len(serviceConf.Expose.Paths) >= 1 {
-					reply.Expose.Paths = serviceConf.Expose.Paths
+					thisReply.Expose.Paths = serviceConf.Expose.Paths
 				}
 				if serviceConf.MeshGateway.Mode != structs.MeshGatewayModeDefault {
-					reply.MeshGateway.Mode = serviceConf.MeshGateway.Mode
+					thisReply.MeshGateway.Mode = serviceConf.MeshGateway.Mode
 				}
 				if serviceConf.Protocol != "" {
-					if reply.ProxyConfig == nil {
-						reply.ProxyConfig = make(map[string]interface{})
+					if thisReply.ProxyConfig == nil {
+						thisReply.ProxyConfig = make(map[string]interface{})
 					}
-					reply.ProxyConfig["protocol"] = serviceConf.Protocol
+					thisReply.ProxyConfig["protocol"] = serviceConf.Protocol
 				}
 			}
 
@@ -378,26 +523,51 @@ func (c *ConfigEntry) ResolveServiceConfig(args *structs.ServiceConfigRequest, r
 
 			// don't allocate the slices just to not fill them
 			if len(usConfigs) == 0 {
+				*reply = thisReply
 				return nil
 			}
 
 			if legacyUpstreams {
-				if reply.UpstreamConfigs == nil {
-					reply.UpstreamConfigs = make(map[string]map[string]interface{})
+				if thisReply.UpstreamConfigs == nil {
+					thisReply.UpstreamConfigs = make(map[string]map[string]interface{})
 				}
 				for us, conf := range usConfigs {
-					reply.UpstreamConfigs[us.ID] = conf
+					thisReply.UpstreamConfigs[us.ID] = conf
 				}
 			} else {
-				if reply.UpstreamIDConfigs == nil {
-					reply.UpstreamIDConfigs = make(structs.UpstreamConfigs, 0, len(usConfigs))
+				if thisReply.UpstreamIDConfigs == nil {
+					thisReply.UpstreamIDConfigs = make(structs.UpstreamConfigs, 0, len(usConfigs))
 				}
 
 				for us, conf := range usConfigs {
-					reply.UpstreamIDConfigs = append(reply.UpstreamIDConfigs, structs.UpstreamConfig{Upstream: us, Config: conf})
+					thisReply.UpstreamIDConfigs = append(thisReply.UpstreamIDConfigs, structs.UpstreamConfig{Upstream: us, Config: conf})
 				}
 			}
 
+			*reply = thisReply
 			return nil
 		})
 }
+
+// preflightCheck is meant to have kind-specific system validation outside of
+// content validation. The initial use case is restricting the ability to do
+// writes of service-intentions until the system is finished migration.
+func (c *ConfigEntry) preflightCheck(kind string) error {
+	switch kind {
+	case structs.ServiceIntentions:
+		// Exit early if Connect hasn't been enabled.
+		if !c.srv.config.ConnectEnabled {
+			return ErrConnectNotEnabled
+		}
+
+		usingConfigEntries, err := c.srv.fsm.State().AreIntentionsInConfigEntries()
+		if err != nil {
+			return fmt.Errorf("system metadata lookup failed: %v", err)
+		}
+		if !usingConfigEntries {
+			return ErrIntentionsNotUpgradedYet
+		}
+	}
+
+	return nil
+}
