@@ -1,13 +1,11 @@
 package consul
 
 import (
-	"errors"
 	"fmt"
 	"time"
 
 	metrics "github.com/armon/go-metrics"
 	"github.com/hashicorp/consul/acl"
-	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/consul/discoverychain"
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
@@ -53,39 +51,16 @@ func (c *DiscoveryChain) Get(args *structs.DiscoveryChainRequest, reply *structs
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, entries, err := state.ReadDiscoveryChainConfigEntries(ws, args.Name, entMeta)
-			if err != nil {
-				return err
-			}
-
-			_, config, err := state.CAConfig(ws)
-			if err != nil {
-				return err
-			} else if config == nil {
-				return errors.New("no cluster ca config setup")
-			}
-
-			// Build TrustDomain based on the ClusterID stored.
-			signingID := connect.SpiffeIDSigningForCluster(config)
-			if signingID == nil {
-				// If CA is bootstrapped at all then this should never happen but be
-				// defensive.
-				return errors.New("no cluster trust domain setup")
-			}
-			currentTrustDomain := signingID.Host()
-
-			// Then we compile it into something useful.
-			chain, err := discoverychain.Compile(discoverychain.CompileRequest{
+			req := discoverychain.CompileRequest{
 				ServiceName:            args.Name,
 				EvaluateInNamespace:    entMeta.NamespaceOrDefault(),
 				EvaluateInDatacenter:   evalDC,
-				EvaluateInTrustDomain:  currentTrustDomain,
 				UseInDatacenter:        c.srv.config.Datacenter,
 				OverrideMeshGateway:    args.OverrideMeshGateway,
 				OverrideProtocol:       args.OverrideProtocol,
 				OverrideConnectTimeout: args.OverrideConnectTimeout,
-				Entries:                entries,
-			})
+			}
+			index, chain, err := state.ServiceDiscoveryChain(ws, args.Name, entMeta, req)
 			if err != nil {
 				return err
 			}
