@@ -10,7 +10,6 @@ import (
 
 	"github.com/golang/protobuf/proto"
 	"github.com/hashicorp/consul/agent/connect"
-	"github.com/hashicorp/consul/agent/consul/autopilot"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/sdk/testutil"
@@ -21,7 +20,6 @@ import (
 	"github.com/hashicorp/raft"
 	"github.com/hashicorp/serf/coordinate"
 	"github.com/mitchellh/mapstructure"
-	"github.com/pascaldekloe/goe/verify"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -944,7 +942,7 @@ func TestFSM_ACL_CRUD(t *testing.T) {
 	}
 	bootstrap.ACL.CreateIndex = respACL.CreateIndex
 	bootstrap.ACL.ModifyIndex = respACL.ModifyIndex
-	verify.Values(t, "", respACL, &bootstrap.ACL)
+	require.Equal(t, &bootstrap.ACL, respACL)
 }
 
 func TestFSM_PreparedQuery_CRUD(t *testing.T) {
@@ -1164,7 +1162,7 @@ func TestFSM_Autopilot(t *testing.T) {
 	// Set the autopilot config using a request.
 	req := structs.AutopilotSetConfigRequest{
 		Datacenter: "dc1",
-		Config: autopilot.Config{
+		Config: structs.AutopilotConfig{
 			CleanupDeadServers:   true,
 			LastContactThreshold: 10 * time.Second,
 			MaxTrailingLogs:      300,
@@ -1231,6 +1229,7 @@ func TestFSM_Intention_CRUD(t *testing.T) {
 		Intention:  structs.TestIntention(t),
 	}
 	ixn.Intention.ID = generateUUID()
+	//nolint:staticcheck
 	ixn.Intention.UpdatePrecedence()
 
 	{
@@ -1241,7 +1240,7 @@ func TestFSM_Intention_CRUD(t *testing.T) {
 
 	// Verify it's in the state store.
 	{
-		_, actual, err := fsm.state.IntentionGet(nil, ixn.Intention.ID)
+		_, _, actual, err := fsm.state.IntentionGet(nil, ixn.Intention.ID)
 		assert.Nil(err)
 
 		actual.CreateIndex, actual.ModifyIndex = 0, 0
@@ -1261,7 +1260,7 @@ func TestFSM_Intention_CRUD(t *testing.T) {
 
 	// Verify the update.
 	{
-		_, actual, err := fsm.state.IntentionGet(nil, ixn.Intention.ID)
+		_, _, actual, err := fsm.state.IntentionGet(nil, ixn.Intention.ID)
 		assert.Nil(err)
 
 		actual.CreateIndex, actual.ModifyIndex = 0, 0
@@ -1280,7 +1279,7 @@ func TestFSM_Intention_CRUD(t *testing.T) {
 
 	// Make sure it's gone.
 	{
-		_, actual, err := fsm.state.IntentionGet(nil, ixn.Intention.ID)
+		_, _, actual, err := fsm.state.IntentionGet(nil, ixn.Intention.ID)
 		assert.Nil(err)
 		assert.Nil(actual)
 	}
