@@ -10,7 +10,7 @@ import (
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/testrpc"
 	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
-	"github.com/pascaldekloe/goe/verify"
+	"github.com/stretchr/testify/require"
 )
 
 func TestKVS_Apply(t *testing.T) {
@@ -354,7 +354,7 @@ func TestKVSEndpoint_List_Blocking(t *testing.T) {
 		}
 		var out bool
 		if err := msgpackrpc.CallWithCodec(codec, "KVS.Apply", &arg, &out); err != nil {
-			t.Fatalf("err: %v", err)
+			t.Errorf("RPC call failed: %v", err)
 		}
 	}()
 
@@ -584,7 +584,7 @@ key "zip" {
 		actualKeys = append(actualKeys, entry.Key)
 	}
 
-	verify.Values(t, "", actualKeys, expectedKeys)
+	require.Equal(t, expectedKeys, actualKeys)
 
 	// list keys with a prefix that has list permissions should succeed
 	getKeysReq2 := structs.KeyListRequest{
@@ -598,7 +598,7 @@ key "zip" {
 
 	actualKeys = keyList.Keys
 
-	verify.Values(t, "", actualKeys, expectedKeys)
+	require.Equal(t, expectedKeys, actualKeys)
 
 }
 
@@ -891,7 +891,8 @@ func TestKVS_Issue_1626(t *testing.T) {
 		}
 		var dirent structs.IndexedDirEntries
 		if err := msgpackrpc.CallWithCodec(codec, "KVS.Get", &getR, &dirent); err != nil {
-			t.Fatalf("err: %v", err)
+			t.Errorf("RPC call failed: %v", err)
+			return
 		}
 		doneCh <- &dirent
 	}()
