@@ -11,8 +11,8 @@ import (
 	"time"
 
 	"github.com/armon/go-metrics"
+	"github.com/armon/go-metrics/prometheus"
 	"github.com/hashicorp/consul/acl"
-	"github.com/hashicorp/consul/agent/consul/autopilot"
 	"github.com/hashicorp/consul/agent/metadata"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
@@ -28,6 +28,21 @@ import (
 	"golang.org/x/time/rate"
 )
 
+var LeaderSummaries = []prometheus.SummaryDefinition{
+	{
+		Name: []string{"leader", "barrier"},
+		Help: "Measures the time spent waiting for the raft barrier upon gaining leadership.",
+	},
+	{
+		Name: []string{"leader", "reconcileMember"},
+		Help: "Measures the time spent updating the raft store for a single serf member's information.",
+	},
+	{
+		Name: []string{"leader", "reapTombstones"},
+		Help: "Measures the time spent clearing tombstones.",
+	},
+}
+
 const (
 	newLeaderEvent      = "consul:new-leader"
 	barrierWriteTimeout = 2 * time.Minute
@@ -37,10 +52,6 @@ var (
 	// caRootPruneInterval is how often we check for stale CARoots to remove.
 	caRootPruneInterval = time.Hour
 
-	// minAutopilotVersion is the minimum Consul version in which Autopilot features
-	// are supported.
-	minAutopilotVersion = version.Must(version.NewVersion("0.8.0"))
-
 	// minCentralizedConfigVersion is the minimum Consul version in which centralized
 	// config is supported
 	minCentralizedConfigVersion = version.Must(version.NewVersion("1.5.0"))
@@ -58,7 +69,7 @@ func (s *Server) monitorLeadership() {
 
 	aclModeCheckWait := aclModeCheckMinInterval
 	var aclUpgradeCh <-chan time.Time
-	if s.ACLsEnabled() {
+	if s.config.ACLsEnabled {
 		aclUpgradeCh = time.After(aclModeCheckWait)
 	}
 	var weAreLeaderCh chan struct{}
@@ -151,6 +162,8 @@ func (s *Server) leadershipTransfer() error {
 // leaderLoop runs as long as we are the leader to run various
 // maintenance activities
 func (s *Server) leaderLoop(stopCh chan struct{}) {
+	stopCtx := &lib.StopChannelContext{StopCh: stopCh}
+
 	// Fire a user event indicating a new leader
 	payload := []byte(s.config.NodeName)
 	for name, segment := range s.LANSegments() {
@@ -183,7 +196,7 @@ RECONCILE:
 
 	// Check if we need to handle initial leadership actions
 	if !establishedLeader {
-		if err := s.establishLeadership(); err != nil {
+		if err := s.establishLeadership(stopCtx); err != nil {
 			s.logger.Error("failed to establish leadership", "error", err)
 			// Immediately revoke leadership since we didn't successfully
 			// establish leadership.
@@ -256,7 +269,7 @@ WAIT:
 			// leader, which means revokeLeadership followed by an
 			// establishLeadership().
 			s.revokeLeadership()
-			err := s.establishLeadership()
+			err := s.establishLeadership(stopCtx)
 			errCh <- err
 
 			// in case establishLeadership failed, we will try to
@@ -289,7 +302,8 @@ WAIT:
 // to invoke an initial barrier. The barrier is used to ensure any
 // previously inflight transactions have been committed and that our
 // state is up-to-date.
-func (s *Server) establishLeadership() error {
+func (s *Server) establishLeadership(ctx context.Context) error {
+	start := time.Now()
 	// check for the upgrade here - this helps us transition to new ACLs much
 	// quicker if this is a new cluster or this is a test agent
 	if canUpgrade := s.canUpgradeToNewACLs(true); canUpgrade {
@@ -326,17 +340,8 @@ func (s *Server) establishLeadership() error {
 		return err
 	}
 
-	// attempt to bootstrap config entries
-	if err := s.bootstrapConfigEntries(s.config.ConfigEntryBootstrap); err != nil {
-		return err
-	}
-
 	s.getOrCreateAutopilotConfig()
-	s.autopilot.Start()
-
-	if err := s.caManager.InitializeCA(); err != nil {
-		return err
-	}
+	s.autopilot.Start(ctx)
 
 	s.startConfigReplication()
 
@@ -344,9 +349,20 @@ func (s *Server) establishLeadership() error {
 
 	s.startFederationStateAntiEntropy()
 
-	s.startConnectLeader()
+	if err := s.startConnectLeader(); err != nil {
+		return err
+	}
+
+	// Attempt to bootstrap config entries. We wait until after starting the
+	// Connect leader tasks so we hopefully have transitioned to supporting
+	// service-intentions.
+	if err := s.bootstrapConfigEntries(s.config.ConfigEntryBootstrap); err != nil {
+		return err
+	}
 
 	s.setConsistentReadReady()
+
+	s.logger.Debug("successfully established leadership", "duration", time.Since(start))
 	return nil
 }
 
@@ -371,7 +387,7 @@ func (s *Server) revokeLeadership() {
 	s.stopConnectLeader()
 
 	s.caManager.setCAProvider(nil, nil)
-	s.caManager.setState(CAStateUninitialized, false)
+	s.caManager.setState(caStateUninitialized, false)
 
 	s.stopACLTokenReaping()
 
@@ -379,12 +395,14 @@ func (s *Server) revokeLeadership() {
 
 	s.resetConsistentReadReady()
 
-	s.autopilot.Stop()
+	// Stop returns a chan and we want to block until it is closed
+	// which indicates that autopilot is actually stopped.
+	<-s.autopilot.Stop()
 }
 
 // DEPRECATED (ACL-Legacy-Compat) - Remove once old ACL compatibility is removed
 func (s *Server) initializeLegacyACL() error {
-	if !s.ACLsEnabled() {
+	if !s.config.ACLsEnabled {
 		return nil
 	}
 
@@ -484,7 +502,7 @@ func (s *Server) initializeLegacyACL() error {
 // initializeACLs is used to setup the ACLs if we are the leader
 // and need to do this.
 func (s *Server) initializeACLs(upgrade bool) error {
-	if !s.ACLsEnabled() {
+	if !s.config.ACLsEnabled {
 		return nil
 	}
 
@@ -794,10 +812,19 @@ func (s *Server) runLegacyACLReplication(ctx context.Context) error {
 		}
 
 		if err != nil {
+			metrics.SetGauge([]string{"leader", "replication", "acl-legacy", "status"},
+				0,
+			)
 			lastRemoteIndex = 0
 			s.updateACLReplicationStatusError()
 			legacyACLLogger.Warn("Legacy ACL replication error (will retry if still leader)", "error", err)
 		} else {
+			metrics.SetGauge([]string{"leader", "replication", "acl-legacy", "status"},
+				1,
+			)
+			metrics.SetGauge([]string{"leader", "replication", "acl-legacy", "index"},
+				float32(index),
+			)
 			lastRemoteIndex = index
 			s.updateACLReplicationStatusIndex(structs.ACLReplicateLegacy, index)
 			legacyACLLogger.Debug("Legacy ACL replication completed through remote index", "index", index)
@@ -855,7 +882,7 @@ type replicateFunc func(ctx context.Context, logger hclog.Logger, lastRemoteInde
 func (s *Server) runACLPolicyReplicator(ctx context.Context) error {
 	policyLogger := s.aclReplicationLogger(structs.ACLReplicatePolicies.SingularNoun())
 	policyLogger.Info("started ACL Policy replication")
-	return s.runACLReplicator(ctx, policyLogger, structs.ACLReplicatePolicies, s.replicateACLPolicies)
+	return s.runACLReplicator(ctx, policyLogger, structs.ACLReplicatePolicies, s.replicateACLPolicies, "acl-policies")
 }
 
 // This function is only intended to be run as a managed go routine, it will block until
@@ -863,7 +890,7 @@ func (s *Server) runACLPolicyReplicator(ctx context.Context) error {
 func (s *Server) runACLRoleReplicator(ctx context.Context) error {
 	roleLogger := s.aclReplicationLogger(structs.ACLReplicateRoles.SingularNoun())
 	roleLogger.Info("started ACL Role replication")
-	return s.runACLReplicator(ctx, roleLogger, structs.ACLReplicateRoles, s.replicateACLRoles)
+	return s.runACLReplicator(ctx, roleLogger, structs.ACLReplicateRoles, s.replicateACLRoles, "acl-roles")
 }
 
 // This function is only intended to be run as a managed go routine, it will block until
@@ -871,7 +898,7 @@ func (s *Server) runACLRoleReplicator(ctx context.Context) error {
 func (s *Server) runACLTokenReplicator(ctx context.Context) error {
 	tokenLogger := s.aclReplicationLogger(structs.ACLReplicateTokens.SingularNoun())
 	tokenLogger.Info("started ACL Token replication")
-	return s.runACLReplicator(ctx, tokenLogger, structs.ACLReplicateTokens, s.replicateACLTokens)
+	return s.runACLReplicator(ctx, tokenLogger, structs.ACLReplicateTokens, s.replicateACLTokens, "acl-tokens")
 }
 
 // This function is only intended to be run as a managed go routine, it will block until
@@ -881,6 +908,7 @@ func (s *Server) runACLReplicator(
 	logger hclog.Logger,
 	replicationType structs.ACLReplicationType,
 	replicateFunc replicateFunc,
+	metricName string,
 ) error {
 	var failedAttempts uint
 	limiter := rate.NewLimiter(rate.Limit(s.config.ACLReplicationRate), s.config.ACLReplicationBurst)
@@ -901,6 +929,9 @@ func (s *Server) runACLReplicator(
 		}
 
 		if err != nil {
+			metrics.SetGauge([]string{"leader", "replication", metricName, "status"},
+				0,
+			)
 			lastRemoteIndex = 0
 			s.updateACLReplicationStatusError()
 			logger.Warn("ACL replication error (will retry if still leader)",
@@ -917,6 +948,12 @@ func (s *Server) runACLReplicator(
 				// do nothing
 			}
 		} else {
+			metrics.SetGauge([]string{"leader", "replication", metricName, "status"},
+				1,
+			)
+			metrics.SetGauge([]string{"leader", "replication", metricName, "index"},
+				float32(index),
+			)
 			lastRemoteIndex = index
 			s.updateACLReplicationStatusIndex(replicationType, index)
 			logger.Debug("ACL replication completed through remote index",
@@ -962,16 +999,26 @@ func (s *Server) startFederationStateReplication() {
 		return
 	}
 
+	if s.gatewayLocator != nil {
+		s.gatewayLocator.SetUseReplicationSignal(true)
+		s.gatewayLocator.SetLastFederationStateReplicationError(nil, false)
+	}
+
 	s.leaderRoutineManager.Start(federationStateReplicationRoutineName, s.federationStateReplicator.Run)
 }
 
 func (s *Server) stopFederationStateReplication() {
 	// will be a no-op when not started
 	s.leaderRoutineManager.Stop(federationStateReplicationRoutineName)
+
+	if s.gatewayLocator != nil {
+		s.gatewayLocator.SetUseReplicationSignal(false)
+		s.gatewayLocator.SetLastFederationStateReplicationError(nil, false)
+	}
 }
 
 // getOrCreateAutopilotConfig is used to get the autopilot config, initializing it if necessary
-func (s *Server) getOrCreateAutopilotConfig() *autopilot.Config {
+func (s *Server) getOrCreateAutopilotConfig() *structs.AutopilotConfig {
 	logger := s.loggers.Named(logging.Autopilot)
 	state := s.fsm.State()
 	_, config, err := state.AutopilotConfig()
@@ -983,11 +1030,6 @@ func (s *Server) getOrCreateAutopilotConfig() *autopilot.Config {
 		return config
 	}
 
-	if ok, _ := ServersInDCMeetMinimumVersion(s, s.config.Datacenter, minAutopilotVersion); !ok {
-		logger.Warn("can't initialize until all servers are >= " + minAutopilotVersion.String())
-		return nil
-	}
-
 	config = s.config.AutopilotConfig
 	req := structs.AutopilotSetConfigRequest{Config: *config}
 	if _, err = s.raftApply(structs.AutopilotRequestType, req); err != nil {
@@ -1017,6 +1059,37 @@ func (s *Server) bootstrapConfigEntries(entries []structs.ConfigEntry) error {
 	}
 
 	state := s.fsm.State()
+
+	// Do some quick preflight checks to see if someone is doing something
+	// that's not allowed at this time:
+	//
+	// - Trying to upgrade from an older pre-1.9.0 version of consul with
+	// intentions AND are trying to bootstrap a service-intentions config entry
+	// at the same time.
+	//
+	// - Trying to insert service-intentions config entries when connect is
+	// disabled.
+
+	usingConfigEntries, err := s.fsm.State().AreIntentionsInConfigEntries()
+	if err != nil {
+		return fmt.Errorf("Failed to determine if we are migrating intentions yet: %v", err)
+	}
+
+	if !usingConfigEntries || !s.config.ConnectEnabled {
+		for _, entry := range entries {
+			if entry.GetKind() == structs.ServiceIntentions {
+				if !s.config.ConnectEnabled {
+					return fmt.Errorf("Refusing to apply configuration entry %q / %q because Connect must be enabled to bootstrap intentions",
+						entry.GetKind(), entry.GetName())
+				}
+				if !usingConfigEntries {
+					return fmt.Errorf("Refusing to apply configuration entry %q / %q because intentions are still being migrated to config entries",
+						entry.GetKind(), entry.GetName())
+				}
+			}
+		}
+	}
+
 	for _, entry := range entries {
 		// avoid a round trip through Raft if we know the CAS is going to fail
 		_, existing, err := state.ConfigEntry(nil, entry.GetKind(), entry.GetName(), entry.GetEnterpriseMeta())
@@ -1184,7 +1257,9 @@ func (s *Server) handleAliveMember(member serf.Member) error {
 				Warning: 1,
 			},
 			Meta: map[string]string{
-				"non_voter":             strconv.FormatBool(member.Tags["non_voter"] == "true"),
+				// DEPRECATED - remove nonvoter in favor of read_replica in a future version of consul
+				"non_voter":             strconv.FormatBool(member.Tags["nonvoter"] == "1"),
+				"read_replica":          strconv.FormatBool(member.Tags["read_replica"] == "1"),
 				"raft_version":          strconv.Itoa(parts.RaftVersion),
 				"serf_protocol_current": strconv.FormatUint(uint64(member.ProtocolCur), 10),
 				"serf_protocol_min":     strconv.FormatUint(uint64(member.ProtocolMin), 10),
@@ -1338,8 +1413,8 @@ func (s *Server) handleDeregisterMember(reason string, member serf.Member) error
 	}
 
 	// Remove from Raft peers if this was a server
-	if valid, parts := metadata.IsConsulServer(member); valid {
-		if err := s.removeConsulServer(member, parts.Port); err != nil {
+	if valid, _ := metadata.IsConsulServer(member); valid {
+		if err := s.removeConsulServer(member); err != nil {
 			return err
 		}
 	}
@@ -1381,135 +1456,31 @@ func (s *Server) joinConsulServer(m serf.Member, parts *metadata.Server) error {
 		}
 	}
 
-	// Processing ourselves could result in trying to remove ourselves to
-	// fix up our address, which would make us step down. This is only
-	// safe to attempt if there are multiple servers available.
-	configFuture := s.raft.GetConfiguration()
-	if err := configFuture.Error(); err != nil {
-		s.logger.Error("failed to get raft configuration", "error", err)
-		return err
-	}
-	if m.Name == s.config.NodeName {
-		if l := len(configFuture.Configuration().Servers); l < 3 {
-			s.logger.Debug("Skipping self join check for node since the cluster is too small", "node", m.Name)
-			return nil
-		}
-	}
+	// We used to do a check here and prevent adding the server if the cluster size was too small (1 or 2 servers) as a means
+	// of preventing the case where we may remove ourselves and cause a loss of leadership. The Autopilot AddServer function
+	// will now handle simple address updates better and so long as the address doesn't conflict with another node
+	// it will not require a removal but will instead just update the address. If it would require a removal of other nodes
+	// due to conflicts then the logic regarding cluster sizes will kick in and prevent doing anything dangerous that could
+	// cause loss of leadership.
 
-	// See if it's already in the configuration. It's harmless to re-add it
-	// but we want to avoid doing that if possible to prevent useless Raft
-	// log entries. If the address is the same but the ID changed, remove the
-	// old server before adding the new one.
-	addr := (&net.TCPAddr{IP: m.Addr, Port: parts.Port}).String()
-	minRaftProtocol, err := s.autopilot.MinRaftProtocol()
+	// get the autpilot library version of a server from the serf member
+	apServer, err := s.autopilotServer(m)
 	if err != nil {
 		return err
 	}
-	for _, server := range configFuture.Configuration().Servers {
-		// No-op if the raft version is too low
-		if server.Address == raft.ServerAddress(addr) && (minRaftProtocol < 2 || parts.RaftVersion < 3) {
-			return nil
-		}
 
-		// If the address or ID matches an existing server, see if we need to remove the old one first
-		if server.Address == raft.ServerAddress(addr) || server.ID == raft.ServerID(parts.ID) {
-			// Exit with no-op if this is being called on an existing server
-			if server.Address == raft.ServerAddress(addr) && server.ID == raft.ServerID(parts.ID) {
-				return nil
-			}
-			future := s.raft.RemoveServer(server.ID, 0, 0)
-			if server.Address == raft.ServerAddress(addr) {
-				if err := future.Error(); err != nil {
-					return fmt.Errorf("error removing server with duplicate address %q: %s", server.Address, err)
-				}
-				s.logger.Info("removed server with duplicate address", "address", server.Address)
-			} else {
-				if err := future.Error(); err != nil {
-					return fmt.Errorf("error removing server with duplicate ID %q: %s", server.ID, err)
-				}
-				s.logger.Info("removed server with duplicate ID", "id", server.ID)
-			}
-		}
-	}
-
-	// Attempt to add as a peer
-	switch {
-	case minRaftProtocol >= 3:
-		addFuture := s.raft.AddNonvoter(raft.ServerID(parts.ID), raft.ServerAddress(addr), 0, 0)
-		if err := addFuture.Error(); err != nil {
-			s.logger.Error("failed to add raft peer", "error", err)
-			return err
-		}
-	case minRaftProtocol == 2 && parts.RaftVersion >= 3:
-		addFuture := s.raft.AddVoter(raft.ServerID(parts.ID), raft.ServerAddress(addr), 0, 0)
-		if err := addFuture.Error(); err != nil {
-			s.logger.Error("failed to add raft peer", "error", err)
-			return err
-		}
-	default:
-		addFuture := s.raft.AddPeer(raft.ServerAddress(addr))
-		if err := addFuture.Error(); err != nil {
-			s.logger.Error("failed to add raft peer", "error", err)
-			return err
-		}
-	}
-
-	// Trigger a check to remove dead servers
-	s.autopilot.RemoveDeadServers()
-
-	return nil
+	// now ask autopilot to add it
+	return s.autopilot.AddServer(apServer)
 }
 
 // removeConsulServer is used to try to remove a consul server that has left
-func (s *Server) removeConsulServer(m serf.Member, port int) error {
-	addr := (&net.TCPAddr{IP: m.Addr, Port: port}).String()
-
-	// See if it's already in the configuration. It's harmless to re-remove it
-	// but we want to avoid doing that if possible to prevent useless Raft
-	// log entries.
-	configFuture := s.raft.GetConfiguration()
-	if err := configFuture.Error(); err != nil {
-		s.logger.Error("failed to get raft configuration", "error", err)
+func (s *Server) removeConsulServer(m serf.Member) error {
+	server, err := s.autopilotServer(m)
+	if err != nil || server == nil {
 		return err
 	}
 
-	minRaftProtocol, err := s.autopilot.MinRaftProtocol()
-	if err != nil {
-		return err
-	}
-
-	_, parts := metadata.IsConsulServer(m)
-
-	// Pick which remove API to use based on how the server was added.
-	for _, server := range configFuture.Configuration().Servers {
-		// If we understand the new add/remove APIs and the server was added by ID, use the new remove API
-		if minRaftProtocol >= 2 && server.ID == raft.ServerID(parts.ID) {
-			s.logger.Info("removing server by ID", "id", server.ID)
-			future := s.raft.RemoveServer(raft.ServerID(parts.ID), 0, 0)
-			if err := future.Error(); err != nil {
-				s.logger.Error("failed to remove raft peer",
-					"id", server.ID,
-					"error", err,
-				)
-				return err
-			}
-			break
-		} else if server.Address == raft.ServerAddress(addr) {
-			// If not, use the old remove API
-			s.logger.Info("removing server by address", "address", server.Address)
-			future := s.raft.RemovePeer(raft.ServerAddress(addr))
-			if err := future.Error(); err != nil {
-				s.logger.Error("failed to remove raft peer",
-					"address", addr,
-					"error", err,
-				)
-				return err
-			}
-			break
-		}
-	}
-
-	return nil
+	return s.autopilot.RemoveServer(server.ID)
 }
 
 // reapTombstones is invoked by the current leader to manage garbage
@@ -1534,6 +1505,10 @@ func (s *Server) reapTombstones(index uint64) {
 	}
 }
 
+func (s *Server) setDatacenterSupportsFederationStates() {
+	atomic.StoreInt32(&s.dcSupportsFederationStates, 1)
+}
+
 func (s *Server) DatacenterSupportsFederationStates() bool {
 	if atomic.LoadInt32(&s.dcSupportsFederationStates) != 0 {
 		return true
@@ -1544,7 +1519,7 @@ func (s *Server) DatacenterSupportsFederationStates() bool {
 		found:     false,
 	}
 
-	// check if they are supported in the primary dc
+	// if we are in a secondary, check if they are supported in the primary dc
 	if s.config.PrimaryDatacenter != s.config.Datacenter {
 		s.router.CheckServers(s.config.PrimaryDatacenter, state.update)
 
@@ -1558,7 +1533,7 @@ func (s *Server) DatacenterSupportsFederationStates() bool {
 	s.router.CheckServers(s.config.Datacenter, state.update)
 
 	if state.supported && state.found {
-		atomic.StoreInt32(&s.dcSupportsFederationStates, 1)
+		s.setDatacenterSupportsFederationStates()
 		return true
 	}
 
@@ -1594,3 +1569,68 @@ func (s *serversFederationStatesInfo) update(srv *metadata.Server) bool {
 	// prevent continuing server evaluation
 	return false
 }
+
+func (s *Server) setDatacenterSupportsIntentionsAsConfigEntries() {
+	atomic.StoreInt32(&s.dcSupportsIntentionsAsConfigEntries, 1)
+}
+
+func (s *Server) DatacenterSupportsIntentionsAsConfigEntries() bool {
+	if atomic.LoadInt32(&s.dcSupportsIntentionsAsConfigEntries) != 0 {
+		return true
+	}
+
+	state := serversIntentionsAsConfigEntriesInfo{
+		supported: true,
+		found:     false,
+	}
+
+	// if we are in a secondary, check if they are supported in the primary dc
+	if s.config.PrimaryDatacenter != s.config.Datacenter {
+		s.router.CheckServers(s.config.PrimaryDatacenter, state.update)
+
+		if !state.supported || !state.found {
+			s.logger.Debug("intentions have not been migrated to config entries in the primary dc yet")
+			return false
+		}
+	}
+
+	// check the servers in the local DC
+	s.router.CheckServers(s.config.Datacenter, state.update)
+
+	if state.supported && state.found {
+		s.setDatacenterSupportsIntentionsAsConfigEntries()
+		return true
+	}
+
+	s.logger.Debug("intentions cannot be migrated to config entries in this datacenter", "datacenter", s.config.Datacenter)
+	return false
+}
+
+type serversIntentionsAsConfigEntriesInfo struct {
+	// supported indicates whether every processed server supports intentions as config entries
+	supported bool
+
+	// found indicates that at least one server was processed
+	found bool
+}
+
+func (s *serversIntentionsAsConfigEntriesInfo) update(srv *metadata.Server) bool {
+	if srv.Status != serf.StatusAlive && srv.Status != serf.StatusFailed {
+		// they are left or something so regardless we treat these servers as meeting
+		// the version requirement
+		return true
+	}
+
+	// mark that we processed at least one server
+	s.found = true
+
+	if supported, ok := srv.FeatureFlags["si"]; ok && supported == 1 {
+		return true
+	}
+
+	// mark that at least one server does not support service-intentions
+	s.supported = false
+
+	// prevent continuing server evaluation
+	return false
+}
