@@ -11,9 +11,25 @@ import (
 
 type LeaderRoutine func(ctx context.Context) error
 
+// cancelCh is the ctx.Done()
+// When cancel() is called, if the routine is running a blocking call (e.g. some ACL replication RPCs),
+//   stoppedCh won't be closed till the blocking call returns, while cancelCh will be closed immediately.
+// cancelCh is used to properly detect routine running status between cancel() and close(stoppedCh)
 type leaderRoutine struct {
-	running bool
-	cancel  context.CancelFunc
+	cancel    context.CancelFunc
+	cancelCh  <-chan struct{} // closed when ctx is done
+	stoppedCh chan struct{}   // closed when no longer running
+}
+
+func (r *leaderRoutine) running() bool {
+	select {
+	case <-r.stoppedCh:
+		return false
+	case <-r.cancelCh:
+		return false
+	default:
+		return true
+	}
 }
 
 type LeaderRoutineManager struct {
@@ -41,7 +57,7 @@ func (m *LeaderRoutineManager) IsRunning(name string) bool {
 	defer m.lock.Unlock()
 
 	if routine, ok := m.routines[name]; ok {
-		return routine.running
+		return routine.running()
 	}
 
 	return false
@@ -55,7 +71,7 @@ func (m *LeaderRoutineManager) StartWithContext(parentCtx context.Context, name
 	m.lock.Lock()
 	defer m.lock.Unlock()
 
-	if instance, ok := m.routines[name]; ok && instance.running {
+	if instance, ok := m.routines[name]; ok && instance.running() {
 		return nil
 	}
 
@@ -65,11 +81,16 @@ func (m *LeaderRoutineManager) StartWithContext(parentCtx context.Context, name
 
 	ctx, cancel := context.WithCancel(parentCtx)
 	instance := &leaderRoutine{
-		running: true,
-		cancel:  cancel,
+		cancel:    cancel,
+		cancelCh:  ctx.Done(),
+		stoppedCh: make(chan struct{}),
 	}
 
 	go func() {
+		defer func() {
+			close(instance.stoppedCh)
+		}()
+
 		err := routine(ctx)
 		if err != nil && err != context.DeadlineExceeded && err != context.Canceled {
 			m.logger.Error("routine exited with error",
@@ -77,12 +98,8 @@ func (m *LeaderRoutineManager) StartWithContext(parentCtx context.Context, name
 				"error", err,
 			)
 		} else {
-			m.logger.Debug("stopped routine", "routine", name)
+			m.logger.Info("stopped routine", "routine", name)
 		}
-
-		m.lock.Lock()
-		instance.running = false
-		m.lock.Unlock()
 	}()
 
 	m.routines[name] = instance
@@ -90,7 +107,23 @@ func (m *LeaderRoutineManager) StartWithContext(parentCtx context.Context, name
 	return nil
 }
 
-func (m *LeaderRoutineManager) Stop(name string) error {
+// Caveat: The returned stoppedCh indicates that the routine is completed
+//         It's possible that ctx is canceled, but stoppedCh not yet closed
+//         Use mgr.IsRunning(name) than this stoppedCh to tell whether the
+//         instance is still running (not cancelled or completed).
+func (m *LeaderRoutineManager) Stop(name string) <-chan struct{} {
+	instance := m.stopInstance(name)
+	if instance == nil {
+		// Fabricate a closed channel so it won't block forever.
+		ch := make(chan struct{})
+		close(ch)
+		return ch
+	}
+
+	return instance.stoppedCh
+}
+
+func (m *LeaderRoutineManager) stopInstance(name string) *leaderRoutine {
 	m.lock.Lock()
 	defer m.lock.Unlock()
 
@@ -100,15 +133,16 @@ func (m *LeaderRoutineManager) Stop(name string) error {
 		return nil
 	}
 
-	if !instance.running {
-		return nil
+	if !instance.running() {
+		return instance
 	}
 
-	m.logger.Debug("stopping routine", "routine", name)
+	m.logger.Info("stopping routine", "routine", name)
 	instance.cancel()
 
 	delete(m.routines, name)
-	return nil
+
+	return instance
 }
 
 func (m *LeaderRoutineManager) StopAll() {
@@ -116,10 +150,10 @@ func (m *LeaderRoutineManager) StopAll() {
 	defer m.lock.Unlock()
 
 	for name, routine := range m.routines {
-		if !routine.running {
+		if !routine.running() {
 			continue
 		}
-		m.logger.Debug("stopping routine", "routine", name)
+		m.logger.Info("stopping routine", "routine", name)
 		routine.cancel()
 	}
 
