@@ -4,12 +4,13 @@ import (
 	"fmt"
 
 	"github.com/hashicorp/consul/acl"
-	"github.com/hashicorp/consul/agent/consul/autopilot"
 	"github.com/hashicorp/consul/agent/structs"
+	autopilot "github.com/hashicorp/raft-autopilot"
+	"github.com/hashicorp/serf/serf"
 )
 
 // AutopilotGetConfiguration is used to retrieve the current Autopilot configuration.
-func (op *Operator) AutopilotGetConfiguration(args *structs.DCSpecificRequest, reply *autopilot.Config) error {
+func (op *Operator) AutopilotGetConfiguration(args *structs.DCSpecificRequest, reply *structs.AutopilotConfig) error {
 	if done, err := op.srv.ForwardRPC("Operator.AutopilotGetConfiguration", args, args, reply); done {
 		return err
 	}
@@ -23,7 +24,7 @@ func (op *Operator) AutopilotGetConfiguration(args *structs.DCSpecificRequest, r
 		return err
 	}
 	if rule != nil && rule.OperatorRead(nil) != acl.Allow {
-		return acl.ErrPermissionDenied
+		return acl.PermissionDenied("Missing operator:read permissions")
 	}
 
 	state := op.srv.fsm.State()
@@ -55,7 +56,7 @@ func (op *Operator) AutopilotSetConfiguration(args *structs.AutopilotSetConfigRe
 		return err
 	}
 	if rule != nil && rule.OperatorWrite(nil) != acl.Allow {
-		return acl.ErrPermissionDenied
+		return acl.PermissionDenied("Missing operator:write permissions")
 	}
 
 	// Apply the update
@@ -76,7 +77,7 @@ func (op *Operator) AutopilotSetConfiguration(args *structs.AutopilotSetConfigRe
 }
 
 // ServerHealth is used to get the current health of the servers.
-func (op *Operator) ServerHealth(args *structs.DCSpecificRequest, reply *autopilot.OperatorHealthReply) error {
+func (op *Operator) ServerHealth(args *structs.DCSpecificRequest, reply *structs.AutopilotHealthReply) error {
 	// This must be sent to the leader, so we fix the args since we are
 	// re-using a structure where we don't support all the options.
 	args.RequireConsistent = true
@@ -94,19 +95,81 @@ func (op *Operator) ServerHealth(args *structs.DCSpecificRequest, reply *autopil
 		return err
 	}
 	if rule != nil && rule.OperatorRead(nil) != acl.Allow {
-		return acl.ErrPermissionDenied
+		return acl.PermissionDenied("Missing operator:read permissions")
 	}
 
-	// Exit early if the min Raft version is too low
-	minRaftProtocol, err := op.srv.autopilot.MinRaftProtocol()
-	if err != nil {
-		return fmt.Errorf("error getting server raft protocol versions: %s", err)
-	}
-	if minRaftProtocol < 3 {
-		return fmt.Errorf("all servers must have raft_protocol set to 3 or higher to use this endpoint")
+	state := op.srv.autopilot.GetState()
+
+	if state == nil {
+		// this behavior seems odd but its functionally equivalent to 1.8.5 where if
+		// autopilot didn't have a health reply yet it would just return no error
+		return nil
 	}
 
-	*reply = op.srv.autopilot.GetClusterHealth()
+	health := structs.AutopilotHealthReply{
+		Healthy:          state.Healthy,
+		FailureTolerance: state.FailureTolerance,
+	}
 
+	for _, srv := range state.Servers {
+		srvHealth := structs.AutopilotServerHealth{
+			ID:          string(srv.Server.ID),
+			Name:        srv.Server.Name,
+			Address:     string(srv.Server.Address),
+			Version:     srv.Server.Version,
+			Leader:      srv.State == autopilot.RaftLeader,
+			Voter:       srv.State == autopilot.RaftLeader || srv.State == autopilot.RaftVoter,
+			LastContact: srv.Stats.LastContact,
+			LastTerm:    srv.Stats.LastTerm,
+			LastIndex:   srv.Stats.LastIndex,
+			Healthy:     srv.Health.Healthy,
+			StableSince: srv.Health.StableSince,
+		}
+
+		switch srv.Server.NodeStatus {
+		case autopilot.NodeAlive:
+			srvHealth.SerfStatus = serf.StatusAlive
+		case autopilot.NodeLeft:
+			srvHealth.SerfStatus = serf.StatusLeft
+		case autopilot.NodeFailed:
+			srvHealth.SerfStatus = serf.StatusFailed
+		default:
+			srvHealth.SerfStatus = serf.StatusNone
+		}
+
+		health.Servers = append(health.Servers, srvHealth)
+	}
+
+	*reply = health
+	return nil
+}
+
+func (op *Operator) AutopilotState(args *structs.DCSpecificRequest, reply *autopilot.State) error {
+	// This must be sent to the leader, so we fix the args since we are
+	// re-using a structure where we don't support all the options.
+	args.RequireConsistent = true
+	args.AllowStale = false
+	if done, err := op.srv.ForwardRPC("Operator.AutopilotState", args, args, reply); done {
+		return err
+	}
+
+	// This action requires operator read access.
+	identity, rule, err := op.srv.ResolveTokenToIdentityAndAuthorizer(args.Token)
+	if err != nil {
+		return err
+	}
+	if err := op.srv.validateEnterpriseToken(identity); err != nil {
+		return err
+	}
+	if rule != nil && rule.OperatorRead(nil) != acl.Allow {
+		return acl.PermissionDenied("Missing operator:read permissions")
+	}
+
+	state := op.srv.autopilot.GetState()
+	if state == nil {
+		return fmt.Errorf("Failed to get autopilot state: no state found")
+	}
+
+	*reply = *state
 	return nil
 }
