@@ -1,10 +1,6 @@
 package consul
 
 import (
-	"bytes"
-	"crypto/ecdsa"
-	"crypto/elliptic"
-	"crypto/rand"
 	"crypto/x509"
 	"fmt"
 	"net"
@@ -16,10 +12,13 @@ import (
 	"time"
 
 	"github.com/google/tcpproxy"
-	"github.com/hashicorp/consul/agent/connect/ca"
-	"github.com/hashicorp/consul/ipaddr"
+	uuid "github.com/hashicorp/go-uuid"
 	"github.com/hashicorp/memberlist"
 
+	"github.com/hashicorp/consul/ipaddr"
+
+	"golang.org/x/time/rate"
+
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/metadata"
 	"github.com/hashicorp/consul/agent/structs"
@@ -30,9 +29,6 @@ import (
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/hashicorp/consul/tlsutil"
 	"github.com/hashicorp/consul/types"
-	"github.com/hashicorp/go-hclog"
-	"github.com/hashicorp/go-uuid"
-	"golang.org/x/time/rate"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -45,36 +41,24 @@ const (
 // testTLSCertificates Generates a TLS CA and server key/cert and returns them
 // in PEM encoded form.
 func testTLSCertificates(serverName string) (cert string, key string, cacert string, err error) {
-	// generate CA
-	serial, err := tlsutil.GenerateSerialNumber()
-	if err != nil {
-		return "", "", "", err
-	}
-	signer, err := ecdsa.GenerateKey(elliptic.P256(), rand.Reader)
-	if err != nil {
-		return "", "", "", err
-	}
-	ca, err := tlsutil.GenerateCA(signer, serial, 365, nil)
+	signer, _, err := tlsutil.GeneratePrivateKey()
 	if err != nil {
 		return "", "", "", err
 	}
 
-	// generate leaf
-	serial, err = tlsutil.GenerateSerialNumber()
+	ca, _, err := tlsutil.GenerateCA(tlsutil.CAOpts{Signer: signer})
 	if err != nil {
 		return "", "", "", err
 	}
 
-	cert, privateKey, err := tlsutil.GenerateCert(
-		signer,
-		ca,
-		serial,
-		"Test Cert Name",
-		365,
-		[]string{serverName},
-		nil,
-		[]x509.ExtKeyUsage{x509.ExtKeyUsageServerAuth, x509.ExtKeyUsageClientAuth},
-	)
+	cert, privateKey, err := tlsutil.GenerateCert(tlsutil.CertOpts{
+		Signer:      signer,
+		CA:          ca,
+		Name:        "Test Cert Name",
+		Days:        365,
+		DNSNames:    []string{serverName},
+		ExtKeyUsage: []x509.ExtKeyUsage{x509.ExtKeyUsageServerAuth, x509.ExtKeyUsageClientAuth},
+	})
 	if err != nil {
 		return "", "", "", err
 	}
@@ -82,21 +66,12 @@ func testTLSCertificates(serverName string) (cert string, key string, cacert str
 	return cert, privateKey, ca, nil
 }
 
-// testServerACLConfig wraps another arbitrary Config altering callback
-// to setup some common ACL configurations. A new callback func will
-// be returned that has the original callback invoked after setting
-// up all of the ACL configurations (so they can still be overridden)
-func testServerACLConfig(cb func(*Config)) func(*Config) {
-	return func(c *Config) {
-		c.ACLDatacenter = "dc1"
-		c.ACLsEnabled = true
-		c.ACLMasterToken = TestDefaultMasterToken
-		c.ACLDefaultPolicy = "deny"
-
-		if cb != nil {
-			cb(c)
-		}
-	}
+// testServerACLConfig setup some common ACL configurations.
+func testServerACLConfig(c *Config) {
+	c.ACLDatacenter = "dc1"
+	c.ACLsEnabled = true
+	c.ACLMasterToken = TestDefaultMasterToken
+	c.ACLDefaultPolicy = "deny"
 }
 
 func configureTLS(config *Config) {
@@ -129,9 +104,6 @@ func waitForLeaderEstablishment(t *testing.T, servers ...*Server) {
 
 func testServerConfig(t *testing.T) (string, *Config) {
 	dir := testutil.TempDir(t, "consul")
-	t.Cleanup(func() {
-		os.RemoveAll(dir)
-	})
 	config := DefaultConfig()
 
 	ports := freeport.MustTake(3)
@@ -186,8 +158,6 @@ func testServerConfig(t *testing.T) (string, *Config) {
 	config.ServerHealthInterval = 50 * time.Millisecond
 	config.AutopilotInterval = 100 * time.Millisecond
 
-	config.Build = "1.7.2"
-
 	config.CoordinateUpdatePeriod = 100 * time.Millisecond
 	config.LeaveDrainTime = 1 * time.Millisecond
 
@@ -210,13 +180,12 @@ func testServerConfig(t *testing.T) (string, *Config) {
 	return dir, config
 }
 
+// Deprecated: use testServerWithConfig instead. It does the same thing and more.
 func testServer(t *testing.T) (string, *Server) {
-	return testServerWithConfig(t, func(c *Config) {
-		c.Datacenter = "dc1"
-		c.Bootstrap = true
-	})
+	return testServerWithConfig(t)
 }
 
+// Deprecated: use testServerWithConfig
 func testServerDC(t *testing.T, dc string) (string, *Server) {
 	return testServerWithConfig(t, func(c *Config) {
 		c.Datacenter = dc
@@ -224,6 +193,7 @@ func testServerDC(t *testing.T, dc string) (string, *Server) {
 	})
 }
 
+// Deprecated: use testServerWithConfig
 func testServerDCBootstrap(t *testing.T, dc string, bootstrap bool) (string, *Server) {
 	return testServerWithConfig(t, func(c *Config) {
 		c.Datacenter = dc
@@ -231,6 +201,7 @@ func testServerDCBootstrap(t *testing.T, dc string, bootstrap bool) (string, *Se
 	})
 }
 
+// Deprecated: use testServerWithConfig
 func testServerDCExpect(t *testing.T, dc string, expect int) (string, *Server) {
 	return testServerWithConfig(t, func(c *Config) {
 		c.Datacenter = dc
@@ -239,16 +210,7 @@ func testServerDCExpect(t *testing.T, dc string, expect int) (string, *Server) {
 	})
 }
 
-func testServerDCExpectNonVoter(t *testing.T, dc string, expect int) (string, *Server) {
-	return testServerWithConfig(t, func(c *Config) {
-		c.Datacenter = dc
-		c.Bootstrap = false
-		c.BootstrapExpect = expect
-		c.NonVoter = true
-	})
-}
-
-func testServerWithConfig(t *testing.T, cb func(*Config)) (string, *Server) {
+func testServerWithConfig(t *testing.T, configOpts ...func(*Config)) (string, *Server) {
 	var dir string
 	var srv *Server
 
@@ -256,8 +218,8 @@ func testServerWithConfig(t *testing.T, cb func(*Config)) (string, *Server) {
 	retry.RunWith(retry.ThreeTimes(), t, func(r *retry.R) {
 		var config *Config
 		dir, config = testServerConfig(t)
-		if cb != nil {
-			cb(config)
+		for _, fn := range configOpts {
+			fn(config)
 		}
 
 		var err error
@@ -271,8 +233,11 @@ func testServerWithConfig(t *testing.T, cb func(*Config)) (string, *Server) {
 
 // cb is a function that can alter the test servers configuration prior to the server starting.
 func testACLServerWithConfig(t *testing.T, cb func(*Config), initReplicationToken bool) (string, *Server, rpc.ClientCodec) {
-	dir, srv := testServerWithConfig(t, testServerACLConfig(cb))
-	t.Cleanup(func() { srv.Shutdown() })
+	opts := []func(*Config){testServerACLConfig}
+	if cb != nil {
+		opts = append(opts, cb)
+	}
+	dir, srv := testServerWithConfig(t, opts...)
 
 	if initReplicationToken {
 		// setup some tokens here so we get less warnings in the logs
@@ -280,7 +245,6 @@ func testACLServerWithConfig(t *testing.T, cb func(*Config), initReplicationToke
 	}
 
 	codec := rpcClient(t, srv)
-	t.Cleanup(func() { codec.Close() })
 	return dir, srv, codec
 }
 
@@ -295,22 +259,11 @@ func newServer(t *testing.T, c *Config) (*Server, error) {
 		}
 	}
 
-	logger := hclog.NewInterceptLogger(&hclog.LoggerOptions{
-		Name:   c.NodeName,
-		Level:  hclog.Debug,
-		Output: testutil.NewLogBuffer(t),
-	})
-	tlsConf, err := tlsutil.NewConfigurator(c.ToTLSUtilConfig(), logger)
-	if err != nil {
-		return nil, err
-	}
-	srv, err := NewServer(c,
-		WithLogger(logger),
-		WithTokenStore(new(token.Store)),
-		WithTLSConfigurator(tlsConf))
+	srv, err := NewServer(c, newDefaultDeps(t, c))
 	if err != nil {
 		return nil, err
 	}
+	t.Cleanup(func() { srv.Shutdown() })
 
 	// wait until after listen
 	<-up
@@ -645,6 +598,9 @@ func TestServer_JoinWAN_Flood(t *testing.T) {
 
 // This is a mirror of a similar test in agent/agent_test.go
 func TestServer_JoinWAN_viaMeshGateway(t *testing.T) {
+	// if this test is failing because of expired certificates
+	// use the procedure in test/CA-GENERATION.md
+
 	t.Parallel()
 
 	gwPort := freeport.MustTake(1)
@@ -1187,7 +1143,11 @@ func TestServer_AvoidReBootstrap(t *testing.T) {
 
 func TestServer_Expect_NonVoters(t *testing.T) {
 	t.Parallel()
-	dir1, s1 := testServerDCExpectNonVoter(t, "dc1", 2)
+	dir1, s1 := testServerWithConfig(t, func(c *Config) {
+		c.Bootstrap = false
+		c.BootstrapExpect = 2
+		c.ReadReplica = true
+	})
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
 
@@ -1483,42 +1443,6 @@ func TestServer_RPC_RateLimit(t *testing.T) {
 	})
 }
 
-func TestServer_CALogging(t *testing.T) {
-	t.Parallel()
-	_, conf1 := testServerConfig(t)
-
-	// Setup dummy logger to catch output
-	var buf bytes.Buffer
-	logger := testutil.LoggerWithOutput(t, &buf)
-
-	c, err := tlsutil.NewConfigurator(conf1.ToTLSUtilConfig(), logger)
-	require.NoError(t, err)
-
-	s1, err := NewServer(conf1,
-		WithLogger(logger),
-		WithTokenStore(new(token.Store)),
-		WithTLSConfigurator(c))
-	if err != nil {
-		t.Fatalf("err: %v", err)
-	}
-	defer s1.Shutdown()
-	testrpc.WaitForLeader(t, s1.RPC, "dc1")
-
-	if _, ok := s1.caManager.provider.(ca.NeedsLogger); !ok {
-		t.Fatalf("provider does not implement NeedsLogger")
-	}
-
-	// Wait til CA root is setup
-	retry.Run(t, func(r *retry.R) {
-		var out structs.IndexedCARoots
-		r.Check(s1.RPC("ConnectCA.Roots", structs.DCSpecificRequest{
-			Datacenter: conf1.Datacenter,
-		}, &out))
-	})
-
-	require.Contains(t, buf.String(), "consul CA provider configured")
-}
-
 func TestServer_DatacenterJoinAddresses(t *testing.T) {
 	conf := testClusterConfig{
 		Datacenter: "primary",
