@@ -7,9 +7,11 @@ import (
 
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/sdk/testutil"
+
 	"github.com/hashicorp/go-memdb"
-	"github.com/pascaldekloe/goe/verify"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 func TestStore_CAConfig(t *testing.T) {
@@ -60,9 +62,9 @@ func TestStore_CAConfigCAS(t *testing.T) {
 	ok, err := s.CACheckAndSetConfig(2, 0, &structs.CAConfiguration{
 		Provider: "static",
 	})
-	if ok || err != nil {
-		t.Fatalf("expected (false, nil), got: (%v, %#v)", ok, err)
-	}
+
+	require.False(t, ok)
+	testutil.RequireErrorContains(t, err, "ModifyIndex did not match existing")
 
 	// Check that the index is untouched and the entry
 	// has not been updated.
@@ -127,7 +129,7 @@ func TestStore_CAConfig_Snapshot_Restore(t *testing.T) {
 	if err != nil {
 		t.Fatalf("err: %s", err)
 	}
-	verify.Values(t, "", before, snapped)
+	require.Equal(t, snapped, before)
 
 	s2 := testStateStore(t)
 	restore := s2.Restore()
@@ -143,7 +145,7 @@ func TestStore_CAConfig_Snapshot_Restore(t *testing.T) {
 	if idx != 99 {
 		t.Fatalf("bad index: %d", idx)
 	}
-	verify.Values(t, "", before, res)
+	require.Equal(t, res, before)
 }
 
 // Make sure we handle the case of a leftover blank CA config that
@@ -162,7 +164,7 @@ func TestStore_CAConfig_Snapshot_Restore_BlankConfig(t *testing.T) {
 	if err != nil {
 		t.Fatalf("err: %s", err)
 	}
-	verify.Values(t, "", before, snapped)
+	require.Equal(t, snapped, before)
 
 	s2 := testStateStore(t)
 	restore := s2.Restore()
@@ -194,7 +196,7 @@ func TestStore_CARootSetList(t *testing.T) {
 
 	// Build a valid value
 	ca1 := connect.TestCA(t, nil)
-
+	expected := *ca1
 	// Set
 	ok, err := s.CARootSetCAS(1, 0, []*structs.CARoot{ca1})
 	assert.Nil(err)
@@ -205,18 +207,17 @@ func TestStore_CARootSetList(t *testing.T) {
 	assert.True(watchFired(ws), "watch fired")
 
 	// Read it back out and verify it.
-	expected := *ca1
+
 	expected.RaftIndex = structs.RaftIndex{
 		CreateIndex: 1,
 		ModifyIndex: 1,
 	}
-
 	ws = memdb.NewWatchSet()
 	_, roots, err := s.CARoots(ws)
 	assert.Nil(err)
 	assert.Len(roots, 1)
 	actual := roots[0]
-	assert.Equal(&expected, actual)
+	assertDeepEqual(t, expected, *actual)
 }
 
 func TestStore_CARootSet_emptyID(t *testing.T) {
@@ -429,15 +430,15 @@ func TestStore_CABuiltinProvider(t *testing.T) {
 		// Since we've already written to the builtin provider table the serial
 		// numbers will initialize from the max index of the provider table.
 		// That's why this first serial is 2 and not 1.
-		sn, err := s.CAIncrementProviderSerialNumber()
+		sn, err := s.CAIncrementProviderSerialNumber(10)
 		assert.NoError(err)
 		assert.Equal(uint64(2), sn)
 
-		sn, err = s.CAIncrementProviderSerialNumber()
+		sn, err = s.CAIncrementProviderSerialNumber(10)
 		assert.NoError(err)
 		assert.Equal(uint64(3), sn)
 
-		sn, err = s.CAIncrementProviderSerialNumber()
+		sn, err = s.CAIncrementProviderSerialNumber(10)
 		assert.NoError(err)
 		assert.Equal(uint64(4), sn)
 	}
