@@ -4,13 +4,9 @@ package state
 
 import (
 	"fmt"
-
-	"github.com/hashicorp/go-memdb"
 )
 
-func (g *Graveyard) insertTombstoneWithTxn(tx *memdb.Txn,
-	table string, stone *Tombstone, updateMax bool) error {
-
+func (g *Graveyard) insertTombstoneWithTxn(tx WriteTxn, _ string, stone *Tombstone, updateMax bool) error {
 	if err := tx.Insert("tombstones", stone); err != nil {
 		return err
 	}
