@@ -269,7 +269,7 @@ func TestEventList_Blocking(t *testing.T) {
 		time.Sleep(50 * time.Millisecond)
 		p := &UserEvent{Name: "second"}
 		if err := a.UserEvent("dc1", "root", p); err != nil {
-			t.Fatalf("err: %v", err)
+			t.Errorf("err: %v", err)
 		}
 	}()
 
@@ -302,11 +302,11 @@ func TestEventList_EventBufOrder(t *testing.T) {
 	expected := &UserEvent{Name: "foo"}
 
 	for _, e := range []*UserEvent{
-		&UserEvent{Name: "foo"},
-		&UserEvent{Name: "bar"},
-		&UserEvent{Name: "foo"},
+		{Name: "foo"},
+		{Name: "bar"},
+		{Name: "foo"},
 		expected,
-		&UserEvent{Name: "bar"},
+		{Name: "bar"},
 	} {
 		if err := a.UserEvent("dc1", "root", e); err != nil {
 			t.Fatalf("err: %v", err)
