@@ -7,8 +7,9 @@ import (
 	"testing"
 	"time"
 
-	"github.com/hashicorp/consul/testrpc"
 	"github.com/hashicorp/go-hclog"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/agent/config"
@@ -17,9 +18,8 @@ import (
 	"github.com/hashicorp/consul/agent/token"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
+	"github.com/hashicorp/consul/testrpc"
 	"github.com/hashicorp/consul/types"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
 )
 
 func unNilMap(in map[string]string) map[string]string {
@@ -28,6 +28,7 @@ func unNilMap(in map[string]string) map[string]string {
 	}
 	return in
 }
+
 func TestAgentAntiEntropy_Services(t *testing.T) {
 	t.Parallel()
 	a := agent.NewTestAgent(t, "")
@@ -1719,61 +1720,72 @@ func TestAgentAntiEntropy_NodeInfo(t *testing.T) {
 	}
 }
 
-func TestAgent_ServiceTokens(t *testing.T) {
-	t.Parallel()
-
+func TestState_ServiceTokens(t *testing.T) {
 	tokens := new(token.Store)
-	tokens.UpdateUserToken("default", token.TokenSourceConfig)
 	cfg := config.DefaultRuntimeConfig(`bind_addr = "127.0.0.1" data_dir = "dummy"`)
 	l := local.NewState(agent.LocalConfig(cfg), nil, tokens)
 	l.TriggerSyncChanges = func() {}
 
-	l.AddService(&structs.NodeService{ID: "redis"}, "")
+	id := structs.NewServiceID("redis", nil)
 
-	// Returns default when no token is set
-	if token := l.ServiceToken(structs.NewServiceID("redis", nil)); token != "default" {
-		t.Fatalf("bad: %s", token)
-	}
+	t.Run("defaults to empty string", func(t *testing.T) {
+		require.Equal(t, "", l.ServiceToken(id))
+	})
 
-	// Returns configured token
-	l.AddService(&structs.NodeService{ID: "redis"}, "abc123")
-	if token := l.ServiceToken(structs.NewServiceID("redis", nil)); token != "abc123" {
-		t.Fatalf("bad: %s", token)
-	}
+	t.Run("empty string when there is no token", func(t *testing.T) {
+		err := l.AddService(&structs.NodeService{ID: "redis"}, "")
+		require.NoError(t, err)
 
-	// Keeps token around for the delete
-	l.RemoveService(structs.NewServiceID("redis", nil))
-	if token := l.ServiceToken(structs.NewServiceID("redis", nil)); token != "abc123" {
-		t.Fatalf("bad: %s", token)
-	}
+		require.Equal(t, "", l.ServiceToken(id))
+	})
+
+	t.Run("returns configured token", func(t *testing.T) {
+		err := l.AddService(&structs.NodeService{ID: "redis"}, "abc123")
+		require.NoError(t, err)
+
+		require.Equal(t, "abc123", l.ServiceToken(id))
+	})
+
+	t.Run("RemoveCheck keeps token around for the delete", func(t *testing.T) {
+		err := l.RemoveService(structs.NewServiceID("redis", nil))
+		require.NoError(t, err)
+
+		require.Equal(t, "abc123", l.ServiceToken(id))
+	})
 }
 
-func TestAgent_CheckTokens(t *testing.T) {
-	t.Parallel()
-
+func TestState_CheckTokens(t *testing.T) {
 	tokens := new(token.Store)
-	tokens.UpdateUserToken("default", token.TokenSourceConfig)
 	cfg := config.DefaultRuntimeConfig(`bind_addr = "127.0.0.1" data_dir = "dummy"`)
 	l := local.NewState(agent.LocalConfig(cfg), nil, tokens)
 	l.TriggerSyncChanges = func() {}
 
-	// Returns default when no token is set
-	l.AddCheck(&structs.HealthCheck{CheckID: types.CheckID("mem")}, "")
-	if token := l.CheckToken(structs.NewCheckID("mem", nil)); token != "default" {
-		t.Fatalf("bad: %s", token)
-	}
+	id := structs.NewCheckID("mem", nil)
 
-	// Returns configured token
-	l.AddCheck(&structs.HealthCheck{CheckID: types.CheckID("mem")}, "abc123")
-	if token := l.CheckToken(structs.NewCheckID("mem", nil)); token != "abc123" {
-		t.Fatalf("bad: %s", token)
-	}
+	t.Run("defaults to empty string", func(t *testing.T) {
+		require.Equal(t, "", l.CheckToken(id))
+	})
 
-	// Keeps token around for the delete
-	l.RemoveCheck(structs.NewCheckID("mem", nil))
-	if token := l.CheckToken(structs.NewCheckID("mem", nil)); token != "abc123" {
-		t.Fatalf("bad: %s", token)
-	}
+	t.Run("empty string when there is no token", func(t *testing.T) {
+		err := l.AddCheck(&structs.HealthCheck{CheckID: "mem"}, "")
+		require.NoError(t, err)
+
+		require.Equal(t, "", l.CheckToken(id))
+	})
+
+	t.Run("returns configured token", func(t *testing.T) {
+		err := l.AddCheck(&structs.HealthCheck{CheckID: "mem"}, "abc123")
+		require.NoError(t, err)
+
+		require.Equal(t, "abc123", l.CheckToken(id))
+	})
+
+	t.Run("RemoveCheck keeps token around for the delete", func(t *testing.T) {
+		err := l.RemoveCheck(structs.NewCheckID("mem", nil))
+		require.NoError(t, err)
+
+		require.Equal(t, "abc123", l.CheckToken(id))
+	})
 }
 
 func TestAgent_CheckCriticalTime(t *testing.T) {
@@ -2195,3 +2207,52 @@ func drainCh(ch chan struct{}) {
 		}
 	}
 }
+
+func TestState_SyncChanges_DuplicateAddServiceOnlySyncsOnce(t *testing.T) {
+	state := local.NewState(local.Config{}, hclog.New(nil), new(token.Store))
+	rpc := &fakeRPC{}
+	state.Delegate = rpc
+	state.TriggerSyncChanges = func() {}
+
+	srv := &structs.NodeService{
+		Kind:           structs.ServiceKindTypical,
+		ID:             "the-service-id",
+		Service:        "web",
+		EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+	}
+	checks := []*structs.HealthCheck{
+		{Node: "this-node", CheckID: "the-id-1", Name: "check-healthy-1"},
+		{Node: "this-node", CheckID: "the-id-2", Name: "check-healthy-2"},
+	}
+	tok := "the-token"
+	err := state.AddServiceWithChecks(srv, checks, tok)
+	require.NoError(t, err)
+	require.NoError(t, state.SyncChanges())
+	// 4 rpc calls, one node register, one service register, two checks
+	require.Len(t, rpc.calls, 4)
+
+	// adding the service again should not catalog register
+	err = state.AddServiceWithChecks(srv, checks, tok)
+	require.NoError(t, err)
+	require.NoError(t, state.SyncChanges())
+	require.Len(t, rpc.calls, 4)
+}
+
+type fakeRPC struct {
+	calls []callRPC
+}
+
+type callRPC struct {
+	method string
+	args   interface{}
+	reply  interface{}
+}
+
+func (f *fakeRPC) RPC(method string, args interface{}, reply interface{}) error {
+	f.calls = append(f.calls, callRPC{method: method, args: args, reply: reply})
+	return nil
+}
+
+func (f *fakeRPC) ResolveTokenToIdentity(_ string) (structs.ACLIdentity, error) {
+	return nil, nil
+}
