@@ -7,11 +7,13 @@ import (
 	"net/http/httptest"
 	"strings"
 	"testing"
+	"time"
 
 	"github.com/hashicorp/consul/testrpc"
+	"github.com/hashicorp/raft"
+	autopilot "github.com/hashicorp/raft-autopilot"
 	"github.com/stretchr/testify/require"
 
-	"github.com/hashicorp/consul/agent/consul/autopilot"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
@@ -403,7 +405,7 @@ func TestOperator_AutopilotSetConfiguration(t *testing.T) {
 		Datacenter: "dc1",
 	}
 
-	var reply autopilot.Config
+	var reply structs.AutopilotConfig
 	if err := a.RPC("Operator.AutopilotGetConfiguration", &args, &reply); err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -432,7 +434,7 @@ func TestOperator_AutopilotCASConfiguration(t *testing.T) {
 		Datacenter: "dc1",
 	}
 
-	var reply autopilot.Config
+	var reply structs.AutopilotConfig
 	if err := a.RPC("Operator.AutopilotGetConfiguration", &args, &reply); err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -544,3 +546,179 @@ func TestOperator_ServerHealth_Unhealthy(t *testing.T) {
 		}
 	})
 }
+
+func TestOperator_AutopilotState(t *testing.T) {
+	a := NewTestAgent(t, "")
+	defer a.Shutdown()
+
+	req, err := http.NewRequest("GET", "/v1/operator/autopilot/state", nil)
+	require.NoError(t, err)
+	retry.Run(t, func(r *retry.R) {
+		resp := httptest.NewRecorder()
+		obj, err := a.srv.OperatorAutopilotState(resp, req)
+		require.NoError(r, err)
+		require.Equal(r, 200, resp.Code)
+		state, ok := obj.(*api.AutopilotState)
+		require.True(r, ok)
+
+		srv, ok := state.Servers[string(a.config.NodeID)]
+		require.True(r, ok)
+		require.True(r, srv.Healthy)
+		require.Equal(r, a.config.NodeName, srv.Name)
+
+	})
+}
+
+func TestAutopilotStateToAPIConversion(t *testing.T) {
+	var leaderID raft.ServerID = "79324811-9588-4311-b208-f272e38aaabf"
+	var follower1ID raft.ServerID = "ef8aee9a-f9d6-4ec4-b383-aac956bdb80f"
+	var follower2ID raft.ServerID = "ae84aefb-a303-4734-8739-5c102d4ee2d9"
+	input := autopilot.State{
+		Healthy:          true,
+		FailureTolerance: 1,
+		Leader:           leaderID,
+		Voters: []raft.ServerID{
+			leaderID,
+			follower1ID,
+			follower2ID,
+		},
+		Servers: map[raft.ServerID]*autopilot.ServerState{
+			leaderID: {
+				Server: autopilot.Server{
+					ID:         leaderID,
+					Name:       "node1",
+					Address:    "198.18.0.1:8300",
+					NodeStatus: autopilot.NodeAlive,
+					Version:    "1.9.0",
+					Meta: map[string]string{
+						"foo": "bar",
+					},
+					NodeType: autopilot.NodeVoter,
+				},
+				State: autopilot.RaftLeader,
+				Stats: autopilot.ServerStats{
+					LastContact: 0,
+					LastTerm:    3,
+					LastIndex:   42,
+				},
+				Health: autopilot.ServerHealth{
+					Healthy:     true,
+					StableSince: time.Date(2020, 11, 6, 14, 51, 0, 0, time.UTC),
+				},
+			},
+			follower1ID: {
+				Server: autopilot.Server{
+					ID:         follower1ID,
+					Name:       "node2",
+					Address:    "198.18.0.2:8300",
+					NodeStatus: autopilot.NodeAlive,
+					Version:    "1.9.0",
+					Meta: map[string]string{
+						"bar": "baz",
+					},
+					NodeType: autopilot.NodeVoter,
+				},
+				State: autopilot.RaftVoter,
+				Stats: autopilot.ServerStats{
+					LastContact: time.Millisecond,
+					LastTerm:    3,
+					LastIndex:   41,
+				},
+				Health: autopilot.ServerHealth{
+					Healthy:     true,
+					StableSince: time.Date(2020, 11, 6, 14, 52, 0, 0, time.UTC),
+				},
+			},
+			follower2ID: {
+				Server: autopilot.Server{
+					ID:         follower2ID,
+					Name:       "node3",
+					Address:    "198.18.0.3:8300",
+					NodeStatus: autopilot.NodeAlive,
+					Version:    "1.9.0",
+					Meta: map[string]string{
+						"baz": "foo",
+					},
+					NodeType: autopilot.NodeVoter,
+				},
+				State: autopilot.RaftVoter,
+				Stats: autopilot.ServerStats{
+					LastContact: 2 * time.Millisecond,
+					LastTerm:    3,
+					LastIndex:   39,
+				},
+				Health: autopilot.ServerHealth{
+					Healthy:     true,
+					StableSince: time.Date(2020, 11, 6, 14, 53, 0, 0, time.UTC),
+				},
+			},
+		},
+	}
+
+	expected := api.AutopilotState{
+		Healthy:                    true,
+		FailureTolerance:           1,
+		OptimisticFailureTolerance: 1,
+		Leader:                     string(leaderID),
+		Voters: []string{
+			string(leaderID),
+			string(follower1ID),
+			string(follower2ID),
+		},
+		Servers: map[string]api.AutopilotServer{
+			string(leaderID): {
+				ID:         string(leaderID),
+				Name:       "node1",
+				Address:    "198.18.0.1:8300",
+				NodeStatus: "alive",
+				Version:    "1.9.0",
+				Meta: map[string]string{
+					"foo": "bar",
+				},
+				NodeType:    api.AutopilotTypeVoter,
+				Status:      api.AutopilotServerLeader,
+				LastContact: api.NewReadableDuration(0),
+				LastTerm:    3,
+				LastIndex:   42,
+				Healthy:     true,
+				StableSince: time.Date(2020, 11, 6, 14, 51, 0, 0, time.UTC),
+			},
+			string(follower1ID): {
+				ID:         string(follower1ID),
+				Name:       "node2",
+				Address:    "198.18.0.2:8300",
+				NodeStatus: "alive",
+				Version:    "1.9.0",
+				Meta: map[string]string{
+					"bar": "baz",
+				},
+				NodeType:    api.AutopilotTypeVoter,
+				Status:      api.AutopilotServerVoter,
+				LastContact: api.NewReadableDuration(time.Millisecond),
+				LastTerm:    3,
+				LastIndex:   41,
+				Healthy:     true,
+				StableSince: time.Date(2020, 11, 6, 14, 52, 0, 0, time.UTC),
+			},
+			string(follower2ID): {
+				ID:         string(follower2ID),
+				Name:       "node3",
+				Address:    "198.18.0.3:8300",
+				NodeStatus: "alive",
+				Version:    "1.9.0",
+				Meta: map[string]string{
+					"baz": "foo",
+				},
+				NodeType:    api.AutopilotTypeVoter,
+				Status:      api.AutopilotServerVoter,
+				LastContact: api.NewReadableDuration(2 * time.Millisecond),
+				LastTerm:    3,
+				LastIndex:   39,
+				Healthy:     true,
+				StableSince: time.Date(2020, 11, 6, 14, 53, 0, 0, time.UTC),
+			},
+		},
+	}
+
+	require.Equal(t, &expected, autopilotToAPIState(&input))
+}
