@@ -223,6 +223,17 @@ func genVerifyResolverWatch(expectedService, expectedDatacenter, expectedKind st
 	}
 }
 
+func genVerifyResolvedConfigWatch(expectedService string, expectedDatacenter string) verifyWatchRequest {
+	return func(t testing.TB, cacheType string, request cache.Request) {
+		require.Equal(t, cachetype.ResolvedServiceConfigName, cacheType)
+
+		reqReal, ok := request.(*structs.ServiceConfigRequest)
+		require.True(t, ok)
+		require.Equal(t, expectedDatacenter, reqReal.Datacenter)
+		require.Equal(t, expectedService, reqReal.Name)
+	}
+}
+
 func genVerifyIntentionWatch(expectedService string, expectedDatacenter string) verifyWatchRequest {
 	return func(t testing.TB, cacheType string, request cache.Request) {
 		require.Equal(t, cachetype.IntentionMatchName, cacheType)
@@ -422,6 +433,8 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 			ns.Proxy.MeshGateway.Mode = meshGatewayProxyConfigValue
 		}
 
+		ixnMatch := TestIntentions()
+
 		stage0 := verificationStage{
 			requiredWatches: map[string]verifyWatchRequest{
 				rootsWatchID:                    genVerifyRootsWatch("dc1"),
@@ -476,12 +489,17 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 			},
 			events: []cache.UpdateEvent{
 				rootWatchEvent(),
-				cache.UpdateEvent{
+				{
 					CorrelationID: leafWatchID,
 					Result:        issuedCert,
 					Err:           nil,
 				},
-				cache.UpdateEvent{
+				{
+					CorrelationID: intentionsWatchID,
+					Result:        ixnMatch,
+					Err:           nil,
+				},
+				{
 					CorrelationID: "discovery-chain:api",
 					Result: &structs.DiscoveryChainResponse{
 						Chain: discoverychain.TestCompileConfigEntries(t, "api", "default", "dc1", "trustdomain.consul", "dc1",
@@ -491,7 +509,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 					},
 					Err: nil,
 				},
-				cache.UpdateEvent{
+				{
 					CorrelationID: "discovery-chain:api-failover-remote?dc=dc2",
 					Result: &structs.DiscoveryChainResponse{
 						Chain: discoverychain.TestCompileConfigEntries(t, "api-failover-remote", "default", "dc2", "trustdomain.consul", "dc1",
@@ -501,7 +519,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 					},
 					Err: nil,
 				},
-				cache.UpdateEvent{
+				{
 					CorrelationID: "discovery-chain:api-failover-local?dc=dc2",
 					Result: &structs.DiscoveryChainResponse{
 						Chain: discoverychain.TestCompileConfigEntries(t, "api-failover-local", "default", "dc2", "trustdomain.consul", "dc1",
@@ -511,7 +529,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 					},
 					Err: nil,
 				},
-				cache.UpdateEvent{
+				{
 					CorrelationID: "discovery-chain:api-failover-direct?dc=dc2",
 					Result: &structs.DiscoveryChainResponse{
 						Chain: discoverychain.TestCompileConfigEntries(t, "api-failover-direct", "default", "dc2", "trustdomain.consul", "dc1",
@@ -521,7 +539,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 					},
 					Err: nil,
 				},
-				cache.UpdateEvent{
+				{
 					CorrelationID: "discovery-chain:api-dc2",
 					Result: &structs.DiscoveryChainResponse{
 						Chain: discoverychain.TestCompileConfigEntries(t, "api-dc2", "default", "dc1", "trustdomain.consul", "dc1",
@@ -555,6 +573,9 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 
 				require.Len(t, snap.ConnectProxy.WatchedServiceChecks, 0, "%+v", snap.ConnectProxy.WatchedServiceChecks)
 				require.Len(t, snap.ConnectProxy.PreparedQueryEndpoints, 0, "%+v", snap.ConnectProxy.PreparedQueryEndpoints)
+
+				require.True(t, snap.ConnectProxy.IntentionsSet)
+				require.Equal(t, ixnMatch.Matches[0], snap.ConnectProxy.Intentions)
 			},
 		}
 
@@ -581,6 +602,9 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 
 				require.Len(t, snap.ConnectProxy.WatchedServiceChecks, 0, "%+v", snap.ConnectProxy.WatchedServiceChecks)
 				require.Len(t, snap.ConnectProxy.PreparedQueryEndpoints, 0, "%+v", snap.ConnectProxy.PreparedQueryEndpoints)
+
+				require.True(t, snap.ConnectProxy.IntentionsSet)
+				require.Equal(t, ixnMatch.Matches[0], snap.ConnectProxy.Intentions)
 			},
 		}
 
@@ -599,11 +623,48 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 	db := structs.NewServiceName("db", nil)
 	dbStr := db.String()
 
+	billing := structs.NewServiceName("billing", nil)
+
 	api := structs.NewServiceName("api", nil)
 	apiStr := api.String()
 
+	dbIxnMatch := &structs.IndexedIntentionMatches{
+		Matches: []structs.Intentions{
+			[]*structs.Intention{
+				{
+					ID:              "abc-123",
+					SourceNS:        "default",
+					SourceName:      "api",
+					DestinationNS:   "default",
+					DestinationName: "db",
+					Action:          structs.IntentionActionAllow,
+				},
+			},
+		},
+	}
+
+	dbConfig := &structs.ServiceConfigResponse{
+		ProxyConfig: map[string]interface{}{
+			"protocol": "grpc",
+		},
+	}
+
+	dbResolver := &structs.IndexedConfigEntries{
+		Kind: structs.ServiceResolver,
+		Entries: []structs.ConfigEntry{
+			&structs.ServiceResolverConfigEntry{
+				Name: "db",
+				Kind: structs.ServiceResolver,
+				Redirect: &structs.ServiceResolverRedirect{
+					Service:    "db",
+					Datacenter: "dc2",
+				},
+			},
+		},
+	}
+
 	cases := map[string]testCase{
-		"initial-gateway": testCase{
+		"initial-gateway": {
 			ns: structs.NodeService{
 				Kind:    structs.ServiceKindMeshGateway,
 				ID:      "mesh-gateway",
@@ -613,7 +674,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 			},
 			sourceDC: "dc1",
 			stages: []verificationStage{
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						rootsWatchID:       genVerifyRootsWatch("dc1"),
 						serviceListWatchID: genVerifyListServicesWatch("dc1"),
@@ -624,7 +685,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.True(t, snap.ConnectProxy.IsEmpty())
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
 						rootWatchEvent(),
 					},
@@ -640,9 +701,9 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Empty(t, snap.MeshGateway.GatewayGroups)
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: serviceListWatchID,
 							Result: &structs.IndexedServiceList{
 								Services: make(structs.ServiceList, 0),
@@ -664,7 +725,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 				},
 			},
 		},
-		"mesh-gateway-do-not-cancel-service-watches": testCase{
+		"mesh-gateway-do-not-cancel-service-watches": {
 			ns: structs.NodeService{
 				Kind:    structs.ServiceKindMeshGateway,
 				ID:      "mesh-gateway",
@@ -674,7 +735,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 			},
 			sourceDC: "dc1",
 			stages: []verificationStage{
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						rootsWatchID:       genVerifyRootsWatch("dc1"),
 						serviceListWatchID: genVerifyListServicesWatch("dc1"),
@@ -682,7 +743,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 					},
 					events: []cache.UpdateEvent{
 						rootWatchEvent(),
-						cache.UpdateEvent{
+						{
 							CorrelationID: serviceListWatchID,
 							Result: &structs.IndexedServiceList{
 								Services: structs.ServiceList{
@@ -698,9 +759,9 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.True(t, snap.MeshGateway.WatchedServicesSet)
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: serviceListWatchID,
 							Result: &structs.IndexedServiceList{
 								Services: structs.ServiceList{
@@ -717,11 +778,11 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.True(t, snap.MeshGateway.WatchedServicesSet)
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: "mesh-gateway:dc4",
-							Result: &structs.IndexedCheckServiceNodes{
+							Result: &structs.IndexedNodesWithGateways{
 								Nodes: TestGatewayNodesDC4Hostname(t),
 							},
 							Err: nil,
@@ -761,9 +822,9 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Equal(t, snap.MeshGateway.HostnameDatacenters["dc4"], expect)
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: federationStateListGatewaysWatchID,
 							Result: &structs.DatacenterIndexedCheckServiceNodes{
 								DatacenterNodes: map[string]structs.CheckServiceNodes{
@@ -809,7 +870,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 				},
 			},
 		},
-		"ingress-gateway": testCase{
+		"ingress-gateway": {
 			ns: structs.NodeService{
 				Kind:    structs.ServiceKindIngressGateway,
 				ID:      "ingress-gateway",
@@ -818,7 +879,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 			},
 			sourceDC: "dc1",
 			stages: []verificationStage{
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						rootsWatchID:           genVerifyRootsWatch("dc1"),
 						gatewayConfigWatchID:   genVerifyConfigEntryWatch(structs.IngressGateway, "ingress-gateway", "dc1"),
@@ -829,7 +890,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.True(t, snap.IngressGateway.IsEmpty())
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
 						rootWatchEvent(),
 					},
@@ -838,7 +899,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Equal(t, indexedRoots, snap.Roots)
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
 						ingressConfigWatchEvent(false),
 					},
@@ -848,9 +909,9 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.False(t, snap.IngressGateway.TLSEnabled)
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: gatewayServicesWatchID,
 							Result: &structs.IndexedGatewayServices{
 								Services: structs.GatewayServices{
@@ -885,12 +946,12 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Contains(t, snap.IngressGateway.WatchedDiscoveryChains, "api")
 					},
 				},
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						leafWatchID: genVerifyLeafWatch("ingress-gateway", "dc1"),
 					},
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: leafWatchID,
 							Result:        issuedCert,
 							Err:           nil,
@@ -901,7 +962,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Equal(t, issuedCert, snap.IngressGateway.Leaf)
 					},
 				},
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						"discovery-chain:api": genVerifyDiscoveryChainWatch(&structs.DiscoveryChainRequest{
 							Name:                 "api",
@@ -911,7 +972,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						}),
 					},
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: "discovery-chain:api",
 							Result: &structs.DiscoveryChainResponse{
 								Chain: discoverychain.TestCompileConfigEntries(t, "api", "default", "dc1", "trustdomain.consul", "dc1", nil),
@@ -924,12 +985,12 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Len(t, snap.IngressGateway.WatchedUpstreams["api"], 1)
 					},
 				},
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						"upstream-target:api.default.dc1:api": genVerifyServiceWatch("api", "", "dc1", true),
 					},
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: "upstream-target:api.default.dc1:api",
 							Result: &structs.IndexedCheckServiceNodes{
 								Nodes: structs.CheckServiceNodes{
@@ -971,7 +1032,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 				},
 			},
 		},
-		"ingress-gateway-with-tls-update-upstreams": testCase{
+		"ingress-gateway-with-tls-update-upstreams": {
 			ns: structs.NodeService{
 				Kind:    structs.ServiceKindIngressGateway,
 				ID:      "ingress-gateway",
@@ -980,7 +1041,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 			},
 			sourceDC: "dc1",
 			stages: []verificationStage{
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						rootsWatchID:           genVerifyRootsWatch("dc1"),
 						gatewayConfigWatchID:   genVerifyConfigEntryWatch(structs.IngressGateway, "ingress-gateway", "dc1"),
@@ -989,7 +1050,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 					events: []cache.UpdateEvent{
 						rootWatchEvent(),
 						ingressConfigWatchEvent(true),
-						cache.UpdateEvent{
+						{
 							CorrelationID: gatewayServicesWatchID,
 							Result: &structs.IndexedGatewayServices{
 								Services: structs.GatewayServices{
@@ -1003,7 +1064,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 							},
 							Err: nil,
 						},
-						cache.UpdateEvent{
+						{
 							CorrelationID: leafWatchID,
 							Result:        issuedCert,
 							Err:           nil,
@@ -1020,7 +1081,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Contains(t, snap.IngressGateway.WatchedDiscoveryChains, "api")
 					},
 				},
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						leafWatchID: genVerifyLeafWatchWithDNSSANs("ingress-gateway", "dc1", []string{
 							"test.example.com",
@@ -1031,7 +1092,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						}),
 					},
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: gatewayServicesWatchID,
 							Result:        &structs.IndexedGatewayServices{},
 							Err:           nil,
@@ -1046,7 +1107,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 				},
 			},
 		},
-		"terminating-gateway-initial": testCase{
+		"terminating-gateway-initial": {
 			ns: structs.NodeService{
 				Kind:    structs.ServiceKindTerminatingGateway,
 				ID:      "terminating-gateway",
@@ -1055,7 +1116,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 			},
 			sourceDC: "dc1",
 			stages: []verificationStage{
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						rootsWatchID: genVerifyRootsWatch("dc1"),
 						gatewayServicesWatchID: genVerifyServiceSpecificRequest(gatewayServicesWatchID,
@@ -1068,7 +1129,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.True(t, snap.IngressGateway.IsEmpty())
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
 						rootWatchEvent(),
 					},
@@ -1083,7 +1144,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 				},
 			},
 		},
-		"terminating-gateway-handle-update": testCase{
+		"terminating-gateway-handle-update": {
 			ns: structs.NodeService{
 				Kind:    structs.ServiceKindTerminatingGateway,
 				ID:      "terminating-gateway",
@@ -1092,7 +1153,7 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 			},
 			sourceDC: "dc1",
 			stages: []verificationStage{
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						rootsWatchID: genVerifyRootsWatch("dc1"),
 						gatewayServicesWatchID: genVerifyServiceSpecificRequest(gatewayServicesWatchID,
@@ -1100,12 +1161,12 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 					},
 					events: []cache.UpdateEvent{
 						rootWatchEvent(),
-						cache.UpdateEvent{
+						{
 							CorrelationID: gatewayServicesWatchID,
 							Result: &structs.IndexedGatewayServices{
 								Services: structs.GatewayServices{
 									{
-										Service: structs.NewServiceName("db", nil),
+										Service: db,
 										Gateway: structs.NewServiceName("terminating-gateway", nil),
 									},
 								},
@@ -1115,25 +1176,28 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 					},
 					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
 						require.True(t, snap.Valid(), "gateway with service list is valid")
+						require.Len(t, snap.TerminatingGateway.ValidServices(), 0)
+
 						require.Len(t, snap.TerminatingGateway.WatchedServices, 1)
+						require.Contains(t, snap.TerminatingGateway.WatchedServices, db)
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: gatewayServicesWatchID,
 							Result: &structs.IndexedGatewayServices{
 								Services: structs.GatewayServices{
 									{
-										Service: structs.NewServiceName("db", nil),
+										Service: db,
 										Gateway: structs.NewServiceName("terminating-gateway", nil),
 									},
 									{
-										Service: structs.NewServiceName("billing", nil),
+										Service: billing,
 										Gateway: structs.NewServiceName("terminating-gateway", nil),
 									},
 									{
-										Service: structs.NewServiceName("api", nil),
+										Service: api,
 										Gateway: structs.NewServiceName("terminating-gateway", nil),
 									},
 								},
@@ -1142,11 +1206,9 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						},
 					},
 					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
-						db := structs.NewServiceName("db", nil)
-						billing := structs.NewServiceName("billing", nil)
-						api := structs.NewServiceName("api", nil)
-
 						require.True(t, snap.Valid(), "gateway with service list is valid")
+						require.Len(t, snap.TerminatingGateway.ValidServices(), 0)
+
 						require.Len(t, snap.TerminatingGateway.WatchedServices, 3)
 						require.Contains(t, snap.TerminatingGateway.WatchedServices, db)
 						require.Contains(t, snap.TerminatingGateway.WatchedServices, billing)
@@ -1162,6 +1224,11 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Contains(t, snap.TerminatingGateway.WatchedLeaves, billing)
 						require.Contains(t, snap.TerminatingGateway.WatchedLeaves, api)
 
+						require.Len(t, snap.TerminatingGateway.WatchedConfigs, 3)
+						require.Contains(t, snap.TerminatingGateway.WatchedConfigs, db)
+						require.Contains(t, snap.TerminatingGateway.WatchedConfigs, billing)
+						require.Contains(t, snap.TerminatingGateway.WatchedConfigs, api)
+
 						require.Len(t, snap.TerminatingGateway.WatchedResolvers, 3)
 						require.Contains(t, snap.TerminatingGateway.WatchedResolvers, db)
 						require.Contains(t, snap.TerminatingGateway.WatchedResolvers, billing)
@@ -1173,12 +1240,12 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						require.Contains(t, snap.TerminatingGateway.GatewayServices, api)
 					},
 				},
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						"external-service:" + dbStr: genVerifyServiceWatch("db", "", "dc1", false),
 					},
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: "external-service:" + dbStr,
 							Result: &structs.IndexedCheckServiceNodes{
 								Nodes: structs.CheckServiceNodes{
@@ -1198,8 +1265,11 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						},
 					},
 					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
+						require.True(t, snap.Valid(), "gateway with service list is valid")
+						require.Len(t, snap.TerminatingGateway.ValidServices(), 0)
+
 						require.Len(t, snap.TerminatingGateway.ServiceGroups, 1)
-						require.Equal(t, snap.TerminatingGateway.ServiceGroups[structs.NewServiceName("db", nil)],
+						require.Equal(t, snap.TerminatingGateway.ServiceGroups[db],
 							structs.CheckServiceNodes{
 								{
 									Node: &structs.Node{
@@ -1215,12 +1285,12 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						)
 					},
 				},
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						"external-service:" + apiStr: genVerifyServiceWatch("api", "", "dc1", false),
 					},
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: "external-service:" + apiStr,
 							Result: &structs.IndexedCheckServiceNodes{
 								Nodes: structs.CheckServiceNodes{
@@ -1263,6 +1333,9 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						},
 					},
 					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
+						require.True(t, snap.Valid(), "gateway with service list is valid")
+						require.Len(t, snap.TerminatingGateway.ValidServices(), 0)
+
 						require.Len(t, snap.TerminatingGateway.ServiceGroups, 2)
 						expect := structs.CheckServiceNodes{
 							{
@@ -1299,71 +1372,101 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 								},
 							},
 						}
-						sn := structs.NewServiceName("api", nil)
-						require.Equal(t, snap.TerminatingGateway.ServiceGroups[sn], expect)
+						require.Equal(t, snap.TerminatingGateway.ServiceGroups[api], expect)
 
 						// The instance in node3 should not be present in HostnameDatacenters because it has a valid IP
-						require.ElementsMatch(t, snap.TerminatingGateway.HostnameServices[sn], expect[:2])
+						require.ElementsMatch(t, snap.TerminatingGateway.HostnameServices[api], expect[:2])
 					},
 				},
-				verificationStage{
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						"service-leaf:" + dbStr: genVerifyLeafWatch("db", "dc1"),
 					},
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: "service-leaf:" + dbStr,
 							Result:        issuedCert,
 							Err:           nil,
 						},
 					},
 					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
-						require.Equal(t, snap.TerminatingGateway.ServiceLeaves[structs.NewServiceName("db", nil)], issuedCert)
+						require.True(t, snap.Valid(), "gateway with service list is valid")
+						require.Len(t, snap.TerminatingGateway.ValidServices(), 0)
+
+						require.Equal(t, snap.TerminatingGateway.ServiceLeaves[db], issuedCert)
 					},
 				},
-				verificationStage{
+				{
+					requiredWatches: map[string]verifyWatchRequest{
+						serviceIntentionsIDPrefix + dbStr: genVerifyIntentionWatch("db", "dc1"),
+					},
+					events: []cache.UpdateEvent{
+						{
+							CorrelationID: serviceIntentionsIDPrefix + dbStr,
+							Result:        dbIxnMatch,
+							Err:           nil,
+						},
+					},
+					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
+						require.True(t, snap.Valid(), "gateway with service list is valid")
+						require.Len(t, snap.TerminatingGateway.ValidServices(), 0)
+
+						require.Len(t, snap.TerminatingGateway.Intentions, 1)
+						dbIxn, ok := snap.TerminatingGateway.Intentions[db]
+						require.True(t, ok)
+						require.Equal(t, dbIxnMatch.Matches[0], dbIxn)
+					},
+				},
+				{
+					requiredWatches: map[string]verifyWatchRequest{
+						serviceConfigIDPrefix + dbStr: genVerifyResolvedConfigWatch("db", "dc1"),
+					},
+					events: []cache.UpdateEvent{
+						{
+							CorrelationID: serviceConfigIDPrefix + dbStr,
+							Result:        dbConfig,
+							Err:           nil,
+						},
+					},
+					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
+						require.True(t, snap.Valid(), "gateway with service list is valid")
+						require.Len(t, snap.TerminatingGateway.ValidServices(), 0)
+
+						require.Len(t, snap.TerminatingGateway.ServiceConfigs, 1)
+						require.Equal(t, snap.TerminatingGateway.ServiceConfigs[db], dbConfig)
+					},
+				},
+				{
 					requiredWatches: map[string]verifyWatchRequest{
 						"service-resolver:" + dbStr: genVerifyResolverWatch("db", "dc1", structs.ServiceResolver),
 					},
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: "service-resolver:" + dbStr,
-							Result: &structs.IndexedConfigEntries{
-								Kind: structs.ServiceResolver,
-								Entries: []structs.ConfigEntry{
-									&structs.ServiceResolverConfigEntry{
-										Name: "db",
-										Kind: structs.ServiceResolver,
-										Redirect: &structs.ServiceResolverRedirect{
-											Service:    "db",
-											Datacenter: "dc2",
-										},
-									},
-								},
-							},
-							Err: nil,
+							Result:        dbResolver,
+							Err:           nil,
 						},
 					},
 					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
-						want := &structs.ServiceResolverConfigEntry{
-							Kind: structs.ServiceResolver,
-							Name: "db",
-							Redirect: &structs.ServiceResolverRedirect{
-								Service:    "db",
-								Datacenter: "dc2",
-							},
-						}
-						require.Equal(t, want, snap.TerminatingGateway.ServiceResolvers[structs.NewServiceName("db", nil)])
+						require.True(t, snap.Valid(), "gateway with service list is valid")
+						// Finally we have everything we need
+						require.Equal(t, []structs.ServiceName{db}, snap.TerminatingGateway.ValidServices())
+
+						require.Len(t, snap.TerminatingGateway.ServiceResolversSet, 1)
+						require.True(t, snap.TerminatingGateway.ServiceResolversSet[db])
+
+						require.Len(t, snap.TerminatingGateway.ServiceResolvers, 1)
+						require.Equal(t, dbResolver.Entries[0], snap.TerminatingGateway.ServiceResolvers[db])
 					},
 				},
-				verificationStage{
+				{
 					events: []cache.UpdateEvent{
-						cache.UpdateEvent{
+						{
 							CorrelationID: gatewayServicesWatchID,
 							Result: &structs.IndexedGatewayServices{
 								Services: structs.GatewayServices{
 									{
-										Service: structs.NewServiceName("billing", nil),
+										Service: billing,
 										Gateway: structs.NewServiceName("terminating-gateway", nil),
 									},
 								},
@@ -1372,9 +1475,8 @@ func TestState_WatchesAndUpdates(t *testing.T) {
 						},
 					},
 					verifySnapshot: func(t testing.TB, snap *ConfigSnapshot) {
-						billing := structs.NewServiceName("billing", nil)
-
 						require.True(t, snap.Valid(), "gateway with service list is valid")
+						require.Len(t, snap.TerminatingGateway.ValidServices(), 0)
 
 						// All the watches should have been cancelled for db
 						require.Len(t, snap.TerminatingGateway.WatchedServices, 1)
