@@ -4,6 +4,7 @@ import (
 	"context"
 	"fmt"
 	"io/ioutil"
+	"math"
 	"path"
 	"path/filepath"
 	"sync"
@@ -99,11 +100,11 @@ func TestLeafForCA(t testing.T, ca *structs.CARoot) *structs.IssuedCert {
 
 // TestIntentions returns a sample intentions match result useful to
 // mocking service discovery cache results.
-func TestIntentions(t testing.T) *structs.IndexedIntentionMatches {
+func TestIntentions() *structs.IndexedIntentionMatches {
 	return &structs.IndexedIntentionMatches{
 		Matches: []structs.Intentions{
 			[]*structs.Intention{
-				&structs.Intention{
+				{
 					ID:              "foo",
 					SourceNS:        "default",
 					SourceName:      "billing",
@@ -677,7 +678,7 @@ func TestConfigSnapshot(t testing.T) *ConfigSnapshot {
 					"db": dbChain,
 				},
 				WatchedUpstreamEndpoints: map[string]map[string]structs.CheckServiceNodes{
-					"db": map[string]structs.CheckServiceNodes{
+					"db": {
 						"db.default.dc1": TestUpstreamNodes(t),
 					},
 				},
@@ -685,6 +686,8 @@ func TestConfigSnapshot(t testing.T) *ConfigSnapshot {
 			PreparedQueryEndpoints: map[string]structs.CheckServiceNodes{
 				"prepared_query:geo-cache": TestUpstreamNodes(t),
 			},
+			Intentions:    nil, // no intentions defined
+			IntentionsSet: true,
 		},
 		Datacenter: "dc1",
 	}
@@ -763,6 +766,10 @@ func TestConfigSnapshotDiscoveryChainWithRouter(t testing.T) *ConfigSnapshot {
 	return testConfigSnapshotDiscoveryChain(t, "chain-and-router")
 }
 
+func TestConfigSnapshotDiscoveryChainWithLB(t testing.T) *ConfigSnapshot {
+	return testConfigSnapshotDiscoveryChain(t, "lb-resolver")
+}
+
 func testConfigSnapshotDiscoveryChain(t testing.T, variation string, additionalEntries ...structs.ConfigEntry) *ConfigSnapshot {
 	roots, leaf := TestCerts(t)
 
@@ -977,10 +984,10 @@ func setupTestVariationConfigEntriesAndSnapshot(
 				Kind: structs.ServiceResolver,
 				Name: "db",
 				Subsets: map[string]structs.ServiceResolverSubset{
-					"v1": structs.ServiceResolverSubset{
+					"v1": {
 						Filter: "Service.Meta.version == v1",
 					},
-					"v2": structs.ServiceResolverSubset{
+					"v2": {
 						Filter: "Service.Meta.version == v2",
 					},
 				},
@@ -1248,6 +1255,55 @@ func setupTestVariationConfigEntriesAndSnapshot(
 				},
 			},
 		)
+	case "lb-resolver":
+		entries = append(entries,
+			&structs.ProxyConfigEntry{
+				Kind: structs.ProxyDefaults,
+				Name: structs.ProxyConfigGlobal,
+				Config: map[string]interface{}{
+					"protocol": "http",
+				},
+			},
+			&structs.ServiceSplitterConfigEntry{
+				Kind: structs.ServiceSplitter,
+				Name: "db",
+				Splits: []structs.ServiceSplit{
+					{Weight: 95.5, Service: "something-else"},
+					{Weight: 4.5, Service: "db"},
+				},
+			},
+			&structs.ServiceResolverConfigEntry{
+				Kind: structs.ServiceResolver,
+				Name: "db",
+				LoadBalancer: &structs.LoadBalancer{
+					Policy: "ring_hash",
+					RingHashConfig: &structs.RingHashConfig{
+						MinimumRingSize: 20,
+						MaximumRingSize: 30,
+					},
+					HashPolicies: []structs.HashPolicy{
+						{
+							Field:      "cookie",
+							FieldValue: "chocolate-chip",
+							Terminal:   true,
+						},
+						{
+							Field:        "cookie",
+							FieldValue:   "chocolate-chip",
+							CookieConfig: &structs.CookieConfig{Session: true},
+						},
+						{
+							Field:      "header",
+							FieldValue: "x-user-id",
+						},
+						{
+							SourceIP: true,
+							Terminal: true,
+						},
+					},
+				},
+			},
+		)
 	case "http-multiple-services":
 	default:
 		t.Fatalf("unexpected variation: %q", variation)
@@ -1266,7 +1322,7 @@ func setupTestVariationConfigEntriesAndSnapshot(
 			"db": dbChain,
 		},
 		WatchedUpstreamEndpoints: map[string]map[string]structs.CheckServiceNodes{
-			"db": map[string]structs.CheckServiceNodes{
+			"db": {
 				"db.default.dc1": TestUpstreamNodes(t),
 			},
 		},
@@ -1288,7 +1344,7 @@ func setupTestVariationConfigEntriesAndSnapshot(
 		snap.WatchedUpstreamEndpoints["db"]["db.default.dc2"] =
 			TestUpstreamNodesDC2(t)
 		snap.WatchedGatewayEndpoints = map[string]map[string]structs.CheckServiceNodes{
-			"db": map[string]structs.CheckServiceNodes{
+			"db": {
 				"dc2": TestGatewayNodesDC2(t),
 			},
 		}
@@ -1301,7 +1357,7 @@ func setupTestVariationConfigEntriesAndSnapshot(
 	case "failover-through-double-remote-gateway":
 		snap.WatchedUpstreamEndpoints["db"]["db.default.dc3"] = TestUpstreamNodesDC2(t)
 		snap.WatchedGatewayEndpoints = map[string]map[string]structs.CheckServiceNodes{
-			"db": map[string]structs.CheckServiceNodes{
+			"db": {
 				"dc2": TestGatewayNodesDC2(t),
 				"dc3": TestGatewayNodesDC3(t),
 			},
@@ -1314,7 +1370,7 @@ func setupTestVariationConfigEntriesAndSnapshot(
 		snap.WatchedUpstreamEndpoints["db"]["db.default.dc2"] =
 			TestUpstreamNodesDC2(t)
 		snap.WatchedGatewayEndpoints = map[string]map[string]structs.CheckServiceNodes{
-			"db": map[string]structs.CheckServiceNodes{
+			"db": {
 				"dc1": TestGatewayNodesDC1(t),
 			},
 		}
@@ -1327,7 +1383,7 @@ func setupTestVariationConfigEntriesAndSnapshot(
 	case "failover-through-double-local-gateway":
 		snap.WatchedUpstreamEndpoints["db"]["db.default.dc3"] = TestUpstreamNodesDC2(t)
 		snap.WatchedGatewayEndpoints = map[string]map[string]structs.CheckServiceNodes{
-			"db": map[string]structs.CheckServiceNodes{
+			"db": {
 				"dc1": TestGatewayNodesDC1(t),
 			},
 		}
@@ -1346,6 +1402,7 @@ func setupTestVariationConfigEntriesAndSnapshot(
 		snap.WatchedUpstreamEndpoints["bar"] = map[string]structs.CheckServiceNodes{
 			"bar.default.dc1": TestUpstreamNodesAlternate(t),
 		}
+	case "lb-resolver":
 	default:
 		t.Fatalf("unexpected variation: %q", variation)
 		return ConfigSnapshotUpstreams{}
@@ -1362,6 +1419,56 @@ func TestConfigSnapshotMeshGatewayUsingFederationStates(t testing.T) *ConfigSnap
 	return testConfigSnapshotMeshGateway(t, true, true)
 }
 
+func TestConfigSnapshotMeshGatewayNewerInformationInFederationStates(t testing.T) *ConfigSnapshot {
+	snap := TestConfigSnapshotMeshGateway(t)
+
+	// Create a duplicate entry in FedStateGateways, with a high ModifyIndex, to
+	// verify that fresh data in the federation state is preferred over stale data
+	// in GatewayGroups.
+	svc := structs.TestNodeServiceMeshGatewayWithAddrs(t,
+		"10.0.1.3", 8443,
+		structs.ServiceAddress{Address: "10.0.1.3", Port: 8443},
+		structs.ServiceAddress{Address: "198.18.1.3", Port: 443},
+	)
+	svc.RaftIndex.ModifyIndex = math.MaxUint64
+
+	snap.MeshGateway.FedStateGateways = map[string]structs.CheckServiceNodes{
+		"dc2": {
+			{
+				Node:    snap.MeshGateway.GatewayGroups["dc2"][0].Node,
+				Service: svc,
+			},
+		},
+	}
+
+	return snap
+}
+
+func TestConfigSnapshotMeshGatewayOlderInformationInFederationStates(t testing.T) *ConfigSnapshot {
+	snap := TestConfigSnapshotMeshGateway(t)
+
+	// Create a duplicate entry in FedStateGateways, with a low ModifyIndex, to
+	// verify that stale data in the federation state is ignored in favor of the
+	// fresher data in GatewayGroups.
+	svc := structs.TestNodeServiceMeshGatewayWithAddrs(t,
+		"10.0.1.3", 8443,
+		structs.ServiceAddress{Address: "10.0.1.3", Port: 8443},
+		structs.ServiceAddress{Address: "198.18.1.3", Port: 443},
+	)
+	svc.RaftIndex.ModifyIndex = 0
+
+	snap.MeshGateway.FedStateGateways = map[string]structs.CheckServiceNodes{
+		"dc2": {
+			{
+				Node:    snap.MeshGateway.GatewayGroups["dc2"][0].Node,
+				Service: svc,
+			},
+		},
+	}
+
+	return snap
+}
+
 func TestConfigSnapshotMeshGatewayNoServices(t testing.T) *ConfigSnapshot {
 	return testConfigSnapshotMeshGateway(t, false, false)
 }
@@ -1378,11 +1485,11 @@ func testConfigSnapshotMeshGateway(t testing.T, populateServices bool, useFedera
 			Config: map[string]interface{}{},
 		},
 		TaggedAddresses: map[string]structs.ServiceAddress{
-			structs.TaggedAddressLAN: structs.ServiceAddress{
+			structs.TaggedAddressLAN: {
 				Address: "1.2.3.4",
 				Port:    8443,
 			},
-			structs.TaggedAddressWAN: structs.ServiceAddress{
+			structs.TaggedAddressWAN: {
 				Address: "198.18.0.1",
 				Port:    443,
 			},
@@ -1556,6 +1663,10 @@ func TestConfigSnapshotIngressGatewayNoServices(t testing.T) *ConfigSnapshot {
 	return testConfigSnapshotIngressGateway(t, false, "tcp", "default")
 }
 
+func TestConfigSnapshotIngressWithLB(t testing.T) *ConfigSnapshot {
+	return testConfigSnapshotIngressGateway(t, true, "http", "lb-resolver")
+}
+
 func TestConfigSnapshotIngressDiscoveryChainWithEntries(t testing.T, additionalEntries ...structs.ConfigEntry) *ConfigSnapshot {
 	return testConfigSnapshotIngressGateway(t, true, "http", "simple", additionalEntries...)
 }
@@ -1579,7 +1690,7 @@ func testConfigSnapshotIngressGateway(
 				t, variation, leaf, additionalEntries...,
 			),
 			Upstreams: map[IngressListenerKey]structs.Upstreams{
-				IngressListenerKey{protocol, 9191}: structs.Upstreams{
+				{protocol, 9191}: {
 					{
 						// We rely on this one having default type in a few tests...
 						DestinationName:  "db",
@@ -1641,7 +1752,7 @@ func testConfigSnapshotTerminatingGateway(t testing.T, populateServices bool) *C
 		ProxyID: structs.NewServiceID("terminating-gateway", nil),
 		Address: "1.2.3.4",
 		TaggedAddresses: map[string]structs.ServiceAddress{
-			structs.TaggedAddressWAN: structs.ServiceAddress{
+			structs.TaggedAddressWAN: {
 				Address: "198.18.0.1",
 				Port:    443,
 			},
@@ -1793,6 +1904,12 @@ func testConfigSnapshotTerminatingGateway(t testing.T, populateServices bool) *C
 				db:    dbNodes,
 				cache: cacheNodes,
 			},
+			ServiceResolversSet: map[structs.ServiceName]bool{
+				web:   true,
+				api:   true,
+				db:    true,
+				cache: true,
+			},
 			GatewayServices: map[structs.ServiceName]structs.GatewayService{
 				web: {
 					Service: web,
@@ -1817,20 +1934,43 @@ func testConfigSnapshotTerminatingGateway(t testing.T, populateServices bool) *C
 				cache: {cacheNodes[0], cacheNodes[1]},
 			},
 		}
+
+		snap.TerminatingGateway.ServiceConfigs = map[structs.ServiceName]*structs.ServiceConfigResponse{
+			web: {
+				ProxyConfig: map[string]interface{}{"protocol": "tcp"},
+			},
+			api: {
+				ProxyConfig: map[string]interface{}{"protocol": "tcp"},
+			},
+			db: {
+				ProxyConfig: map[string]interface{}{"protocol": "tcp"},
+			},
+			cache: {
+				ProxyConfig: map[string]interface{}{"protocol": "tcp"},
+			},
+		}
+		snap.TerminatingGateway.Intentions = map[structs.ServiceName]structs.Intentions{
+			// no intentions defined for thse services
+			web:   nil,
+			api:   nil,
+			db:    nil,
+			cache: nil,
+		}
+
 		snap.TerminatingGateway.ServiceLeaves = map[structs.ServiceName]*structs.IssuedCert{
-			structs.NewServiceName("web", nil): {
+			web: {
 				CertPEM:       golden(t, "test-leaf-cert"),
 				PrivateKeyPEM: golden(t, "test-leaf-key"),
 			},
-			structs.NewServiceName("api", nil): {
+			api: {
 				CertPEM:       golden(t, "alt-test-leaf-cert"),
 				PrivateKeyPEM: golden(t, "alt-test-leaf-key"),
 			},
-			structs.NewServiceName("db", nil): {
+			db: {
 				CertPEM:       golden(t, "db-test-leaf-cert"),
 				PrivateKeyPEM: golden(t, "db-test-leaf-key"),
 			},
-			structs.NewServiceName("cache", nil): {
+			cache: {
 				CertPEM:       golden(t, "cache-test-leaf-cert"),
 				PrivateKeyPEM: golden(t, "cache-test-leaf-key"),
 			},
@@ -1873,7 +2013,7 @@ func TestConfigSnapshotIngress_MultipleListenersDuplicateService(t testing.T) *C
 	snap := TestConfigSnapshotIngress_HTTPMultipleServices(t)
 
 	snap.IngressGateway.Upstreams = map[IngressListenerKey]structs.Upstreams{
-		IngressListenerKey{Protocol: "http", Port: 8080}: structs.Upstreams{
+		{Protocol: "http", Port: 8080}: {
 			{
 				DestinationName: "foo",
 				LocalBindPort:   8080,
@@ -1883,7 +2023,7 @@ func TestConfigSnapshotIngress_MultipleListenersDuplicateService(t testing.T) *C
 				LocalBindPort:   8080,
 			},
 		},
-		IngressListenerKey{Protocol: "http", Port: 443}: structs.Upstreams{
+		{Protocol: "http", Port: 443}: {
 			{
 				DestinationName: "foo",
 				LocalBindPort:   443,
