@@ -17,7 +17,7 @@ import (
 func TestServiceManager_RegisterService(t *testing.T) {
 	require := require.New(t)
 
-	a := NewTestAgent(t, "enable_central_service_config = true")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
@@ -64,7 +64,7 @@ func TestServiceManager_RegisterService(t *testing.T) {
 func TestServiceManager_RegisterSidecar(t *testing.T) {
 	require := require.New(t)
 
-	a := NewTestAgent(t, "enable_central_service_config = true")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
@@ -151,7 +151,7 @@ func TestServiceManager_RegisterSidecar(t *testing.T) {
 func TestServiceManager_RegisterMeshGateway(t *testing.T) {
 	require := require.New(t)
 
-	a := NewTestAgent(t, "enable_central_service_config = true")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
@@ -207,7 +207,7 @@ func TestServiceManager_RegisterMeshGateway(t *testing.T) {
 func TestServiceManager_RegisterTerminatingGateway(t *testing.T) {
 	require := require.New(t)
 
-	a := NewTestAgent(t, "enable_central_service_config = true")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
@@ -268,7 +268,7 @@ func TestServiceManager_PersistService_API(t *testing.T) {
 	require := require.New(t)
 
 	// Launch a server to manage the config entries.
-	serverAgent := NewTestAgent(t, `enable_central_service_config = true`)
+	serverAgent := NewTestAgent(t, "")
 	defer serverAgent.Shutdown()
 	testrpc.WaitForLeader(t, serverAgent.RPC, "dc1")
 
@@ -293,7 +293,6 @@ func TestServiceManager_PersistService_API(t *testing.T) {
 
 	// Now launch a single client agent
 	cfg := `
-	    enable_central_service_config = true
 		server = false
 		bootstrap = false
 	`
@@ -363,8 +362,8 @@ func TestServiceManager_PersistService_API(t *testing.T) {
 
 	svcID := svc.CompoundServiceID()
 
-	svcFile := filepath.Join(a.Config.DataDir, servicesDir, svcID.StringHash())
-	configFile := filepath.Join(a.Config.DataDir, serviceConfigDir, svcID.StringHash())
+	svcFile := filepath.Join(a.Config.DataDir, servicesDir, svcID.StringHashSHA256())
+	configFile := filepath.Join(a.Config.DataDir, serviceConfigDir, svcID.StringHashSHA256())
 
 	// Service is not persisted unless requested, but we always persist service configs.
 	require.NoError(a.AddService(svc, nil, false, "", ConfigSourceRemote))
@@ -481,7 +480,7 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 	t.Parallel()
 
 	// Launch a server to manage the config entries.
-	serverAgent := NewTestAgent(t, `enable_central_service_config = true`)
+	serverAgent := NewTestAgent(t, "")
 	defer serverAgent.Shutdown()
 	testrpc.WaitForLeader(t, serverAgent.RPC, "dc1")
 
@@ -526,7 +525,6 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 	`
 
 	cfg := `
-	    enable_central_service_config = true
 		server = false
 		bootstrap = false
 	` + serviceSnippet
@@ -544,6 +542,7 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 
 	// Now register a sidecar proxy via the API.
 	svcID := "web-sidecar-proxy"
+	defaultMeta := structs.DefaultEnterpriseMeta()
 
 	expectState := &structs.NodeService{
 		Kind:            structs.ServiceKindConnectProxy,
@@ -562,9 +561,10 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 			},
 			Upstreams: structs.Upstreams{
 				{
-					DestinationType: "service",
-					DestinationName: "redis",
-					LocalBindPort:   5000,
+					DestinationType:      "service",
+					DestinationName:      "redis",
+					DestinationNamespace: defaultMeta.NamespaceOrEmpty(),
+					LocalBindPort:        5000,
 					Config: map[string]interface{}{
 						"protocol": "tcp",
 					},
@@ -575,7 +575,7 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 			Passing: 1,
 			Warning: 1,
 		},
-		EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+		EnterpriseMeta: *defaultMeta,
 	}
 
 	// Now wait until we've re-registered using central config updated data.
@@ -589,8 +589,8 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 		require.Equal(r, expectState, current)
 	})
 
-	svcFile := filepath.Join(a.Config.DataDir, servicesDir, stringHash(svcID))
-	configFile := filepath.Join(a.Config.DataDir, serviceConfigDir, stringHash(svcID))
+	svcFile := filepath.Join(a.Config.DataDir, servicesDir, stringHashSHA256(svcID))
+	configFile := filepath.Join(a.Config.DataDir, serviceConfigDir, stringHashSHA256(svcID))
 
 	// Service is never persisted, but we always persist service configs.
 	requireFileIsAbsent(t, svcFile)
