@@ -18,7 +18,7 @@ func (a *Agent) sidecarServiceID(serviceID string) string {
 // config.
 //
 // It assumes the ns has been validated already which means the nested
-// SidecarService is also already validated.It also assumes that any check
+// SidecarService is also already validated. It also assumes that any check
 // definitions within the sidecar service definition have been validated if
 // necessary. If no sidecar service is defined in ns, then nil is returned with
 // nil error.
@@ -34,6 +34,9 @@ func (a *Agent) sidecarServiceFromNodeService(ns *structs.NodeService, token str
 		return nil, nil, "", nil
 	}
 
+	// for now at least these must be identical
+	ns.Connect.SidecarService.EnterpriseMeta = ns.EnterpriseMeta
+
 	// Start with normal conversion from service definition
 	sidecar := ns.Connect.SidecarService.NodeService()
 
@@ -41,9 +44,6 @@ func (a *Agent) sidecarServiceFromNodeService(ns *structs.NodeService, token str
 	// ID. We rely on this for lifecycle management of the nested definition.
 	sidecar.ID = a.sidecarServiceID(ns.ID)
 
-	// for now at least these must be identical
-	sidecar.EnterpriseMeta = ns.EnterpriseMeta
-
 	// Set some meta we can use to disambiguate between service instances we added
 	// later and are responsible for deregistering.
 	if sidecar.Meta != nil {
@@ -172,14 +172,14 @@ func (a *Agent) sidecarServiceFromNodeService(ns *structs.NodeService, token str
 	// Setup default check if none given
 	if len(checks) < 1 {
 		checks = []*structs.CheckType{
-			&structs.CheckType{
+			{
 				Name: "Connect Sidecar Listening",
 				// Default to localhost rather than agent/service public IP. The checks
 				// can always be overridden if a non-loopback IP is needed.
 				TCP:      ipaddr.FormatAddressPort(sidecar.Proxy.LocalServiceAddress, sidecar.Port),
 				Interval: 10 * time.Second,
 			},
-			&structs.CheckType{
+			{
 				Name:         "Connect Sidecar Aliasing " + ns.ID,
 				AliasService: ns.ID,
 			},
