@@ -60,12 +60,12 @@ func TestAgent_sidecarServiceFromNodeService(t *testing.T) {
 				},
 			},
 			wantChecks: []*structs.CheckType{
-				&structs.CheckType{
+				{
 					Name:     "Connect Sidecar Listening",
 					TCP:      "127.0.0.1:2222",
 					Interval: 10 * time.Second,
 				},
-				&structs.CheckType{
+				{
 					Name:         "Connect Sidecar Aliasing web1",
 					AliasService: "web1",
 				},
@@ -124,11 +124,12 @@ func TestAgent_sidecarServiceFromNodeService(t *testing.T) {
 					LocalServiceAddress:    "127.0.127.0",
 					LocalServicePort:       9999,
 					Config:                 map[string]interface{}{"baz": "qux"},
-					Upstreams:              structs.TestAddDefaultsToUpstreams(t, structs.TestUpstreams(t)),
+					Upstreams: structs.TestAddDefaultsToUpstreams(t, structs.TestUpstreams(t),
+						*structs.DefaultEnterpriseMeta()),
 				},
 			},
 			wantChecks: []*structs.CheckType{
-				&structs.CheckType{
+				{
 					ScriptArgs: []string{"sleep", "1"},
 					Interval:   999 * time.Second,
 				},
@@ -200,12 +201,12 @@ func TestAgent_sidecarServiceFromNodeService(t *testing.T) {
 				},
 			},
 			wantChecks: []*structs.CheckType{
-				&structs.CheckType{
+				{
 					Name:     "Connect Sidecar Listening",
 					TCP:      "127.0.0.1:2222",
 					Interval: 10 * time.Second,
 				},
-				&structs.CheckType{
+				{
 					Name:         "Connect Sidecar Aliasing web1",
 					AliasService: "web1",
 				},
@@ -288,12 +289,12 @@ func TestAgent_sidecarServiceFromNodeService(t *testing.T) {
 				},
 			},
 			wantChecks: []*structs.CheckType{
-				&structs.CheckType{
+				{
 					Name:     "Connect Sidecar Listening",
 					TCP:      "127.0.0.1:2222",
 					Interval: 10 * time.Second,
 				},
-				&structs.CheckType{
+				{
 					Name:         "Connect Sidecar Aliasing web1",
 					AliasService: "web1",
 				},
@@ -303,7 +304,7 @@ func TestAgent_sidecarServiceFromNodeService(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			// Set port range to be tiny (one availabl) to test consuming all of it.
+			// Set port range to be tiny (one available) to test consuming all of it.
 			// This allows a single assigned port at 2222 thanks to being inclusive at
 			// both ends.
 			if tt.maxPort == 0 {
