@@ -4,6 +4,7 @@ import (
 	"encoding/json"
 	"fmt"
 	"math"
+	"net/http"
 	"regexp"
 	"sort"
 	"strconv"
@@ -16,6 +17,37 @@ import (
 	"github.com/mitchellh/hashstructure"
 )
 
+const (
+	// Names of Envoy's LB policies
+	LBPolicyMaglev       = "maglev"
+	LBPolicyRingHash     = "ring_hash"
+	LBPolicyRandom       = "random"
+	LBPolicyLeastRequest = "least_request"
+	LBPolicyRoundRobin   = "round_robin"
+
+	// Names of Envoy's LB policies
+	HashPolicyCookie     = "cookie"
+	HashPolicyHeader     = "header"
+	HashPolicyQueryParam = "query_parameter"
+)
+
+var (
+	validLBPolicies = map[string]bool{
+		"":                   true,
+		LBPolicyRandom:       true,
+		LBPolicyRoundRobin:   true,
+		LBPolicyLeastRequest: true,
+		LBPolicyRingHash:     true,
+		LBPolicyMaglev:       true,
+	}
+
+	validHashPolicies = map[string]bool{
+		HashPolicyHeader:     true,
+		HashPolicyCookie:     true,
+		HashPolicyQueryParam: true,
+	}
+)
+
 // ServiceRouterConfigEntry defines L7 (e.g. http) routing rules for a named
 // service exposed in Connect.
 //
@@ -77,15 +109,12 @@ func (e *ServiceRouterConfigEntry) Normalize() error {
 		}
 
 		httpMatch := route.Match.HTTP
-		if len(httpMatch.Methods) == 0 {
-			continue
-		}
-
 		for j := 0; j < len(httpMatch.Methods); j++ {
 			httpMatch.Methods[j] = strings.ToUpper(httpMatch.Methods[j])
 		}
+
 		if route.Destination != nil && route.Destination.Namespace == "" {
-			route.Destination.Namespace = e.EnterpriseMeta.NamespaceOrDefault()
+			route.Destination.Namespace = e.EnterpriseMeta.NamespaceOrEmpty()
 		}
 	}
 
@@ -178,6 +207,9 @@ func (e *ServiceRouterConfigEntry) Validate() error {
 			if len(route.Match.HTTP.Methods) > 0 {
 				found := make(map[string]struct{})
 				for _, m := range route.Match.HTTP.Methods {
+					if !isValidHTTPMethod(m) {
+						return fmt.Errorf("Route[%d] Methods contains an invalid method %q", i, m)
+					}
 					if _, ok := found[m]; ok {
 						return fmt.Errorf("Route[%d] Methods contains %q more than once", i, m)
 					}
@@ -196,6 +228,23 @@ func (e *ServiceRouterConfigEntry) Validate() error {
 	return nil
 }
 
+func isValidHTTPMethod(method string) bool {
+	switch method {
+	case http.MethodGet,
+		http.MethodHead,
+		http.MethodPost,
+		http.MethodPut,
+		http.MethodPatch,
+		http.MethodDelete,
+		http.MethodConnect,
+		http.MethodOptions,
+		http.MethodTrace:
+		return true
+	default:
+		return false
+	}
+}
+
 func (e *ServiceRouterConfigEntry) CanRead(rule acl.Authorizer) bool {
 	return canReadDiscoveryChain(e, rule)
 }
@@ -233,7 +282,7 @@ func (e *ServiceRouterConfigEntry) ListRelatedServices() []ServiceID {
 	}
 
 	out := make([]ServiceID, 0, len(found))
-	for svc, _ := range found {
+	for svc := range found {
 		out = append(out, svc)
 	}
 	sort.Slice(out, func(i, j int) bool {
@@ -569,7 +618,7 @@ func (e *ServiceSplitterConfigEntry) ListRelatedServices() []ServiceID {
 	}
 
 	out := make([]ServiceID, 0, len(found))
-	for svc, _ := range found {
+	for svc := range found {
 		out = append(out, svc)
 	}
 	sort.Slice(out, func(i, j int) bool {
@@ -663,6 +712,10 @@ type ServiceResolverConfigEntry struct {
 	// to this service.
 	ConnectTimeout time.Duration `json:",omitempty" alias:"connect_timeout"`
 
+	// LoadBalancer determines the load balancing policy and configuration for services
+	// issuing requests to this upstream service.
+	LoadBalancer *LoadBalancer `json:",omitempty" alias:"load_balancer"`
+
 	Meta           map[string]string `json:",omitempty"`
 	EnterpriseMeta `hcl:",squash" mapstructure:",squash"`
 	RaftIndex
@@ -720,7 +773,8 @@ func (e *ServiceResolverConfigEntry) IsDefault() bool {
 		len(e.Subsets) == 0 &&
 		e.Redirect == nil &&
 		len(e.Failover) == 0 &&
-		e.ConnectTimeout == 0
+		e.ConnectTimeout == 0 &&
+		e.LoadBalancer == nil
 }
 
 func (e *ServiceResolverConfigEntry) GetKind() string {
@@ -764,7 +818,7 @@ func (e *ServiceResolverConfigEntry) Validate() error {
 	}
 
 	if len(e.Subsets) > 0 {
-		for name, _ := range e.Subsets {
+		for name := range e.Subsets {
 			if name == "" {
 				return fmt.Errorf("Subset defined with empty name")
 			}
@@ -843,6 +897,56 @@ func (e *ServiceResolverConfigEntry) Validate() error {
 		return fmt.Errorf("Bad ConnectTimeout '%s', must be >= 0", e.ConnectTimeout)
 	}
 
+	if e.LoadBalancer != nil {
+		lb := e.LoadBalancer
+
+		if ok := validLBPolicies[lb.Policy]; !ok {
+			return fmt.Errorf("Bad LoadBalancer policy: %q is not supported", lb.Policy)
+		}
+
+		if lb.Policy != LBPolicyRingHash && lb.RingHashConfig != nil {
+			return fmt.Errorf("Bad LoadBalancer configuration. "+
+				"RingHashConfig specified for incompatible load balancing policy %q", lb.Policy)
+		}
+		if lb.Policy != LBPolicyLeastRequest && lb.LeastRequestConfig != nil {
+			return fmt.Errorf("Bad LoadBalancer configuration. "+
+				"LeastRequestConfig specified for incompatible load balancing policy %q", lb.Policy)
+		}
+		if !lb.IsHashBased() && len(lb.HashPolicies) > 0 {
+			return fmt.Errorf("Bad LoadBalancer configuration: "+
+				"HashPolicies specified for non-hash-based Policy: %q", lb.Policy)
+		}
+
+		for i, hp := range lb.HashPolicies {
+			if ok := validHashPolicies[hp.Field]; hp.Field != "" && !ok {
+				return fmt.Errorf("Bad LoadBalancer HashPolicy[%d]: %q is not a supported field", i, hp.Field)
+			}
+
+			if hp.SourceIP && hp.Field != "" {
+				return fmt.Errorf("Bad LoadBalancer HashPolicy[%d]: "+
+					"A single hash policy cannot hash both a source address and a %q", i, hp.Field)
+			}
+			if hp.SourceIP && hp.FieldValue != "" {
+				return fmt.Errorf("Bad LoadBalancer HashPolicy[%d]: "+
+					"A FieldValue cannot be specified when hashing SourceIP", i)
+			}
+			if hp.Field != "" && hp.FieldValue == "" {
+				return fmt.Errorf("Bad LoadBalancer HashPolicy[%d]: Field %q was specified without a FieldValue", i, hp.Field)
+			}
+			if hp.FieldValue != "" && hp.Field == "" {
+				return fmt.Errorf("Bad LoadBalancer HashPolicy[%d]: FieldValue requires a Field to apply to", i)
+			}
+			if hp.CookieConfig != nil {
+				if hp.Field != HashPolicyCookie {
+					return fmt.Errorf("Bad LoadBalancer HashPolicy[%d]: cookie_config provided for %q", i, hp.Field)
+				}
+				if hp.CookieConfig.Session && hp.CookieConfig.TTL != 0*time.Second {
+					return fmt.Errorf("Bad LoadBalancer HashPolicy[%d]: a session cookie cannot have an associated TTL", i)
+				}
+			}
+		}
+	}
+
 	return nil
 }
 
@@ -895,7 +999,7 @@ func (e *ServiceResolverConfigEntry) ListRelatedServices() []ServiceID {
 	}
 
 	out := make([]ServiceID, 0, len(found))
-	for svc, _ := range found {
+	for svc := range found {
 		out = append(out, svc)
 	}
 	sort.Slice(out, func(i, j int) bool {
@@ -979,6 +1083,92 @@ type ServiceResolverFailover struct {
 	Datacenters []string `json:",omitempty"`
 }
 
+// LoadBalancer determines the load balancing policy and configuration for services
+// issuing requests to this upstream service.
+type LoadBalancer struct {
+	// Policy is the load balancing policy used to select a host
+	Policy string `json:",omitempty"`
+
+	// RingHashConfig contains configuration for the "ring_hash" policy type
+	RingHashConfig *RingHashConfig `json:",omitempty" alias:"ring_hash_config"`
+
+	// LeastRequestConfig contains configuration for the "least_request" policy type
+	LeastRequestConfig *LeastRequestConfig `json:",omitempty" alias:"least_request_config"`
+
+	// HashPolicies is a list of hash policies to use for hashing load balancing algorithms.
+	// Hash policies are evaluated individually and combined such that identical lists
+	// result in the same hash.
+	// If no hash policies are present, or none are successfully evaluated,
+	// then a random backend host will be selected.
+	HashPolicies []HashPolicy `json:",omitempty" alias:"hash_policies"`
+}
+
+// RingHashConfig contains configuration for the "ring_hash" policy type
+type RingHashConfig struct {
+	// MinimumRingSize determines the minimum number of entries in the hash ring
+	MinimumRingSize uint64 `json:",omitempty" alias:"minimum_ring_size"`
+
+	// MaximumRingSize determines the maximum number of entries in the hash ring
+	MaximumRingSize uint64 `json:",omitempty" alias:"maximum_ring_size"`
+}
+
+// LeastRequestConfig contains configuration for the "least_request" policy type
+type LeastRequestConfig struct {
+	// ChoiceCount determines the number of random healthy hosts from which to select the one with the least requests.
+	ChoiceCount uint32 `json:",omitempty" alias:"choice_count"`
+}
+
+// HashPolicy defines which attributes will be hashed by hash-based LB algorithms
+type HashPolicy struct {
+	// Field is the attribute type to hash on.
+	// Must be one of "header","cookie", or "query_parameter".
+	// Cannot be specified along with SourceIP.
+	Field string `json:",omitempty"`
+
+	// FieldValue is the value to hash.
+	// ie. header name, cookie name, URL query parameter name
+	// Cannot be specified along with SourceIP.
+	FieldValue string `json:",omitempty" alias:"field_value"`
+
+	// CookieConfig contains configuration for the "cookie" hash policy type.
+	CookieConfig *CookieConfig `json:",omitempty" alias:"cookie_config"`
+
+	// SourceIP determines whether the hash should be of the source IP rather than of a field and field value.
+	// Cannot be specified along with Field or FieldValue.
+	SourceIP bool `json:",omitempty" alias:"source_ip"`
+
+	// Terminal will short circuit the computation of the hash when multiple hash policies are present.
+	// If a hash is computed when a Terminal policy is evaluated,
+	// then that hash will be used and subsequent hash policies will be ignored.
+	Terminal bool `json:",omitempty"`
+}
+
+// CookieConfig contains configuration for the "cookie" hash policy type.
+// This is specified to have Envoy generate a cookie for a client on its first request.
+type CookieConfig struct {
+	// Generates a session cookie with no expiration.
+	Session bool `json:",omitempty"`
+
+	// TTL for generated cookies. Cannot be specified for session cookies.
+	TTL time.Duration `json:",omitempty"`
+
+	// The path to set for the cookie
+	Path string `json:",omitempty"`
+}
+
+func (lb *LoadBalancer) IsHashBased() bool {
+	if lb == nil {
+		return false
+	}
+
+	switch lb.Policy {
+	case LBPolicyMaglev, LBPolicyRingHash:
+		return true
+	default:
+		return false
+	}
+}
+
 type discoveryChainConfigEntry interface {
 	ConfigEntry
 	// ListRelatedServices returns a list of other names of services referenced
@@ -1248,3 +1438,12 @@ func defaultIfEmpty(val, defaultVal string) string {
 	}
 	return defaultVal
 }
+
+func IsProtocolHTTPLike(protocol string) bool {
+	switch protocol {
+	case "http", "http2", "grpc":
+		return true
+	default:
+		return false
+	}
+}
