@@ -46,6 +46,10 @@ func (m *EnterpriseMeta) NamespaceOrDefault() string {
 	return IntentionDefaultNamespace
 }
 
+func (m *EnterpriseMeta) NamespaceOrEmpty() string {
+	return ""
+}
+
 func EnterpriseMetaInitializer(_ string) EnterpriseMeta {
 	return emptyEnterpriseMeta
 }
@@ -70,6 +74,11 @@ func (_ *EnterpriseMeta) FillAuthzContext(_ *acl.AuthorizerContext) {}
 
 func (_ *EnterpriseMeta) Normalize() {}
 
+// GetNamespace always returns the empty string.
+func (_ *EnterpriseMeta) GetNamespace() string {
+	return ""
+}
+
 // FillAuthzContext stub
 func (_ *DirEntry) FillAuthzContext(_ *acl.AuthorizerContext) {}
 
@@ -97,7 +106,7 @@ func ParseServiceIDString(input string) (string, *EnterpriseMeta) {
 	return input, DefaultEnterpriseMeta()
 }
 
-func (sid *ServiceID) String() string {
+func (sid ServiceID) String() string {
 	return sid.ID
 }
 
@@ -110,7 +119,7 @@ func ParseServiceNameString(input string) (string, *EnterpriseMeta) {
 	return input, DefaultEnterpriseMeta()
 }
 
-func (n *ServiceName) String() string {
+func (n ServiceName) String() string {
 	return n.Name
 }
 
@@ -119,7 +128,7 @@ func ServiceNameFromString(input string) ServiceName {
 	return ServiceName{Name: id}
 }
 
-func (cid *CheckID) String() string {
+func (cid CheckID) String() string {
 	return string(cid.ID)
 }
 
@@ -127,6 +136,10 @@ func (_ *HealthCheck) Validate() error {
 	return nil
 }
 
+func enterpriseRequestType(m MessageType) (string, bool) {
+	return "", false
+}
+
 // CheckIDs returns the IDs for all checks associated with a session, regardless of type
 func (s *Session) CheckIDs() []types.CheckID {
 	// Merge all check IDs into a single slice, since they will be handled the same way
