@@ -11,16 +11,16 @@ import (
 	"time"
 
 	"github.com/hashicorp/consul/sdk/testutil"
-	"github.com/pascaldekloe/goe/verify"
+	"github.com/stretchr/testify/require"
 )
 
-func TestStringHash(t *testing.T) {
+func TestStringHashSHA256(t *testing.T) {
 	t.Parallel()
-	in := "hello world"
-	expected := "5eb63bbbe01eeed093cb22bb8f5acdc3"
+	in := "hello world\n"
+	expected := "a948904f2f0f479b8f8197694b30184b0d2ed1c1cd2a1ec0fb85d299a192a447"
 
-	if out := stringHash(in); out != expected {
-		t.Fatalf("bad: %s", out)
+	if out := stringHashSHA256(in); out != expected {
+		t.Fatalf("bad: %s expected %s", out, expected)
 	}
 }
 
@@ -31,7 +31,6 @@ func TestSetFilePermissions(t *testing.T) {
 	}
 	tempFile := testutil.TempFile(t, "consul")
 	path := tempFile.Name()
-	defer os.Remove(path)
 
 	// Bad UID fails
 	if err := setFilePermissions(path, "%", "", ""); err == nil {
@@ -121,7 +120,7 @@ func TestDurationFixer(t *testing.T) {
 	}
 
 	// Ensure we only processed the intended fieldnames
-	verify.Values(t, "", obj, expected)
+	require.Equal(t, expected, obj)
 }
 
 // helperProcessSentinel is a sentinel value that is put as the first
