@@ -12,11 +12,12 @@ import (
 	envoy "github.com/envoyproxy/go-control-plane/envoy/api/v2"
 	envoycore "github.com/envoyproxy/go-control-plane/envoy/api/v2/core"
 	envoyendpoint "github.com/envoyproxy/go-control-plane/envoy/api/v2/endpoint"
+	testinf "github.com/mitchellh/go-testing-interface"
+
 	"github.com/hashicorp/consul/agent/proxycfg"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/agent/xds/proxysupport"
 	"github.com/hashicorp/consul/sdk/testutil"
-	testinf "github.com/mitchellh/go-testing-interface"
 )
 
 func Test_makeLoadAssignment(t *testing.T) {
@@ -243,6 +244,14 @@ func Test_endpointsFromSnapshot(t *testing.T) {
 			create: proxycfg.TestConfigSnapshotMeshGatewayUsingFederationStates,
 			setup:  nil,
 		},
+		{
+			name:   "mesh-gateway-newer-information-in-federation-states",
+			create: proxycfg.TestConfigSnapshotMeshGatewayNewerInformationInFederationStates,
+		},
+		{
+			name:   "mesh-gateway-older-information-in-federation-states",
+			create: proxycfg.TestConfigSnapshotMeshGatewayOlderInformationInFederationStates,
+		},
 		{
 			name:   "mesh-gateway-no-services",
 			create: proxycfg.TestConfigSnapshotMeshGatewayNoServices,
@@ -328,27 +337,27 @@ func Test_endpointsFromSnapshot(t *testing.T) {
 			create: proxycfg.TestConfigSnapshotMeshGateway,
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.MeshGateway.ServiceResolvers = map[structs.ServiceName]*structs.ServiceResolverConfigEntry{
-					structs.NewServiceName("bar", nil): &structs.ServiceResolverConfigEntry{
+					structs.NewServiceName("bar", nil): {
 						Kind: structs.ServiceResolver,
 						Name: "bar",
 						Subsets: map[string]structs.ServiceResolverSubset{
-							"v1": structs.ServiceResolverSubset{
+							"v1": {
 								Filter: "Service.Meta.version == 1",
 							},
-							"v2": structs.ServiceResolverSubset{
+							"v2": {
 								Filter:      "Service.Meta.version == 2",
 								OnlyPassing: true,
 							},
 						},
 					},
-					structs.NewServiceName("foo", nil): &structs.ServiceResolverConfigEntry{
+					structs.NewServiceName("foo", nil): {
 						Kind: structs.ServiceResolver,
 						Name: "foo",
 						Subsets: map[string]structs.ServiceResolverSubset{
-							"v1": structs.ServiceResolverSubset{
+							"v1": {
 								Filter: "Service.Meta.version == 1",
 							},
-							"v2": structs.ServiceResolverSubset{
+							"v2": {
 								Filter:      "Service.Meta.version == 2",
 								OnlyPassing: true,
 							},
@@ -362,29 +371,29 @@ func Test_endpointsFromSnapshot(t *testing.T) {
 			create: proxycfg.TestConfigSnapshotMeshGateway,
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.MeshGateway.ServiceResolvers = map[structs.ServiceName]*structs.ServiceResolverConfigEntry{
-					structs.NewServiceName("bar", nil): &structs.ServiceResolverConfigEntry{
+					structs.NewServiceName("bar", nil): {
 						Kind:          structs.ServiceResolver,
 						Name:          "bar",
 						DefaultSubset: "v2",
 						Subsets: map[string]structs.ServiceResolverSubset{
-							"v1": structs.ServiceResolverSubset{
+							"v1": {
 								Filter: "Service.Meta.version == 1",
 							},
-							"v2": structs.ServiceResolverSubset{
+							"v2": {
 								Filter:      "Service.Meta.version == 2",
 								OnlyPassing: true,
 							},
 						},
 					},
-					structs.NewServiceName("foo", nil): &structs.ServiceResolverConfigEntry{
+					structs.NewServiceName("foo", nil): {
 						Kind:          structs.ServiceResolver,
 						Name:          "foo",
 						DefaultSubset: "v2",
 						Subsets: map[string]structs.ServiceResolverSubset{
-							"v1": structs.ServiceResolverSubset{
+							"v1": {
 								Filter: "Service.Meta.version == 1",
 							},
-							"v2": structs.ServiceResolverSubset{
+							"v2": {
 								Filter:      "Service.Meta.version == 2",
 								OnlyPassing: true,
 							},
@@ -517,7 +526,7 @@ func Test_endpointsFromSnapshot(t *testing.T) {
 			create: proxycfg.TestConfigSnapshotTerminatingGateway,
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.TerminatingGateway.ServiceResolvers = map[structs.ServiceName]*structs.ServiceResolverConfigEntry{
-					structs.NewServiceName("web", nil): &structs.ServiceResolverConfigEntry{
+					structs.NewServiceName("web", nil): {
 						Kind:          structs.ServiceResolver,
 						Name:          "web",
 						DefaultSubset: "v2",
@@ -531,7 +540,7 @@ func Test_endpointsFromSnapshot(t *testing.T) {
 							},
 						},
 					},
-					structs.NewServiceName("web", nil): &structs.ServiceResolverConfigEntry{
+					structs.NewServiceName("web", nil): {
 						Kind:          structs.ServiceResolver,
 						Name:          "web",
 						DefaultSubset: "v2",
@@ -555,8 +564,10 @@ func Test_endpointsFromSnapshot(t *testing.T) {
 		},
 	}
 
+	latestEnvoyVersion := proxysupport.EnvoyVersions[0]
 	for _, envoyVersion := range proxysupport.EnvoyVersions {
-		sf := determineSupportedProxyFeaturesFromString(envoyVersion)
+		sf, err := determineSupportedProxyFeaturesFromString(envoyVersion)
+		require.NoError(t, err)
 		t.Run("envoy-"+envoyVersion, func(t *testing.T) {
 			for _, tt := range tests {
 				t.Run(tt.name, func(t *testing.T) {
@@ -575,10 +586,7 @@ func Test_endpointsFromSnapshot(t *testing.T) {
 					}
 
 					// Need server just for logger dependency
-					logger := testutil.Logger(t)
-					s := Server{
-						Logger: logger,
-					}
+					s := Server{Logger: testutil.Logger(t)}
 
 					cInfo := connectionInfo{
 						Token:         "my-token",
@@ -599,7 +607,7 @@ func Test_endpointsFromSnapshot(t *testing.T) {
 						gName = tt.overrideGoldenName
 					}
 
-					require.JSONEq(goldenEnvoy(t, filepath.Join("endpoints", gName), envoyVersion, gotJSON), gotJSON)
+					require.JSONEq(goldenEnvoy(t, filepath.Join("endpoints", gName), envoyVersion, latestEnvoyVersion, gotJSON), gotJSON)
 				})
 			}
 		})
