@@ -2,19 +2,22 @@ package xds
 
 import (
 	"bytes"
-	"fmt"
 	"path/filepath"
 	"sort"
 	"testing"
 	"text/template"
+	"time"
 
 	envoy "github.com/envoyproxy/go-control-plane/envoy/api/v2"
+	"github.com/envoyproxy/go-control-plane/pkg/wellknown"
+	testinf "github.com/mitchellh/go-testing-interface"
+	"github.com/stretchr/testify/require"
+
 	"github.com/hashicorp/consul/agent/proxycfg"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/agent/xds/proxysupport"
 	"github.com/hashicorp/consul/sdk/testutil"
-	testinf "github.com/mitchellh/go-testing-interface"
-	"github.com/stretchr/testify/require"
+	"github.com/hashicorp/consul/types"
 )
 
 func TestListenersFromSnapshot(t *testing.T) {
@@ -27,6 +30,7 @@ func TestListenersFromSnapshot(t *testing.T) {
 		// test input.
 		setup              func(snap *proxycfg.ConfigSnapshot)
 		overrideGoldenName string
+		serverSetup        func(*Server)
 	}{
 		{
 			name:   "defaults",
@@ -80,6 +84,66 @@ func TestListenersFromSnapshot(t *testing.T) {
 					})
 			},
 		},
+		{
+			name:   "custom-public-listener-http",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Config["protocol"] = "http"
+				snap.Proxy.Config["envoy_public_listener_json"] =
+					customHTTPListenerJSON(t, customHTTPListenerJSONOptions{
+						Name: "custom-public-listen",
+					})
+			},
+		},
+		{
+			name:   "custom-public-listener-http-typed",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Config["protocol"] = "http"
+				snap.Proxy.Config["envoy_public_listener_json"] =
+					customHTTPListenerJSON(t, customHTTPListenerJSONOptions{
+						Name:        "custom-public-listen",
+						TypedConfig: true,
+					})
+			},
+		},
+		{
+			name:   "custom-public-listener-http-2",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Config["protocol"] = "http"
+				snap.Proxy.Config["envoy_public_listener_json"] =
+					customHTTPListenerJSON(t, customHTTPListenerJSONOptions{
+						Name:                      "custom-public-listen",
+						HTTPConnectionManagerName: httpConnectionManagerNewName,
+					})
+			},
+		},
+		{
+			name:   "custom-public-listener-http-2-typed",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Config["protocol"] = "http"
+				snap.Proxy.Config["envoy_public_listener_json"] =
+					customHTTPListenerJSON(t, customHTTPListenerJSONOptions{
+						Name:                      "custom-public-listen",
+						HTTPConnectionManagerName: httpConnectionManagerNewName,
+						TypedConfig:               true,
+					})
+			},
+		},
+		{
+			name:   "custom-public-listener-http-missing",
+			create: proxycfg.TestConfigSnapshot,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Config["protocol"] = "http"
+				snap.Proxy.Config["envoy_public_listener_json"] =
+					customListenerJSON(t, customListenerJSONOptions{
+						Name:        "custom-public-listen",
+						IncludeType: false,
+					})
+			},
+		},
 		{
 			name:               "custom-public-listener-typed",
 			create:             proxycfg.TestConfigSnapshot,
@@ -231,6 +295,38 @@ func TestListenersFromSnapshot(t *testing.T) {
 				}
 			},
 		},
+		{
+			// NOTE: if IPv6 is not supported in the kernel per
+			// kernelSupportsIPv6() then this test will fail because the golden
+			// files were generated assuming ipv6 support was present
+			name:   "expose-checks",
+			create: proxycfg.TestConfigSnapshotExposeConfig,
+			setup: func(snap *proxycfg.ConfigSnapshot) {
+				snap.Proxy.Expose = structs.ExposeConfig{
+					Checks: true,
+				}
+			},
+			serverSetup: func(s *Server) {
+				s.CfgFetcher = configFetcherFunc(func() string {
+					return "192.0.2.1"
+				})
+
+				s.CheckFetcher = httpCheckFetcherFunc(func(sid structs.ServiceID) []structs.CheckType {
+					if sid != structs.NewServiceID("web", nil) {
+						return nil
+					}
+					return []structs.CheckType{{
+						CheckID:   types.CheckID("http"),
+						Name:      "http",
+						HTTP:      "http://127.0.0.1:8181/debug",
+						ProxyHTTP: "http://:21500/debug",
+						Method:    "GET",
+						Interval:  10 * time.Second,
+						Timeout:   1 * time.Second,
+					}}
+				})
+			},
+		},
 		{
 			name:   "mesh-gateway",
 			create: proxycfg.TestConfigSnapshotMeshGateway,
@@ -259,15 +355,15 @@ func TestListenersFromSnapshot(t *testing.T) {
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.Proxy.Config = map[string]interface{}{
 					"envoy_mesh_gateway_bind_addresses": map[string]structs.ServiceAddress{
-						"foo": structs.ServiceAddress{
+						"foo": {
 							Address: "198.17.2.3",
 							Port:    8080,
 						},
-						"bar": structs.ServiceAddress{
+						"bar": {
 							Address: "2001:db8::ff",
 							Port:    9999,
 						},
-						"baz": structs.ServiceAddress{
+						"baz": {
 							Address: "127.0.0.1",
 							Port:    8765,
 						},
@@ -285,14 +381,14 @@ func TestListenersFromSnapshot(t *testing.T) {
 			create: proxycfg.TestConfigSnapshotIngressGateway,
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.TaggedAddresses = map[string]structs.ServiceAddress{
-					"lan": structs.ServiceAddress{Address: "10.0.0.1"},
-					"wan": structs.ServiceAddress{Address: "172.16.0.1"},
+					"lan": {Address: "10.0.0.1"},
+					"wan": {Address: "172.16.0.1"},
 				}
 				snap.Proxy.Config = map[string]interface{}{
 					"envoy_gateway_no_default_bind":       true,
 					"envoy_gateway_bind_tagged_addresses": true,
 					"envoy_gateway_bind_addresses": map[string]structs.ServiceAddress{
-						"foo": structs.ServiceAddress{Address: "8.8.8.8"},
+						"foo": {Address: "8.8.8.8"},
 					},
 				}
 			},
@@ -345,15 +441,15 @@ func TestListenersFromSnapshot(t *testing.T) {
 					"envoy_gateway_no_default_bind":       true,
 					"envoy_gateway_bind_tagged_addresses": true,
 					"envoy_gateway_bind_addresses": map[string]structs.ServiceAddress{
+						// This bind address should not get a listener due to deduplication and it sorts to the end
+						"z-duplicate-of-tagged-wan-addr": {
+							Address: "198.18.0.1",
+							Port:    443,
+						},
 						"foo": {
 							Address: "198.17.2.3",
 							Port:    8080,
 						},
-						// This bind address should not get a listener due to deduplication
-						"duplicate-of-tagged-wan-addr": {
-							Address: "198.18.0.1",
-							Port:    443,
-						},
 					},
 				}
 			},
@@ -376,19 +472,9 @@ func TestListenersFromSnapshot(t *testing.T) {
 							},
 						},
 					},
-					structs.NewServiceName("web", nil): {
-						Kind: structs.ServiceResolver,
-						Name: "web",
-						Subsets: map[string]structs.ServiceResolverSubset{
-							"v1": {
-								Filter: "Service.Meta.version == 1",
-							},
-							"v2": {
-								Filter:      "Service.Meta.version == 2",
-								OnlyPassing: true,
-							},
-						},
-					},
+				}
+				snap.TerminatingGateway.ServiceConfigs[structs.NewServiceName("web", nil)] = &structs.ServiceConfigResponse{
+					ProxyConfig: map[string]interface{}{"protocol": "http"},
 				}
 			},
 		},
@@ -397,7 +483,7 @@ func TestListenersFromSnapshot(t *testing.T) {
 			create: proxycfg.TestConfigSnapshotIngress_HTTPMultipleServices,
 			setup: func(snap *proxycfg.ConfigSnapshot) {
 				snap.IngressGateway.Upstreams = map[proxycfg.IngressListenerKey]structs.Upstreams{
-					proxycfg.IngressListenerKey{Protocol: "http", Port: 8080}: structs.Upstreams{
+					{Protocol: "http", Port: 8080}: {
 						{
 							DestinationName: "foo",
 							LocalBindPort:   8080,
@@ -407,7 +493,7 @@ func TestListenersFromSnapshot(t *testing.T) {
 							LocalBindPort:   8080,
 						},
 					},
-					proxycfg.IngressListenerKey{Protocol: "http", Port: 443}: structs.Upstreams{
+					{Protocol: "http", Port: 443}: {
 						{
 							DestinationName: "baz",
 							LocalBindPort:   443,
@@ -434,8 +520,10 @@ func TestListenersFromSnapshot(t *testing.T) {
 		},
 	}
 
+	latestEnvoyVersion := proxysupport.EnvoyVersions[0]
 	for _, envoyVersion := range proxysupport.EnvoyVersions {
-		sf := determineSupportedProxyFeaturesFromString(envoyVersion)
+		sf, err := determineSupportedProxyFeaturesFromString(envoyVersion)
+		require.NoError(t, err)
 		t.Run("envoy-"+envoyVersion, func(t *testing.T) {
 			for _, tt := range tests {
 				t.Run(tt.name, func(t *testing.T) {
@@ -454,9 +542,9 @@ func TestListenersFromSnapshot(t *testing.T) {
 					}
 
 					// Need server just for logger dependency
-					logger := testutil.Logger(t)
-					s := Server{
-						Logger: logger,
+					s := Server{Logger: testutil.Logger(t)}
+					if tt.serverSetup != nil {
+						tt.serverSetup(&s)
 					}
 
 					cInfo := connectionInfo{
@@ -464,24 +552,14 @@ func TestListenersFromSnapshot(t *testing.T) {
 						ProxyFeatures: sf,
 					}
 					listeners, err := s.listenersFromSnapshot(cInfo, snap)
+					require.NoError(err)
+
+					// The order of listeners returned via LDS isn't relevant, so it's safe
+					// to sort these for the purposes of test comparisons.
 					sort.Slice(listeners, func(i, j int) bool {
 						return listeners[i].(*envoy.Listener).Name < listeners[j].(*envoy.Listener).Name
 					})
 
-					// For terminating gateways we create filter chain matches for services/subsets from the ServiceGroups map
-					for i := 0; i < len(listeners); i++ {
-						l := listeners[i].(*envoy.Listener)
-
-						if l.FilterChains != nil {
-							// Sort chains by the matched name with the exception of the last one
-							// The last chain is a fallback and does not have a FilterChainMatch
-							sort.Slice(l.FilterChains[:len(l.FilterChains)-1], func(i, j int) bool {
-								return l.FilterChains[i].FilterChainMatch.ServerNames[0] < l.FilterChains[j].FilterChainMatch.ServerNames[0]
-							})
-						}
-					}
-
-					require.NoError(err)
 					r, err := createResponse(ListenerType, "00000001", "00000001", listeners)
 					require.NoError(err)
 
@@ -492,18 +570,14 @@ func TestListenersFromSnapshot(t *testing.T) {
 						gName = tt.overrideGoldenName
 					}
 
-					require.JSONEq(goldenEnvoy(t, filepath.Join("listeners", gName), envoyVersion, gotJSON), gotJSON)
+					require.JSONEq(goldenEnvoy(t, filepath.Join("listeners", gName), envoyVersion, latestEnvoyVersion, gotJSON), gotJSON)
 				})
 			}
 		})
 	}
 }
 
-func expectListenerJSONResources(t *testing.T, snap *proxycfg.ConfigSnapshot, token string, v, n uint64) map[string]string {
-	tokenVal := ""
-	if token != "" {
-		tokenVal = fmt.Sprintf(",\n"+`"value": "%s"`, token)
-	}
+func expectListenerJSONResources(t *testing.T, snap *proxycfg.ConfigSnapshot) map[string]string {
 	return map[string]string{
 		"public_listener": `{
 				"@type": "type.googleapis.com/envoy.api.v2.Listener",
@@ -519,18 +593,9 @@ func expectListenerJSONResources(t *testing.T, snap *proxycfg.ConfigSnapshot, to
 						"tlsContext": ` + expectedPublicTLSContextJSON(t, snap) + `,
 						"filters": [
 							{
-								"name": "envoy.ext_authz",
+								"name": "envoy.filters.network.rbac",
 								"config": {
-										"grpc_service": {
-												"envoy_grpc": {
-													"cluster_name": "local_agent"
-												},
-												"initial_metadata": [
-													{
-														"key": "x-consul-token"
-														` + tokenVal + `
-													}
-												]
+										"rules": {
 											},
 										"stat_prefix": "connect_authz"
 									}
@@ -539,7 +604,7 @@ func expectListenerJSONResources(t *testing.T, snap *proxycfg.ConfigSnapshot, to
 								"name": "envoy.tcp_proxy",
 								"config": {
 									"cluster": "local_app",
-									"stat_prefix": "public_listener_tcp"
+									"stat_prefix": "public_listener"
 								}
 							}
 						]
@@ -562,7 +627,7 @@ func expectListenerJSONResources(t *testing.T, snap *proxycfg.ConfigSnapshot, to
 							"name": "envoy.tcp_proxy",
 							"config": {
 								"cluster": "db.default.dc1.internal.11111111-2222-3333-4444-555555555555.consul",
-								"stat_prefix": "upstream_db_tcp"
+								"stat_prefix": "upstream.db.default.dc1"
 							}
 						}
 					]
@@ -585,7 +650,7 @@ func expectListenerJSONResources(t *testing.T, snap *proxycfg.ConfigSnapshot, to
 							"name": "envoy.tcp_proxy",
 							"config": {
 								"cluster": "geo-cache.default.dc1.query.11111111-2222-3333-4444-555555555555.consul",
-								"stat_prefix": "upstream_prepared_query_geo-cache_tcp"
+								"stat_prefix": "upstream.prepared_query_geo-cache"
 							}
 						}
 					]
@@ -595,7 +660,7 @@ func expectListenerJSONResources(t *testing.T, snap *proxycfg.ConfigSnapshot, to
 	}
 }
 
-func expectListenerJSONFromResources(t *testing.T, snap *proxycfg.ConfigSnapshot, token string, v, n uint64, resourcesJSON map[string]string) string {
+func expectListenerJSONFromResources(snap *proxycfg.ConfigSnapshot, v, n uint64, resourcesJSON map[string]string) string {
 	resJSON := ""
 	// Sort resources into specific order because that matters in JSONEq
 	// comparison later.
@@ -621,16 +686,14 @@ func expectListenerJSONFromResources(t *testing.T, snap *proxycfg.ConfigSnapshot
 		}`
 }
 
-func expectListenerJSON(t *testing.T, snap *proxycfg.ConfigSnapshot, token string, v, n uint64) string {
-	return expectListenerJSONFromResources(t, snap, token, v, n,
-		expectListenerJSONResources(t, snap, token, v, n))
+func expectListenerJSON(t *testing.T, snap *proxycfg.ConfigSnapshot, v, n uint64) string {
+	return expectListenerJSONFromResources(snap, v, n, expectListenerJSONResources(t, snap))
 }
 
 type customListenerJSONOptions struct {
-	Name          string
-	IncludeType   bool
-	OverrideAuthz bool
-	TLSContext    string
+	Name        string
+	IncludeType bool
+	TLSContext  string
 }
 
 const customListenerJSONTpl = `{
@@ -650,25 +713,6 @@ const customListenerJSONTpl = `{
 			"tlsContext": {{ .TLSContext }},
 			{{- end }}
 			"filters": [
-				{{ if .OverrideAuthz -}}
-				{
-					"name": "envoy.ext_authz",
-					"config": {
-							"grpc_service": {
-										"envoy_grpc": {
-													"cluster_name": "local_agent"
-												},
-										"initial_metadata": [
-													{
-																"key": "x-consul-token",
-																"value": "my-token"
-															}
-												]
-									},
-							"stat_prefix": "connect_authz"
-						}
-				},
-				{{- end }}
 				{
 					"name": "envoy.tcp_proxy",
 					"config": {
@@ -681,12 +725,98 @@ const customListenerJSONTpl = `{
 	]
 }`
 
-var customListenerJSONTemplate = template.Must(template.New("").Parse(customListenerJSONTpl))
+type customHTTPListenerJSONOptions struct {
+	Name                      string
+	HTTPConnectionManagerName string
+	TypedConfig               bool
+}
+
+const customHTTPListenerJSONTpl = `{
+	"name": "{{ .Name }}",
+	"address": {
+		"socketAddress": {
+			"address": "11.11.11.11",
+			"portValue": 11111
+		}
+	},
+	"filterChains": [
+		{
+			"filters": [
+				{
+					"name": "{{ .HTTPConnectionManagerName }}",
+					{{ if .TypedConfig -}}
+					"typedConfig": {
+					"@type": "type.googleapis.com/envoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager",
+					{{ else -}}
+					"config": {
+					{{- end }}
+						"http_filters": [
+							{
+								"name": "envoy.router"
+							}
+						],
+						"route_config": {
+							"name": "public_listener",
+							"virtual_hosts": [
+								{
+									"domains": [
+										"*"
+									],
+									"name": "public_listener",
+									"routes": [
+										{
+											"match": {
+												"prefix": "/"
+											},
+											"route": {
+												"cluster": "random-cluster"
+											}
+										}
+									]
+								}
+							]
+						}
+					}
+				}
+			]
+		}
+	]
+}`
+
+var (
+	customListenerJSONTemplate     = template.Must(template.New("").Parse(customListenerJSONTpl))
+	customHTTPListenerJSONTemplate = template.Must(template.New("").Parse(customHTTPListenerJSONTpl))
+)
 
 func customListenerJSON(t *testing.T, opts customListenerJSONOptions) string {
 	t.Helper()
 	var buf bytes.Buffer
-	err := customListenerJSONTemplate.Execute(&buf, opts)
-	require.NoError(t, err)
+	require.NoError(t, customListenerJSONTemplate.Execute(&buf, opts))
 	return buf.String()
 }
+
+func customHTTPListenerJSON(t *testing.T, opts customHTTPListenerJSONOptions) string {
+	t.Helper()
+	if opts.HTTPConnectionManagerName == "" {
+		opts.HTTPConnectionManagerName = wellknown.HTTPConnectionManager
+	}
+	var buf bytes.Buffer
+	require.NoError(t, customHTTPListenerJSONTemplate.Execute(&buf, opts))
+	return buf.String()
+}
+
+type httpCheckFetcherFunc func(serviceID structs.ServiceID) []structs.CheckType
+
+var _ HTTPCheckFetcher = (httpCheckFetcherFunc)(nil)
+
+func (f httpCheckFetcherFunc) ServiceHTTPBasedChecks(serviceID structs.ServiceID) []structs.CheckType {
+	return f(serviceID)
+}
+
+type configFetcherFunc func() string
+
+var _ ConfigFetcher = (configFetcherFunc)(nil)
+
+func (f configFetcherFunc) AdvertiseAddrLAN() string {
+	return f()
+}
