@@ -5,28 +5,33 @@ import (
 	"fmt"
 	"net"
 	"strings"
+	"time"
 
 	envoy "github.com/envoyproxy/go-control-plane/envoy/api/v2"
 	envoyroute "github.com/envoyproxy/go-control-plane/envoy/api/v2/route"
 	envoymatcher "github.com/envoyproxy/go-control-plane/envoy/type/matcher"
 	"github.com/golang/protobuf/proto"
 	"github.com/golang/protobuf/ptypes"
+	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/proxycfg"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/logging"
 )
 
 // routesFromSnapshot returns the xDS API representation of the "routes" in the
 // snapshot.
-func routesFromSnapshot(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
+func (s *Server) routesFromSnapshot(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
 	if cfgSnap == nil {
 		return nil, errors.New("nil config given")
 	}
 
 	switch cfgSnap.Kind {
 	case structs.ServiceKindConnectProxy:
-		return routesFromSnapshotConnectProxy(cInfo, cfgSnap)
+		return routesForConnectProxy(cInfo, cfgSnap.Proxy.Upstreams, cfgSnap.ConnectProxy.DiscoveryChain)
 	case structs.ServiceKindIngressGateway:
-		return routesFromSnapshotIngressGateway(cInfo, cfgSnap)
+		return routesForIngressGateway(cInfo, cfgSnap.IngressGateway.Upstreams, cfgSnap.IngressGateway.DiscoveryChain)
+	case structs.ServiceKindTerminatingGateway:
+		return s.routesFromSnapshotTerminatingGateway(cInfo, cfgSnap)
 	default:
 		return nil, fmt.Errorf("Invalid service kind: %v", cfgSnap.Kind)
 	}
@@ -34,18 +39,19 @@ func routesFromSnapshot(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot)
 
 // routesFromSnapshotConnectProxy returns the xDS API representation of the
 // "routes" in the snapshot.
-func routesFromSnapshotConnectProxy(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
-	if cfgSnap == nil {
-		return nil, errors.New("nil config given")
-	}
+func routesForConnectProxy(
+	cInfo connectionInfo,
+	upstreams structs.Upstreams,
+	chains map[string]*structs.CompiledDiscoveryChain,
+) ([]proto.Message, error) {
 
 	var resources []proto.Message
-	for _, u := range cfgSnap.Proxy.Upstreams {
+	for _, u := range upstreams {
 		upstreamID := u.Identifier()
 
 		var chain *structs.CompiledDiscoveryChain
 		if u.DestinationType != structs.UpstreamDestTypePreparedQuery {
-			chain = cfgSnap.ConnectProxy.DiscoveryChain[upstreamID]
+			chain = chains[upstreamID]
 		}
 
 		if chain == nil || chain.IsDefault() {
@@ -72,15 +78,106 @@ func routesFromSnapshotConnectProxy(cInfo connectionInfo, cfgSnap *proxycfg.Conf
 	return resources, nil
 }
 
-// routesFromSnapshotIngressGateway returns the xDS API representation of the
-// "routes" in the snapshot.
-func routesFromSnapshotIngressGateway(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
+// routesFromSnapshotTerminatingGateway returns the xDS API representation of the "routes" in the snapshot.
+// For any HTTP service we will return a default route.
+func (s *Server) routesFromSnapshotTerminatingGateway(_ connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
 	if cfgSnap == nil {
 		return nil, errors.New("nil config given")
 	}
+	logger := s.Logger.Named(logging.TerminatingGateway)
+
+	var resources []proto.Message
+	for _, svc := range cfgSnap.TerminatingGateway.ValidServices() {
+		clusterName := connect.ServiceSNI(svc.Name, "", svc.NamespaceOrDefault(), cfgSnap.Datacenter, cfgSnap.Roots.TrustDomain)
+		resolver, hasResolver := cfgSnap.TerminatingGateway.ServiceResolvers[svc]
+
+		svcConfig := cfgSnap.TerminatingGateway.ServiceConfigs[svc]
+
+		cfg, err := ParseProxyConfig(svcConfig.ProxyConfig)
+		if err != nil {
+			return nil, fmt.Errorf("failed to parse upstream config: %v", err)
+		}
+		if !structs.IsProtocolHTTPLike(cfg.Protocol) {
+			// Routes can only be defined for HTTP services
+			continue
+		}
+
+		if !hasResolver {
+			// Use a zero value resolver with no timeout and no subsets
+			resolver = &structs.ServiceResolverConfigEntry{}
+		}
+
+		var lb *structs.LoadBalancer
+		if resolver.LoadBalancer != nil {
+			lb = resolver.LoadBalancer
+		}
+		route, err := makeNamedDefaultRouteWithLB(clusterName, lb, true)
+		if err != nil {
+			logger.Error("failed to make route", "cluster", clusterName, "error", err)
+			continue
+		}
+		resources = append(resources, route)
+
+		// If there is a service-resolver for this service then also setup routes for each subset
+		for name := range resolver.Subsets {
+			clusterName = connect.ServiceSNI(svc.Name, name, svc.NamespaceOrDefault(), cfgSnap.Datacenter, cfgSnap.Roots.TrustDomain)
+			route, err := makeNamedDefaultRouteWithLB(clusterName, lb, true)
+			if err != nil {
+				logger.Error("failed to make route", "cluster", clusterName, "error", err)
+				continue
+			}
+			resources = append(resources, route)
+		}
+	}
+
+	return resources, nil
+}
+
+func makeNamedDefaultRouteWithLB(clusterName string, lb *structs.LoadBalancer, autoHostRewrite bool) (*envoy.RouteConfiguration, error) {
+	action := makeRouteActionFromName(clusterName)
+
+	if err := injectLBToRouteAction(lb, action.Route); err != nil {
+		return nil, fmt.Errorf("failed to apply load balancer configuration to route action: %v", err)
+	}
+
+	// Configure Envoy to rewrite Host header
+	if autoHostRewrite {
+		action.Route.HostRewriteSpecifier = &envoyroute.RouteAction_AutoHostRewrite{
+			AutoHostRewrite: makeBoolValue(true),
+		}
+	}
+
+	return &envoy.RouteConfiguration{
+		Name: clusterName,
+		VirtualHosts: []*envoyroute.VirtualHost{
+			{
+				Name:    clusterName,
+				Domains: []string{"*"},
+				Routes: []*envoyroute.Route{
+					{
+						Match:  makeDefaultRouteMatch(),
+						Action: action,
+					},
+				},
+			},
+		},
+		// ValidateClusters defaults to true when defined statically and false
+		// when done via RDS. Re-set the sane value of true to prevent
+		// null-routing traffic.
+		ValidateClusters: makeBoolValue(true),
+	}, nil
+}
+
+// routesForIngressGateway returns the xDS API representation of the
+// "routes" in the snapshot.
+func routesForIngressGateway(
+	cInfo connectionInfo,
+	upstreams map[proxycfg.IngressListenerKey]structs.Upstreams,
+	chains map[string]*structs.CompiledDiscoveryChain,
+) ([]proto.Message, error) {
 
 	var result []proto.Message
-	for listenerKey, upstreams := range cfgSnap.IngressGateway.Upstreams {
+	for listenerKey, upstreams := range upstreams {
 		// Do not create any route configuration for TCP listeners
 		if listenerKey.Protocol == "tcp" {
 			continue
@@ -95,7 +192,7 @@ func routesFromSnapshotIngressGateway(cInfo connectionInfo, cfgSnap *proxycfg.Co
 		}
 		for _, u := range upstreams {
 			upstreamID := u.Identifier()
-			chain := cfgSnap.IngressGateway.DiscoveryChain[upstreamID]
+			chain := chains[upstreamID]
 			if chain == nil {
 				continue
 			}
@@ -178,7 +275,7 @@ func makeUpstreamRouteForDiscoveryChain(
 		routes = make([]*envoyroute.Route, 0, len(startNode.Routes))
 
 		for _, discoveryRoute := range startNode.Routes {
-			routeMatch := makeRouteMatchForDiscoveryRoute(cInfo, discoveryRoute, chain.Protocol)
+			routeMatch := makeRouteMatchForDiscoveryRoute(cInfo, discoveryRoute)
 
 			var (
 				routeAction *envoyroute.Route_Route
@@ -186,6 +283,12 @@ func makeUpstreamRouteForDiscoveryChain(
 			)
 
 			nextNode := chain.Nodes[discoveryRoute.NextNode]
+
+			var lb *structs.LoadBalancer
+			if nextNode.LoadBalancer != nil {
+				lb = nextNode.LoadBalancer
+			}
+
 			switch nextNode.Type {
 			case structs.DiscoveryGraphNodeTypeSplitter:
 				routeAction, err = makeRouteActionForSplitter(nextNode.Splits, chain)
@@ -193,8 +296,16 @@ func makeUpstreamRouteForDiscoveryChain(
 					return nil, err
 				}
 
+				if err := injectLBToRouteAction(lb, routeAction.Route); err != nil {
+					return nil, fmt.Errorf("failed to apply load balancer configuration to route action: %v", err)
+				}
+
 			case structs.DiscoveryGraphNodeTypeResolver:
-				routeAction = makeRouteActionForSingleCluster(nextNode.Resolver.Target, chain)
+				routeAction = makeRouteActionForChainCluster(nextNode.Resolver.Target, chain)
+
+				if err := injectLBToRouteAction(lb, routeAction.Route); err != nil {
+					return nil, fmt.Errorf("failed to apply load balancer configuration to route action: %v", err)
+				}
 
 			default:
 				return nil, fmt.Errorf("unexpected graph node after route %q", nextNode.Type)
@@ -247,6 +358,14 @@ func makeUpstreamRouteForDiscoveryChain(
 			return nil, err
 		}
 
+		var lb *structs.LoadBalancer
+		if startNode.LoadBalancer != nil {
+			lb = startNode.LoadBalancer
+		}
+		if err := injectLBToRouteAction(lb, routeAction.Route); err != nil {
+			return nil, fmt.Errorf("failed to apply load balancer configuration to route action: %v", err)
+		}
+
 		defaultRoute := &envoyroute.Route{
 			Match:  makeDefaultRouteMatch(),
 			Action: routeAction,
@@ -255,7 +374,15 @@ func makeUpstreamRouteForDiscoveryChain(
 		routes = []*envoyroute.Route{defaultRoute}
 
 	case structs.DiscoveryGraphNodeTypeResolver:
-		routeAction := makeRouteActionForSingleCluster(startNode.Resolver.Target, chain)
+		routeAction := makeRouteActionForChainCluster(startNode.Resolver.Target, chain)
+
+		var lb *structs.LoadBalancer
+		if startNode.LoadBalancer != nil {
+			lb = startNode.LoadBalancer
+		}
+		if err := injectLBToRouteAction(lb, routeAction.Route); err != nil {
+			return nil, fmt.Errorf("failed to apply load balancer configuration to route action: %v", err)
+		}
 
 		defaultRoute := &envoyroute.Route{
 			Match:  makeDefaultRouteMatch(),
@@ -277,7 +404,7 @@ func makeUpstreamRouteForDiscoveryChain(
 	return host, nil
 }
 
-func makeRouteMatchForDiscoveryRoute(cInfo connectionInfo, discoveryRoute *structs.DiscoveryRoute, protocol string) *envoyroute.RouteMatch {
+func makeRouteMatchForDiscoveryRoute(_ connectionInfo, discoveryRoute *structs.DiscoveryRoute) *envoyroute.RouteMatch {
 	match := discoveryRoute.Definition.Match
 	if match == nil || match.IsEmpty() {
 		return makeDefaultRouteMatch()
@@ -295,14 +422,8 @@ func makeRouteMatchForDiscoveryRoute(cInfo connectionInfo, discoveryRoute *struc
 			Prefix: match.HTTP.PathPrefix,
 		}
 	case match.HTTP.PathRegex != "":
-		if cInfo.ProxyFeatures.RouterMatchSafeRegex {
-			em.PathSpecifier = &envoyroute.RouteMatch_SafeRegex{
-				SafeRegex: makeEnvoyRegexMatch(match.HTTP.PathRegex),
-			}
-		} else {
-			em.PathSpecifier = &envoyroute.RouteMatch_Regex{
-				Regex: match.HTTP.PathRegex,
-			}
+		em.PathSpecifier = &envoyroute.RouteMatch_SafeRegex{
+			SafeRegex: makeEnvoyRegexMatch(match.HTTP.PathRegex),
 		}
 	default:
 		em.PathSpecifier = &envoyroute.RouteMatch_Prefix{
@@ -323,14 +444,8 @@ func makeRouteMatchForDiscoveryRoute(cInfo connectionInfo, discoveryRoute *struc
 					ExactMatch: hdr.Exact,
 				}
 			case hdr.Regex != "":
-				if cInfo.ProxyFeatures.RouterMatchSafeRegex {
-					eh.HeaderMatchSpecifier = &envoyroute.HeaderMatcher_SafeRegexMatch{
-						SafeRegexMatch: makeEnvoyRegexMatch(hdr.Regex),
-					}
-				} else {
-					eh.HeaderMatchSpecifier = &envoyroute.HeaderMatcher_RegexMatch{
-						RegexMatch: hdr.Regex,
-					}
+				eh.HeaderMatchSpecifier = &envoyroute.HeaderMatcher_SafeRegexMatch{
+					SafeRegexMatch: makeEnvoyRegexMatch(hdr.Regex),
 				}
 			case hdr.Prefix != "":
 				eh.HeaderMatchSpecifier = &envoyroute.HeaderMatcher_PrefixMatch{
@@ -361,15 +476,9 @@ func makeRouteMatchForDiscoveryRoute(cInfo connectionInfo, discoveryRoute *struc
 
 		eh := &envoyroute.HeaderMatcher{
 			Name: ":method",
-		}
-		if cInfo.ProxyFeatures.RouterMatchSafeRegex {
-			eh.HeaderMatchSpecifier = &envoyroute.HeaderMatcher_SafeRegexMatch{
+			HeaderMatchSpecifier: &envoyroute.HeaderMatcher_SafeRegexMatch{
 				SafeRegexMatch: makeEnvoyRegexMatch(methodHeaderRegex),
-			}
-		} else {
-			eh.HeaderMatchSpecifier = &envoyroute.HeaderMatcher_RegexMatch{
-				RegexMatch: methodHeaderRegex,
-			}
+			},
 		}
 
 		em.Headers = append(em.Headers, eh)
@@ -384,37 +493,24 @@ func makeRouteMatchForDiscoveryRoute(cInfo connectionInfo, discoveryRoute *struc
 
 			switch {
 			case qm.Exact != "":
-				if cInfo.ProxyFeatures.RouterMatchSafeRegex {
-					eq.QueryParameterMatchSpecifier = &envoyroute.QueryParameterMatcher_StringMatch{
-						StringMatch: &envoymatcher.StringMatcher{
-							MatchPattern: &envoymatcher.StringMatcher_Exact{
-								Exact: qm.Exact,
-							},
+				eq.QueryParameterMatchSpecifier = &envoyroute.QueryParameterMatcher_StringMatch{
+					StringMatch: &envoymatcher.StringMatcher{
+						MatchPattern: &envoymatcher.StringMatcher_Exact{
+							Exact: qm.Exact,
 						},
-					}
-				} else {
-					eq.Value = qm.Exact
+					},
 				}
 			case qm.Regex != "":
-				if cInfo.ProxyFeatures.RouterMatchSafeRegex {
-					eq.QueryParameterMatchSpecifier = &envoyroute.QueryParameterMatcher_StringMatch{
-						StringMatch: &envoymatcher.StringMatcher{
-							MatchPattern: &envoymatcher.StringMatcher_SafeRegex{
-								SafeRegex: makeEnvoyRegexMatch(qm.Regex),
-							},
+				eq.QueryParameterMatchSpecifier = &envoyroute.QueryParameterMatcher_StringMatch{
+					StringMatch: &envoymatcher.StringMatcher{
+						MatchPattern: &envoymatcher.StringMatcher_SafeRegex{
+							SafeRegex: makeEnvoyRegexMatch(qm.Regex),
 						},
-					}
-				} else {
-					eq.Value = qm.Regex
-					eq.Regex = makeBoolValue(true)
+					},
 				}
 			case qm.Present:
-				if cInfo.ProxyFeatures.RouterMatchSafeRegex {
-					eq.QueryParameterMatchSpecifier = &envoyroute.QueryParameterMatcher_PresentMatch{
-						PresentMatch: true,
-					}
-				} else {
-					eq.Value = ""
+				eq.QueryParameterMatchSpecifier = &envoyroute.QueryParameterMatcher_PresentMatch{
+					PresentMatch: true,
 				}
 			default:
 				continue // skip this impossible situation
@@ -440,11 +536,12 @@ func makeDefaultRouteMatch() *envoyroute.RouteMatch {
 	}
 }
 
-func makeRouteActionForSingleCluster(targetID string, chain *structs.CompiledDiscoveryChain) *envoyroute.Route_Route {
+func makeRouteActionForChainCluster(targetID string, chain *structs.CompiledDiscoveryChain) *envoyroute.Route_Route {
 	target := chain.Targets[targetID]
+	return makeRouteActionFromName(CustomizeClusterName(target.Name, chain))
+}
 
-	clusterName := CustomizeClusterName(target.Name, chain)
-
+func makeRouteActionFromName(clusterName string) *envoyroute.Route_Route {
 	return &envoyroute.Route_Route{
 		Route: &envoyroute.RouteAction{
 			ClusterSpecifier: &envoyroute.RouteAction_Cluster{
@@ -490,11 +587,71 @@ func makeRouteActionForSplitter(splits []*structs.DiscoverySplit, chain *structs
 	}, nil
 }
 
-func makeEnvoyRegexMatch(patt string) *envoymatcher.RegexMatcher {
-	return &envoymatcher.RegexMatcher{
-		EngineType: &envoymatcher.RegexMatcher_GoogleRe2{
-			GoogleRe2: &envoymatcher.RegexMatcher_GoogleRE2{},
-		},
-		Regex: patt,
+func injectLBToRouteAction(lb *structs.LoadBalancer, action *envoyroute.RouteAction) error {
+	if lb == nil || !lb.IsHashBased() {
+		return nil
 	}
+
+	result := make([]*envoyroute.RouteAction_HashPolicy, 0, len(lb.HashPolicies))
+	for _, policy := range lb.HashPolicies {
+		if policy.SourceIP {
+			result = append(result, &envoyroute.RouteAction_HashPolicy{
+				PolicySpecifier: &envoyroute.RouteAction_HashPolicy_ConnectionProperties_{
+					ConnectionProperties: &envoyroute.RouteAction_HashPolicy_ConnectionProperties{
+						SourceIp: true,
+					},
+				},
+				Terminal: policy.Terminal,
+			})
+
+			continue
+		}
+
+		switch policy.Field {
+		case structs.HashPolicyHeader:
+			result = append(result, &envoyroute.RouteAction_HashPolicy{
+				PolicySpecifier: &envoyroute.RouteAction_HashPolicy_Header_{
+					Header: &envoyroute.RouteAction_HashPolicy_Header{
+						HeaderName: policy.FieldValue,
+					},
+				},
+				Terminal: policy.Terminal,
+			})
+		case structs.HashPolicyCookie:
+			cookie := envoyroute.RouteAction_HashPolicy_Cookie{
+				Name: policy.FieldValue,
+			}
+			if policy.CookieConfig != nil {
+				cookie.Path = policy.CookieConfig.Path
+
+				if policy.CookieConfig.TTL != 0*time.Second {
+					cookie.Ttl = ptypes.DurationProto(policy.CookieConfig.TTL)
+				}
+
+				// Envoy will generate a session cookie if the ttl is present and zero.
+				if policy.CookieConfig.Session {
+					cookie.Ttl = ptypes.DurationProto(0 * time.Second)
+				}
+			}
+			result = append(result, &envoyroute.RouteAction_HashPolicy{
+				PolicySpecifier: &envoyroute.RouteAction_HashPolicy_Cookie_{
+					Cookie: &cookie,
+				},
+				Terminal: policy.Terminal,
+			})
+		case structs.HashPolicyQueryParam:
+			result = append(result, &envoyroute.RouteAction_HashPolicy{
+				PolicySpecifier: &envoyroute.RouteAction_HashPolicy_QueryParameter_{
+					QueryParameter: &envoyroute.RouteAction_HashPolicy_QueryParameter{
+						Name: policy.FieldValue,
+					},
+				},
+				Terminal: policy.Terminal,
+			})
+		default:
+			return fmt.Errorf("unsupported load balancer hash policy field: %v", policy.Field)
+		}
+	}
+	action.HashPolicy = result
+	return nil
 }
