@@ -6,7 +6,6 @@ import (
 	"fmt"
 	"io"
 	"net/http"
-	"net/url"
 )
 
 // ServiceKind is the kind of service being registered.
@@ -93,6 +92,8 @@ type AgentService struct {
 	// to include the Namespace in the hash. When we do, then we are in for lots of fun with tests.
 	// For now though, ignoring it works well enough.
 	Namespace string `json:",omitempty" bexpr:"-" hash:"ignore"`
+	// Datacenter is only ever returned and is ignored if presented.
+	Datacenter string `json:",omitempty" bexpr:"-" hash:"ignore"`
 }
 
 // AgentServiceChecksInfo returns information about a Service and its checks
@@ -162,7 +163,7 @@ const (
 	// MemberTagKeyReadReplica is the key used to indicate that the member is a read
 	// replica server (will remain a Raft non-voter).
 	// Read Replicas are a Consul Enterprise feature.
-	MemberTagKeyReadReplica = "nonvoter"
+	MemberTagKeyReadReplica = "read_replica"
 	// MemberTagValueReadReplica is the value of the MemberTagKeyReadReplica key when
 	// the member is in fact a read-replica. Any other value indicates that it is not.
 	// Read Replicas are a Consul Enterprise feature.
@@ -187,10 +188,18 @@ const (
 
 // AgentMember represents a cluster member known to the agent
 type AgentMember struct {
-	Name        string
-	Addr        string
-	Port        uint16
-	Tags        map[string]string
+	Name string
+	Addr string
+	Port uint16
+	Tags map[string]string
+	// Status of the Member which corresponds to  github.com/hashicorp/serf/serf.MemberStatus
+	// Value is one of:
+	//
+	// 	  AgentMemberNone    = 0
+	//	  AgentMemberAlive   = 1
+	//	  AgentMemberLeaving = 2
+	//	  AgentMemberLeft    = 3
+	//	  AgentMemberFailed  = 4
 	Status      int
 	ProtocolMin uint8
 	ProtocolMax uint8
@@ -256,11 +265,11 @@ type AgentServiceRegistration struct {
 	Namespace         string                          `json:",omitempty" bexpr:"-" hash:"ignore"`
 }
 
-//ServiceRegisterOpts is used to pass extra options to the service register.
+// ServiceRegisterOpts is used to pass extra options to the service register.
 type ServiceRegisterOpts struct {
-	//Missing healthchecks will be deleted from the agent.
-	//Using this parameter allows to idempotently register a service and its checks without
-	//having to manually deregister checks.
+	// Missing healthchecks will be deleted from the agent.
+	// Using this parameter allows to idempotently register a service and its checks without
+	// having to manually deregister checks.
 	ReplaceExistingChecks bool
 }
 
@@ -528,7 +537,7 @@ func (a *Agent) ServicesWithFilter(filter string) (map[string]*AgentService, err
 // - If the service is found, will return (critical|passing|warning), AgentServiceChecksInfo, nil)
 // - In all other cases, will return an error
 func (a *Agent) AgentHealthServiceByID(serviceID string) (string, *AgentServiceChecksInfo, error) {
-	path := fmt.Sprintf("/v1/agent/health/service/id/%v", url.PathEscape(serviceID))
+	path := fmt.Sprintf("/v1/agent/health/service/id/%v", serviceID)
 	r := a.c.newRequest("GET", path)
 	r.params.Add("format", "json")
 	r.header.Set("Accept", "application/json")
@@ -562,7 +571,7 @@ func (a *Agent) AgentHealthServiceByID(serviceID string) (string, *AgentServiceC
 // - If the service is found, will return (critical|passing|warning), []api.AgentServiceChecksInfo, nil)
 // - In all other cases, will return an error
 func (a *Agent) AgentHealthServiceByName(service string) (string, []AgentServiceChecksInfo, error) {
-	path := fmt.Sprintf("/v1/agent/health/service/name/%v", url.PathEscape(service))
+	path := fmt.Sprintf("/v1/agent/health/service/name/%v", service)
 	r := a.c.newRequest("GET", path)
 	r.params.Add("format", "json")
 	r.header.Set("Accept", "application/json")
@@ -863,8 +872,8 @@ func (a *Agent) ForceLeave(node string) error {
 	return nil
 }
 
-//ForceLeavePrune is used to have an a failed agent removed
-//from the list of members
+// ForceLeavePrune is used to have an a failed agent removed
+// from the list of members
 func (a *Agent) ForceLeavePrune(node string) error {
 	r := a.c.newRequest("PUT", "/v1/agent/force-leave/"+node)
 	r.params.Set("prune", "1")
