@@ -99,7 +99,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 	t.Helper()
 
 	registrations := map[string]*CatalogRegistration{
-		"Node foo": &CatalogRegistration{
+		"Node foo": {
 			Datacenter: datacenter,
 			Node:       "foo",
 			ID:         "e0155642-135d-4739-9853-a1ee6c9f945b",
@@ -129,7 +129,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Service redis v1 on foo": &CatalogRegistration{
+		"Service redis v1 on foo": {
 			Datacenter:     datacenter,
 			Node:           "foo",
 			SkipNodeUpdate: true,
@@ -154,7 +154,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Service redis v2 on foo": &CatalogRegistration{
+		"Service redis v2 on foo": {
 			Datacenter:     datacenter,
 			Node:           "foo",
 			SkipNodeUpdate: true,
@@ -179,7 +179,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Node bar": &CatalogRegistration{
+		"Node bar": {
 			Datacenter: datacenter,
 			Node:       "bar",
 			ID:         "c6e7a976-8f4f-44b5-bdd3-631be7e8ecac",
@@ -202,7 +202,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Service redis v1 on bar": &CatalogRegistration{
+		"Service redis v1 on bar": {
 			Datacenter:     datacenter,
 			Node:           "bar",
 			SkipNodeUpdate: true,
@@ -227,7 +227,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Service web v1 on bar": &CatalogRegistration{
+		"Service web v1 on bar": {
 			Datacenter:     datacenter,
 			Node:           "bar",
 			SkipNodeUpdate: true,
@@ -253,7 +253,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Node baz": &CatalogRegistration{
+		"Node baz": {
 			Datacenter: datacenter,
 			Node:       "baz",
 			ID:         "12f96b27-a7b0-47bd-add7-044a2bfc7bfb",
@@ -282,7 +282,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Service web v1 on baz": &CatalogRegistration{
+		"Service web v1 on baz": {
 			Datacenter:     datacenter,
 			Node:           "baz",
 			SkipNodeUpdate: true,
@@ -308,7 +308,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Service web v2 on baz": &CatalogRegistration{
+		"Service web v2 on baz": {
 			Datacenter:     datacenter,
 			Node:           "baz",
 			SkipNodeUpdate: true,
@@ -334,7 +334,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Service critical on baz": &CatalogRegistration{
+		"Service critical on baz": {
 			Datacenter:     datacenter,
 			Node:           "baz",
 			SkipNodeUpdate: true,
@@ -359,7 +359,7 @@ func testNodeServiceCheckRegistrations(t *testing.T, client *Client, datacenter
 				},
 			},
 		},
-		"Service warning on baz": &CatalogRegistration{
+		"Service warning on baz": {
 			Datacenter:     datacenter,
 			Node:           "baz",
 			SkipNodeUpdate: true,
@@ -399,6 +399,8 @@ func TestAPI_DefaultConfig_env(t *testing.T) {
 	// (environment) which has non-deterministic effects on the other tests
 	// which derive their default configuration from the environment
 
+	// if this test is failing because of expired certificates
+	// use the procedure in test/CA-GENERATION.md
 	addr := "1.2.3.4:5678"
 	token := "abcd1234"
 	auth := "username:password"
@@ -480,6 +482,8 @@ func TestAPI_DefaultConfig_env(t *testing.T) {
 }
 
 func TestAPI_SetupTLSConfig(t *testing.T) {
+	// if this test is failing because of expired certificates
+	// use the procedure in test/CA-GENERATION.md
 	t.Parallel()
 	// A default config should result in a clean default client config.
 	tlsConfig := &TLSConfig{}
@@ -840,6 +844,7 @@ func TestAPI_ParseQueryMeta(t *testing.T) {
 	resp.Header.Set("X-Consul-LastContact", "80")
 	resp.Header.Set("X-Consul-KnownLeader", "true")
 	resp.Header.Set("X-Consul-Translate-Addresses", "true")
+	resp.Header.Set("X-Consul-Default-ACL-Policy", "deny")
 
 	qm := &QueryMeta{}
 	if err := parseQueryMeta(resp, qm); err != nil {
@@ -858,6 +863,9 @@ func TestAPI_ParseQueryMeta(t *testing.T) {
 	if !qm.AddressTranslationEnabled {
 		t.Fatalf("Bad: %v", qm)
 	}
+	if qm.DefaultACLPolicy != "deny" {
+		t.Fatalf("Bad: %v", qm)
+	}
 }
 
 func TestAPI_UnixSocket(t *testing.T) {
@@ -867,7 +875,6 @@ func TestAPI_UnixSocket(t *testing.T) {
 	}
 
 	tempDir := testutil.TempDir(t, "consul")
-	defer os.RemoveAll(tempDir)
 	socket := filepath.Join(tempDir, "test.sock")
 
 	c, s := makeClientWithConfig(t, func(c *Config) {
