@@ -3,13 +3,11 @@ package roleread
 import (
 	"encoding/json"
 	"fmt"
-	"os"
 	"strings"
 	"testing"
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/api"
-	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/hashicorp/go-uuid"
 	"github.com/mitchellh/cli"
@@ -28,9 +26,6 @@ func TestRoleReadCommand_noTabs(t *testing.T) {
 func TestRoleReadCommand(t *testing.T) {
 	t.Parallel()
 
-	testDir := testutil.TempDir(t, "acl")
-	defer os.RemoveAll(testDir)
-
 	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
@@ -98,7 +93,7 @@ func TestRoleReadCommand(t *testing.T) {
 			&api.ACLRole{
 				Name: "test-role-by-id",
 				ServiceIdentities: []*api.ACLServiceIdentity{
-					&api.ACLServiceIdentity{
+					{
 						ServiceName: "fake",
 					},
 				},
@@ -131,7 +126,7 @@ func TestRoleReadCommand(t *testing.T) {
 			&api.ACLRole{
 				Name: "test-role-by-id-prefix",
 				ServiceIdentities: []*api.ACLServiceIdentity{
-					&api.ACLServiceIdentity{
+					{
 						ServiceName: "fake",
 					},
 				},
@@ -164,7 +159,7 @@ func TestRoleReadCommand(t *testing.T) {
 			&api.ACLRole{
 				Name: "test-role-by-name",
 				ServiceIdentities: []*api.ACLServiceIdentity{
-					&api.ACLServiceIdentity{
+					{
 						ServiceName: "fake",
 					},
 				},
@@ -195,9 +190,6 @@ func TestRoleReadCommand(t *testing.T) {
 func TestRoleReadCommand_JSON(t *testing.T) {
 	t.Parallel()
 
-	testDir := testutil.TempDir(t, "acl")
-	defer os.RemoveAll(testDir)
-
 	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
@@ -218,7 +210,7 @@ func TestRoleReadCommand_JSON(t *testing.T) {
 			&api.ACLRole{
 				Name: "test-role-by-id",
 				ServiceIdentities: []*api.ACLServiceIdentity{
-					&api.ACLServiceIdentity{
+					{
 						ServiceName: "fake",
 					},
 				},
