@@ -2,13 +2,11 @@ package roleupdate
 
 import (
 	"encoding/json"
-	"os"
 	"strings"
 	"testing"
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/api"
-	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/mitchellh/cli"
 	"github.com/stretchr/testify/assert"
@@ -28,9 +26,6 @@ func TestRoleUpdateCommand_noTabs(t *testing.T) {
 func TestRoleUpdateCommand(t *testing.T) {
 	t.Parallel()
 
-	testDir := testutil.TempDir(t, "acl")
-	defer os.RemoveAll(testDir)
-
 	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
@@ -62,7 +57,7 @@ func TestRoleUpdateCommand(t *testing.T) {
 		&api.ACLRole{
 			Name: "test-role",
 			ServiceIdentities: []*api.ACLServiceIdentity{
-				&api.ACLServiceIdentity{
+				{
 					ServiceName: "fake",
 				},
 			},
@@ -201,9 +196,6 @@ func TestRoleUpdateCommand(t *testing.T) {
 func TestRoleUpdateCommand_JSON(t *testing.T) {
 	t.Parallel()
 
-	testDir := testutil.TempDir(t, "acl")
-	defer os.RemoveAll(testDir)
-
 	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
@@ -229,7 +221,7 @@ func TestRoleUpdateCommand_JSON(t *testing.T) {
 		&api.ACLRole{
 			Name: "test-role",
 			ServiceIdentities: []*api.ACLServiceIdentity{
-				&api.ACLServiceIdentity{
+				{
 					ServiceName: "fake",
 				},
 			},
@@ -283,9 +275,6 @@ func TestRoleUpdateCommand_JSON(t *testing.T) {
 func TestRoleUpdateCommand_noMerge(t *testing.T) {
 	t.Parallel()
 
-	testDir := testutil.TempDir(t, "acl")
-	defer os.RemoveAll(testDir)
-
 	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
@@ -327,12 +316,12 @@ func TestRoleUpdateCommand_noMerge(t *testing.T) {
 				Name:        "test-role-" + roleUnq,
 				Description: "original description",
 				ServiceIdentities: []*api.ACLServiceIdentity{
-					&api.ACLServiceIdentity{
+					{
 						ServiceName: "fake",
 					},
 				},
 				Policies: []*api.ACLRolePolicyLink{
-					&api.ACLRolePolicyLink{
+					{
 						ID: policy3.ID,
 					},
 				},
