@@ -2,13 +2,11 @@ package tokenupdate
 
 import (
 	"encoding/json"
-	"os"
 	"strings"
 	"testing"
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/api"
-	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/mitchellh/cli"
@@ -27,9 +25,6 @@ func TestTokenUpdateCommand_noTabs(t *testing.T) {
 func TestTokenUpdateCommand(t *testing.T) {
 	t.Parallel()
 
-	testDir := testutil.TempDir(t, "acl")
-	defer os.RemoveAll(testDir)
-
 	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
@@ -112,11 +107,11 @@ func TestTokenUpdateCommand(t *testing.T) {
 
 		require.Len(t, token.NodeIdentities, 2)
 		expected := []*api.ACLNodeIdentity{
-			&api.ACLNodeIdentity{
+			{
 				NodeName:   "foo",
 				Datacenter: "bar",
 			},
-			&api.ACLNodeIdentity{
+			{
 				NodeName:   "bar",
 				Datacenter: "baz",
 			},
@@ -199,9 +194,6 @@ func TestTokenUpdateCommand_JSON(t *testing.T) {
 	// Alias because we need to access require package in Retry below
 	req := require.New(t)
 
-	testDir := testutil.TempDir(t, "acl")
-	defer os.RemoveAll(testDir)
-
 	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
