@@ -2,18 +2,19 @@ package agent
 
 import (
 	"fmt"
+	"io"
 	"os"
 	"os/exec"
 	"path/filepath"
 	"strings"
 	"testing"
 
-	"github.com/hashicorp/consul/testrpc"
+	"github.com/mitchellh/cli"
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
-	"github.com/mitchellh/cli"
+	"github.com/hashicorp/consul/testrpc"
 )
 
 // TestConfigFail should test command line flags that lead to an immediate error.
@@ -21,7 +22,6 @@ func TestConfigFail(t *testing.T) {
 	t.Parallel()
 
 	dataDir := testutil.TempDir(t, "consul")
-	defer os.RemoveAll(dataDir)
 
 	tests := []struct {
 		args []string
@@ -108,10 +108,9 @@ func TestRetryJoin(t *testing.T) {
 func TestRetryJoinFail(t *testing.T) {
 	t.Parallel()
 	tmpDir := testutil.TempDir(t, "consul")
-	defer os.RemoveAll(tmpDir)
 
-	ui := cli.NewMockUi()
-	cmd := New(ui, "", "", "", "", nil)
+	ui := newCaptureUI()
+	cmd := New(ui)
 
 	args := []string{
 		"-bind", "127.0.0.1",
@@ -129,10 +128,9 @@ func TestRetryJoinFail(t *testing.T) {
 func TestRetryJoinWanFail(t *testing.T) {
 	t.Parallel()
 	tmpDir := testutil.TempDir(t, "consul")
-	defer os.RemoveAll(tmpDir)
 
-	ui := cli.NewMockUi()
-	cmd := New(ui, "", "", "", "", nil)
+	ui := newCaptureUI()
+	cmd := New(ui)
 
 	args := []string{
 		"-server",
@@ -151,14 +149,12 @@ func TestRetryJoinWanFail(t *testing.T) {
 func TestProtectDataDir(t *testing.T) {
 	t.Parallel()
 	dir := testutil.TempDir(t, "consul")
-	defer os.RemoveAll(dir)
 
 	if err := os.MkdirAll(filepath.Join(dir, "mdb"), 0700); err != nil {
 		t.Fatalf("err: %v", err)
 	}
 
 	cfgDir := testutil.TempDir(t, "consul-config")
-	defer os.RemoveAll(cfgDir)
 
 	cfgFilePath := filepath.Join(cfgDir, "consul.json")
 	cfgFile, err := os.Create(cfgFilePath)
@@ -172,8 +168,8 @@ func TestProtectDataDir(t *testing.T) {
 		t.Fatalf("err: %v", err)
 	}
 
-	ui := cli.NewMockUi()
-	cmd := New(ui, "", "", "", "", nil)
+	ui := newCaptureUI()
+	cmd := New(ui)
 	args := []string{"-config-file=" + cfgFile.Name()}
 	if code := cmd.Run(args); code == 0 {
 		t.Fatalf("should fail")
@@ -186,16 +182,13 @@ func TestProtectDataDir(t *testing.T) {
 func TestBadDataDirPermissions(t *testing.T) {
 	t.Parallel()
 	dir := testutil.TempDir(t, "consul")
-	defer os.RemoveAll(dir)
-
 	dataDir := filepath.Join(dir, "mdb")
 	if err := os.MkdirAll(dataDir, 0400); err != nil {
 		t.Fatalf("err: %v", err)
 	}
-	defer os.RemoveAll(dataDir)
 
-	ui := cli.NewMockUi()
-	cmd := New(ui, "", "", "", "", nil)
+	ui := newCaptureUI()
+	cmd := New(ui)
 	args := []string{"-data-dir=" + dataDir, "-server=true", "-bind=10.0.0.1"}
 	if code := cmd.Run(args); code == 0 {
 		t.Fatalf("Should fail with bad data directory permissions")
@@ -204,3 +197,19 @@ func TestBadDataDirPermissions(t *testing.T) {
 		t.Fatalf("expected permission denied error, got: %s", out)
 	}
 }
+
+type captureUI struct {
+	*cli.MockUi
+}
+
+func (c *captureUI) Stdout() io.Writer {
+	return c.MockUi.OutputWriter
+}
+
+func (c *captureUI) Stderr() io.Writer {
+	return c.MockUi.ErrorWriter
+}
+
+func newCaptureUI() *captureUI {
+	return &captureUI{MockUi: cli.NewMockUi()}
+}
