@@ -3,8 +3,10 @@
 package envoy
 
 import (
+	"bytes"
 	"encoding/json"
 	"fmt"
+	"io"
 	"io/ioutil"
 	"os"
 	"os/exec"
@@ -29,8 +31,6 @@ func TestExecEnvoy(t *testing.T) {
 				"--config-path",
 				"{{ got.ConfigPath }}",
 				"--disable-hot-restart",
-				"--max-obj-name-len",
-				"256",
 				"--fake-envoy-arg",
 			},
 		},
@@ -44,8 +44,6 @@ func TestExecEnvoy(t *testing.T) {
 				// that value further below.
 				"{{ got.ConfigPath }}",
 				// No --disable-hot-restart
-				"--max-obj-name-len",
-				"256",
 				"--fake-envoy-arg",
 				"--restart-epoch",
 				"1",
@@ -61,8 +59,6 @@ func TestExecEnvoy(t *testing.T) {
 				// that value further below.
 				"{{ got.ConfigPath }}",
 				// No --disable-hot-restart
-				"--max-obj-name-len",
-				"256",
 				"--fake-envoy-arg",
 				// Restart epoch defaults to 0 if not given and not disabled.
 				"--drain-time-s",
@@ -79,8 +75,6 @@ func TestExecEnvoy(t *testing.T) {
 				// that value further below.
 				"{{ got.ConfigPath }}",
 				// No --disable-hot-restart
-				"--max-obj-name-len",
-				"256",
 				"--fake-envoy-arg",
 				// Restart epoch defaults to 0 if not given and not disabled.
 				"--parent-shutdown-time-s",
@@ -97,8 +91,6 @@ func TestExecEnvoy(t *testing.T) {
 				// that value further below.
 				"{{ got.ConfigPath }}",
 				// No --disable-hot-restart
-				"--max-obj-name-len",
-				"256",
 				"--fake-envoy-arg",
 				// Restart epoch defaults to 0 if not given and not disabled.
 				"--hot-restart-version",
@@ -196,11 +188,7 @@ func TestHelperProcess(t *testing.T) {
 
 		limitProcessLifetime(2 * time.Minute)
 
-		// This is another level of gross - we are relying on `consul` being on path
-		// and being the correct version but in general that is true under `make
-		// test`. We already make the same assumption for API package tests.
-		testSelfExecOverride = "consul"
-
+		patchExecArgs(t)
 		err := execEnvoy(
 			os.Args[0],
 			[]string{
@@ -271,3 +259,37 @@ func limitProcessLifetime(dur time.Duration) {
 		os.Exit(99)
 	})
 }
+
+// patchExecArgs to use a version that will execute the commands using 'go run'.
+// Also sets up a cleanup function to revert the patch when the test exits.
+func patchExecArgs(t *testing.T) {
+	orig := execArgs
+	// go run will run the consul source from the root of the repo. The relative
+	// path is necessary because `go test` always sets the working directory to
+	// the directory of the package being tested.
+	execArgs = func(args ...string) (string, []string, error) {
+		args = append([]string{"run", "../../.."}, args...)
+		return "go", args, nil
+	}
+	t.Cleanup(func() {
+		execArgs = orig
+	})
+}
+
+func TestMakeBootstrapPipe_DoesNotBlockOnAFullPipe(t *testing.T) {
+	// A named pipe can buffer up to 64k, use a value larger than that
+	bootstrap := bytes.Repeat([]byte("a"), 66000)
+
+	patchExecArgs(t)
+	pipe, err := makeBootstrapPipe(bootstrap)
+	require.NoError(t, err)
+
+	// Read everything from the named pipe, to allow the sub-process to exit
+	f, err := os.Open(pipe)
+	require.NoError(t, err)
+
+	var buf bytes.Buffer
+	_, err = io.Copy(&buf, f)
+	require.NoError(t, err)
+	require.Equal(t, bootstrap, buf.Bytes())
+}
