@@ -1,25 +1,25 @@
 package create
 
 import (
-	"os"
 	"strings"
 	"testing"
 
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/sdk/testutil"
+	"github.com/hashicorp/consul/testrpc"
 	"github.com/mitchellh/cli"
 	"github.com/stretchr/testify/require"
 )
 
-func TestCommand_noTabs(t *testing.T) {
+func TestIntentionCreate_noTabs(t *testing.T) {
 	t.Parallel()
 	if strings.ContainsRune(New(nil).Help(), '\t') {
 		t.Fatal("help has tabs")
 	}
 }
 
-func TestCommand_Validation(t *testing.T) {
+func TestIntentionCreate_Validation(t *testing.T) {
 	t.Parallel()
 
 	ui := cli.NewMockUi()
@@ -56,7 +56,7 @@ func TestCommand_Validation(t *testing.T) {
 	}
 }
 
-func TestCommand(t *testing.T) {
+func TestIntentionCreate(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
@@ -64,6 +64,8 @@ func TestCommand(t *testing.T) {
 	defer a.Shutdown()
 	client := a.Client()
 
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
 	ui := cli.NewMockUi()
 	c := New(ui)
 
@@ -81,7 +83,7 @@ func TestCommand(t *testing.T) {
 	require.Equal(api.IntentionActionAllow, ixns[0].Action)
 }
 
-func TestCommand_deny(t *testing.T) {
+func TestIntentionCreate_deny(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
@@ -89,6 +91,8 @@ func TestCommand_deny(t *testing.T) {
 	defer a.Shutdown()
 	client := a.Client()
 
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
 	ui := cli.NewMockUi()
 	c := New(ui)
 
@@ -107,7 +111,7 @@ func TestCommand_deny(t *testing.T) {
 	require.Equal(api.IntentionActionDeny, ixns[0].Action)
 }
 
-func TestCommand_meta(t *testing.T) {
+func TestIntentionCreate_meta(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
@@ -115,6 +119,8 @@ func TestCommand_meta(t *testing.T) {
 	defer a.Shutdown()
 	client := a.Client()
 
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
 	ui := cli.NewMockUi()
 	c := New(ui)
 
@@ -133,7 +139,7 @@ func TestCommand_meta(t *testing.T) {
 	require.Equal(map[string]string{"hello": "world"}, ixns[0].Meta)
 }
 
-func TestCommand_File(t *testing.T) {
+func TestIntentionCreate_File(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
@@ -141,12 +147,13 @@ func TestCommand_File(t *testing.T) {
 	defer a.Shutdown()
 	client := a.Client()
 
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
 	ui := cli.NewMockUi()
 	c := New(ui)
 
 	contents := `{ "SourceName": "foo", "DestinationName": "bar", "Action": "allow" }`
 	f := testutil.TempFile(t, "intention-create-command-file")
-	defer os.Remove(f.Name())
 	if _, err := f.WriteString(contents); err != nil {
 		t.Fatalf("err: %#v", err)
 	}
@@ -167,13 +174,56 @@ func TestCommand_File(t *testing.T) {
 	require.Equal(api.IntentionActionAllow, ixns[0].Action)
 }
 
-func TestCommand_FileNoExist(t *testing.T) {
+func TestIntentionCreate_File_L7_fails(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
 	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
+	ui := cli.NewMockUi()
+	c := New(ui)
+
+	contents := `
+{
+  "SourceName": "foo",
+  "DestinationName": "bar",
+  "Permissions": [
+    {
+      "Action": "allow",
+      "HTTP": {
+        "PathExact": "/foo"
+      }
+    }
+  ]
+}
+	`
+	f := testutil.TempFile(t, "intention-create-command-file")
+	if _, err := f.WriteString(contents); err != nil {
+		t.Fatalf("err: %#v", err)
+	}
+
+	args := []string{
+		"-http-addr=" + a.HTTPAddr(),
+		"-file",
+		f.Name(),
+	}
+
+	require.Equal(1, c.Run(args), ui.ErrorWriter.String())
+	require.Contains(ui.ErrorWriter.String(), "cannot create L7 intention from file")
+}
+
+func TestIntentionCreate_FileNoExist(t *testing.T) {
+	t.Parallel()
+
+	require := require.New(t)
+	a := agent.NewTestAgent(t, ``)
+	defer a.Shutdown()
+
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
 	ui := cli.NewMockUi()
 	c := New(ui)
 
@@ -187,7 +237,7 @@ func TestCommand_FileNoExist(t *testing.T) {
 	require.Contains(ui.ErrorWriter.String(), "no such file")
 }
 
-func TestCommand_replace(t *testing.T) {
+func TestIntentionCreate_replace(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
@@ -195,6 +245,8 @@ func TestCommand_replace(t *testing.T) {
 	defer a.Shutdown()
 	client := a.Client()
 
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
 	// Create the first
 	{
 		ui := cli.NewMockUi()
@@ -225,7 +277,7 @@ func TestCommand_replace(t *testing.T) {
 			"foo", "bar",
 		}
 		require.Equal(1, c.Run(args), ui.ErrorWriter.String())
-		require.Contains(ui.ErrorWriter.String(), "duplicate")
+		require.Contains(ui.ErrorWriter.String(), "more than once")
 	}
 
 	// Replace it
