@@ -78,8 +78,6 @@ func TestSnapshotSaveCommand(t *testing.T) {
 	c := New(ui)
 
 	dir := testutil.TempDir(t, "snapshot")
-	defer os.RemoveAll(dir)
-
 	file := filepath.Join(dir, "backup.tgz")
 	args := []string{
 		"-http-addr=" + a.HTTPAddr(),
@@ -91,6 +89,10 @@ func TestSnapshotSaveCommand(t *testing.T) {
 		t.Fatalf("bad: %d. %#v", code, ui.ErrorWriter.String())
 	}
 
+	fi, err := os.Stat(file)
+	require.NoError(t, err)
+	require.Equal(t, fi.Mode(), os.FileMode(0600))
+
 	f, err := os.Open(file)
 	if err != nil {
 		t.Fatalf("err: %v", err)
@@ -160,7 +162,6 @@ func TestSnapshotSaveCommand_TruncatedStream(t *testing.T) {
 	})
 
 	dir := testutil.TempDir(t, "snapshot")
-	defer os.RemoveAll(dir)
 
 	for _, removeBytes := range []int{200, 16, 8, 4, 2, 1} {
 		t.Run(fmt.Sprintf("truncate %d bytes from end", removeBytes), func(t *testing.T) {
