@@ -10,11 +10,13 @@ export default function ConsulEnterpriseComparison() {
         imageUrl: require('./img/consul-oss.svg?url'),
         description:
           'Consul Open Source enables individuals to discover services and securely manage connections between them across cloud, on-prem, and hybrid environments.',
-        link: {
-          text: 'View Open Source Features',
-          url: 'https://www.hashicorp.com/products/consul/pricing/',
-          type: 'outbound',
-        },
+        links: [
+          {
+            text: 'View Open Source Features',
+            url: 'https://www.hashicorp.com/products/consul/pricing/',
+            type: 'outbound',
+          },
+        ],
       }}
       itemTwo={{
         title: 'Organizational Complexity',
@@ -22,11 +24,19 @@ export default function ConsulEnterpriseComparison() {
         imageUrl: require('./img/consul-enterprise.svg?url'),
         description:
           'Consul Enterprise provides the foundation for organizations to build an enterprise-ready service networking environment for multiple teams by enabling governance capabilities.',
-        link: {
-          text: 'View Enterprise Features',
-          url: 'https://www.hashicorp.com/products/consul/pricing/',
-          type: 'outbound',
-        },
+        links: [
+          {
+            text: 'View Cloud Features',
+            url:
+              'https://cloud.hashicorp.com/?utm_source=consul_io&utm_content=ent_comparison',
+            type: 'outbound',
+          },
+          {
+            text: 'View Self-Managed Features',
+            url: 'https://www.hashicorp.com/products/consul/pricing/',
+            type: 'outbound',
+          },
+        ],
       }}
       brand="consul"
     />
